@@ -7,10 +7,11 @@
 from ginga import trcalc
 from ginga.util import io_rgb
 from ginga.misc import Bunch
-from ginga.BaseImage import BaseImage, ImageError, Header
+from ginga.BaseImage import BaseImage, Header
 
 import numpy
 
+
 class RGBImage(BaseImage):
 
     def __init__(self, data_np=None, metadata=None,
@@ -18,53 +19,11 @@ class RGBImage(BaseImage):
                  ioclass=io_rgb.RGBFileHandler):
 
         BaseImage.__init__(self, data_np=data_np, metadata=metadata,
-                           logger=logger, name=name)
+                           logger=logger, order=order, name=name)
 
         self.io = ioclass(self.logger)
-        self._calc_order(order)
         self.hasAlpha = 'A' in self.order
 
-    def get_slice(self, ch):
-        data = self._get_data()
-        return data[..., self.order.index(ch.upper())]
-
-    def has_slice(self, ch):
-        return ch.upper() in self.order
-
-    def get_order(self):
-        return self.order
-
-    def get_order_indexes(self, cs):
-        cs = cs.upper()
-        return [ self.order.index(c) for c in cs ]
-
-    def get_array(self, order):
-        order = order.upper()
-        if order == self.order:
-            return self._get_data()
-        l = [ self.get_slice(c) for c in order ]
-        return numpy.dstack(l)
-
-    def _calc_order(self, order):
-        if order is not None:
-            self.order = order.upper()
-        else:
-            # TODO; need something better here than a guess!
-            depth = self.get_depth()
-            if depth == 1:
-                self.order = 'M'
-            elif depth == 2:
-                self.order = 'AM'
-            elif depth == 3:
-                self.order = 'RGB'
-            elif depth == 4:
-                self.order = 'RGBA'
-
-    def set_data(self, data_np, order=None, **kwdargs):
-        super(RGBImage, self).set_data(data_np, **kwdargs)
-
-        self._calc_order(order)
-
     def set_color(self, r, g, b):
         # TODO: handle other sizes
         ch_max = 255
@@ -77,7 +36,7 @@ class RGBImage(BaseImage):
 
     def load_file(self, filepath):
         kwds = Header()
-        metadata = { 'header': kwds, 'path': filepath }
+        metadata = {'header': kwds, 'path': filepath}
 
         # TODO: ideally we would be informed by channel order
         # in result by io_rgb
@@ -112,17 +71,17 @@ class RGBImage(BaseImage):
     def insert_alpha(self, pos, alpha):
         if not self.has_alpha():
             order = list(self.order)
-            l = [ self.get_slice(c) for c in order ]
+            l = [self.get_slice(c) for c in order]
             wd, ht = self.get_size()
             a = numpy.zeros((ht, wd), dtype=numpy.uint8)
-            a[:] = int(alpha)
+            a.fill(alpha)
             l.insert(pos, a)
             self._data = numpy.dstack(l)
             order.insert(pos, 'A')
             self.order = ''.join(order)
 
     def get_scaled_cutout_wdht(self, x1, y1, x2, y2, new_wd, new_ht,
-                                  method='basic'):
+                               method='basic'):
         newdata, (scale_x, scale_y) = trcalc.get_scaled_cutout_wdht(
             self._get_data(), x1, y1, x2, y2, new_wd, new_ht,
             interpolation=method, logger=self.logger)
