@@ -8,7 +8,6 @@ import numpy
 from io import BytesIO
 
 import aggdraw as agg
-from . import AggHelp
 
 from ginga import ImageView
 from ginga.aggw.CanvasRenderAgg import CanvasRenderer
@@ -23,6 +22,7 @@ except ImportError:
 class ImageViewAggError(ImageView.ImageViewError):
     pass
 
+
 class ImageViewAgg(ImageView.ImageViewBase):
 
     def __init__(self, logger=None, rgbmap=None, settings=None):
@@ -31,7 +31,7 @@ class ImageViewAgg(ImageView.ImageViewBase):
                                          settings=settings)
 
         self.surface = None
-        self._rgb_order = 'RGBA'
+        self.rgb_order = 'RGBA'
 
         self.renderer = CanvasRenderer(self)
 
@@ -48,15 +48,15 @@ class ImageViewAgg(ImageView.ImageViewBase):
         self.logger.debug("redraw surface")
 
         # get window contents as a buffer and load it into the AGG surface
-        rgb_buf = self.getwin_buffer(order=self._rgb_order)
-        canvas.fromstring(rgb_buf)
+        rgb_buf = self.getwin_buffer(order=self.rgb_order)
+        canvas.frombytes(rgb_buf)
 
         # for debugging
         #self.save_rgb_image_as_file('/tmp/temp.png', format='png')
 
     def configure_surface(self, width, height):
         # create agg surface the size of the window
-        self.surface = agg.Draw("RGBA", (width, height), 'black')
+        self.surface = agg.Draw(self.rgb_order, (width, height), 'black')
 
         # inform the base class about the actual window size
         self.configure(width, height)
@@ -71,8 +71,8 @@ class ImageViewAgg(ImageView.ImageViewBase):
 
         # Get agg surface as a numpy array
         surface = self.get_surface()
-        arr8 = numpy.fromstring(surface.tostring(), dtype=numpy.uint8)
-        arr8 = arr8.reshape((ht, wd, 4))
+        arr8 = numpy.fromstring(surface.tobytes(), dtype=numpy.uint8)
+        arr8 = arr8.reshape((ht, wd, len(self.rgb_order)))
         return arr8
 
     def get_image_as_buffer(self, output=None):
@@ -84,7 +84,7 @@ class ImageViewAgg(ImageView.ImageViewBase):
             obuf = BytesIO()
 
         surface = self.get_surface()
-        obuf.write(surface.tostring())
+        obuf.write(surface.tobytes())
         return obuf
 
     def get_rgb_image_as_buffer(self, output=None, format='png', quality=90):
@@ -105,13 +105,13 @@ class ImageViewAgg(ImageView.ImageViewBase):
         image = PILimage.fromarray(arr8)
 
         image.save(obuf, format=format, quality=quality)
-        if not (output is None):
+        if output is not None:
             return None
-        return obuf.getvalue()
+        return obuf
 
     def get_rgb_image_as_bytes(self, format='png', quality=90):
         buf = self.get_rgb_image_as_buffer(format=format, quality=quality)
-        return buf
+        return buf.getvalue()
 
     def save_rgb_image_as_file(self, filepath, format='png', quality=90):
         if not have_PIL:
@@ -121,7 +121,7 @@ class ImageViewAgg(ImageView.ImageViewBase):
 
         with open(filepath, 'w') as out_f:
             self.get_rgb_image_as_buffer(output=out_f, format=format,
-                                          quality=quality)
+                                         quality=quality)
         self.logger.debug("wrote %s file '%s'" % (format, filepath))
 
     def update_image(self):
@@ -140,8 +140,32 @@ class ImageViewAgg(ImageView.ImageViewBase):
         # time_sec
         self.delayed_redraw()
 
-    def get_rgb_order(self):
-        return self._rgb_order
 
+class CanvasView(ImageViewAgg):
+    """This class is defined to provide a non-event handling invisible
+    viewer.
+    """
 
-#END
+    def __init__(self, logger=None, settings=None, rgbmap=None,
+                 bindmap=None, bindings=None):
+        # NOTE: bindmap, bindings are ignored
+        ImageViewAgg.__init__(self, logger=logger, settings=settings,
+                              rgbmap=rgbmap)
+        self.defer_redraw = False
+
+        # Needed for UIMixin to propagate events correctly
+        self.objects = [self.private_canvas]
+
+    def set_canvas(self, canvas, private_canvas=None):
+        super(CanvasView, self).set_canvas(canvas,
+                                           private_canvas=private_canvas)
+
+        self.objects[0] = self.private_canvas
+
+    def update_image(self):
+        pass
+
+    def configure_window(self, width, height):
+        return super(CanvasView, self).configure_surface(width, height)
+
+# END
