@@ -8,8 +8,10 @@ import math
 import cairo
 
 from ginga import colors
+from ginga.fonts import font_asst
 # force registration of all canvas types
-import ginga.canvas.types.all
+import ginga.canvas.types.all  # noqa
+
 
 class RenderContext(object):
 
@@ -54,7 +56,7 @@ class RenderContext(object):
 
         if hasattr(shape, 'linestyle'):
             if shape.linestyle == 'dash':
-                self.cr.set_dash([ 3.0, 4.0, 6.0, 4.0], 5.0)
+                self.cr.set_dash([3.0, 4.0, 6.0, 4.0], 5.0)
 
     def set_fill_from_shape(self, shape):
         self.fill = getattr(shape, 'fill', False)
@@ -73,16 +75,17 @@ class RenderContext(object):
                 fontsize = shape.fontsize
             else:
                 fontsize = shape.scale_font(self.viewer)
-            self.cr.select_font_face(shape.font)
+            fontname = font_asst.resolve_alias(shape.font, shape.font)
+            self.cr.select_font_face(fontname)
             self.cr.set_font_size(fontsize)
 
     def initialize_from_shape(self, shape, line=True, fill=True, font=True):
+        if font:
+            self.set_font_from_shape(shape)
         if line:
             self.set_line_from_shape(shape)
         if fill:
             self.set_fill_from_shape(shape)
-        if font:
-            self.set_font_from_shape(shape)
 
     def set_line(self, color, alpha=1.0, linewidth=1, style='solid'):
 
@@ -90,7 +93,7 @@ class RenderContext(object):
         self.cr.set_line_width(linewidth)
 
         if style == 'dash':
-            self.cr.set_dash([ 3.0, 4.0, 6.0, 4.0], 5.0)
+            self.cr.set_dash([3.0, 4.0, 6.0, 4.0], 5.0)
 
     def set_fill(self, color, alpha=1.0):
         if color is None:
@@ -101,6 +104,7 @@ class RenderContext(object):
             self.fill_alpha = alpha
 
     def set_font(self, fontname, fontsize, color='black', alpha=1.0):
+        fontname = font_asst.resolve_alias(fontname, fontname)
         self.cr.select_font_face(fontname)
         self.cr.set_font_size(fontsize)
         self._set_color(color, alpha=alpha)
@@ -134,7 +138,7 @@ class RenderContext(object):
         self.cr.new_path()
 
     def draw_circle(self, cx, cy, cradius):
-        self.cr.arc(cx, cy, cradius, 0, 2*math.pi)
+        self.cr.arc(cx, cy, cradius, 0, 2 * math.pi)
         self.cr.stroke_preserve()
 
         self._draw_fill()
@@ -192,4 +196,4 @@ class CanvasRenderer(object):
         return cr.text_extents(shape.text)
 
 
-#END
+# END
