@@ -5,11 +5,11 @@
 # Please see the file LICENSE.txt for details.
 #
 import time
-import math
 
 from ginga import trcalc
 from ginga.misc.Bunch import Bunch
-from ginga.Bindings import KeyEvent
+from ginga.util.six.moves import filter
+
 from .CanvasMixin import CanvasMixin
 
 __all__ = ['DrawingMixin']
@@ -27,11 +27,11 @@ class DrawingMixin(object):
         from .CanvasObject import drawCatalog
         # For interactive drawing
         self.candraw = False
-        self.draw_dict = drawCatalog
+        self.dc = drawCatalog
         # canvas objects which we know how to draw have an "idraw"
         # class method
-        self.drawtypes = [ key for key in self.draw_dict.keys()
-                           if hasattr(self.draw_dict[key], 'idraw') ]
+        self.drawtypes = [key for key in self.dc.keys()
+                          if hasattr(self.dc[key], 'idraw')]
         self.drawtypes.sort()
         self.t_drawtype = 'point'
         self.t_drawparams = {}
@@ -48,7 +48,8 @@ class DrawingMixin(object):
         self._edit_obj = None
         self._edit_status = False
         self._edit_detail = {}
-        self._pick_cur_obj = None
+        self._pick_cur_objs = set([])
+        self._pick_sel_objs = set([])
 
         # For modes
         self._mode = 'draw'
@@ -64,7 +65,7 @@ class DrawingMixin(object):
                            poly_delete=self.edit_poly_delete)
         self.add_draw_mode('pick', down=self.pick_start,
                            move=self.pick_motion, up=self.pick_stop,
-                           hover=self.pick_hover,
+                           hover=self.pick_hover, key=self.pick_key,
                            poly_add=self.edit_poly_add,
                            poly_delete=self.edit_poly_delete)
 
@@ -86,9 +87,13 @@ class DrawingMixin(object):
                      'cursor-down', 'cursor-up', 'cursor-move',
                      'draw-scroll', 'keydown-poly_add', 'keydown-poly_del',
                      'keydown-edit_del', 'edit-event',
-                     'edit-select', 'drag-drop'):
+                     'edit-select', 'drag-drop', 'cursor-changed'):
             self.enable_callback(name)
 
+        for name in ['key-down', 'key-up', 'btn-down', 'btn-move', 'btn-up',
+                     'scroll', 'pinch', 'pan']:
+            self.enable_callback('%s-none' % (name))
+
     def set_surface(self, viewer):
         self.viewer = viewer
 
@@ -101,11 +106,11 @@ class DrawingMixin(object):
         canvas.add_callback('draw-move', self.draw_motion, viewer)
         canvas.add_callback('draw-up', self.draw_stop, viewer)
 
-        canvas.add_callback('key-press', self._draw_key, 'key', viewer)
+        canvas.add_callback('key-down-none', self._draw_op, 'key', viewer)
         canvas.add_callback('keydown-poly_add', self._draw_op, 'poly_add',
-                          viewer)
+                            viewer)
         canvas.add_callback('keydown-poly_del', self._draw_op, 'poly_delete',
-                          viewer)
+                            viewer)
         canvas.add_callback('keydown-edit_del', self.edit_delete_cb, viewer)
         #canvas.add_callback('draw-scroll', self._edit_rotate_cb, viewer)
         #canvas.add_callback('draw-scroll', self._edit_scale_cb, viewer)
@@ -129,7 +134,7 @@ class DrawingMixin(object):
         return bnch
 
     def set_draw_mode(self, mode):
-        if not mode in self._mode_tbl:
+        if mode not in self._mode_tbl:
             modes = list(self._mode_tbl.keys())
             raise ValueError("mode must be one of: %s" % (str(modes)))
 
@@ -160,17 +165,6 @@ class DrawingMixin(object):
             return method(canvas, event, data_x, data_y, viewer)
         return False
 
-    def _draw_key(self, canvas, keyname, opn, viewer):
-        # synthesize a KeyEvent
-        # TODO: this is hacky--see if we can rethink how this is handled
-        #  so that we get passed an event similar to _draw_op()
-        last_x, last_y = viewer.get_last_data_xy()
-        event = KeyEvent(key=keyname, state='down', mode=self._mode,
-                         modifiers=[], viewer=viewer,
-                         data_x=last_x, data_y=last_y)
-
-        return self._draw_op(canvas, event, last_x, last_y, opn, viewer)
-
     ##### DRAWING LOGIC #####
 
     def _draw_update(self, data_x, data_y, cxt, force_update=False):
@@ -178,7 +172,7 @@ class DrawingMixin(object):
         obj = None
 
         # update the context with current position
-        x, y = cxt.crdmap.data_to(data_x, data_y)
+        x, y = cxt.crdmap.data_to((data_x, data_y))
         cxt.setvals(x=x, y=y, data_x=data_x, data_y=data_y)
 
         draw_class = cxt.draw_class
@@ -201,12 +195,14 @@ class DrawingMixin(object):
             return False
 
         self._draw_obj = None
+        self.clear_selected()
+
         # get the drawing coordinate type (default 'data')
         crdtype = self.t_drawparams.get('coord', 'data')
         crdmap = viewer.get_coordmap(crdtype)
-        x, y = crdmap.data_to(data_x, data_y)
+        x, y = crdmap.data_to((data_x, data_y))
 
-        klass = self.draw_dict.get(self.t_drawtype, None)
+        klass = self.dc.get(self.t_drawtype, None)
 
         # create the drawing context
         self._draw_cxt = Bunch(start_x=x, start_y=y, points=[(x, y)],
@@ -231,7 +227,6 @@ class DrawingMixin(object):
 
             if self.edit_follows_draw:
                 #self.set_draw_mode('edit')
-                self.clear_selected()
                 self.edit_select(obj)
                 self.make_callback('edit-select', self._edit_obj)
             return True
@@ -251,10 +246,10 @@ class DrawingMixin(object):
 
         cxt = self._draw_cxt
         if self.t_drawtype in ('polygon', 'freepolygon', 'path', 'freepath'):
-            x, y = cxt.crdmap.data_to(data_x, data_y)
+            x, y = cxt.crdmap.data_to((data_x, data_y))
             cxt.points.append((x, y))
         elif self.t_drawtype == 'beziercurve' and len(cxt.points) < 3:
-            x, y = cxt.crdmap.data_to(data_x, data_y)
+            x, y = cxt.crdmap.data_to((data_x, data_y))
             cxt.points.append((x, y))
 
         self._draw_update(data_x, data_y, cxt, force_update=True)
@@ -286,8 +281,8 @@ class DrawingMixin(object):
         if drawtype is not None:
             drawtype = drawtype.lower()
             assert drawtype in self.drawtypes, \
-                   ValueError("Bad drawing type '%s': must be one of %s" % (
-                drawtype, self.drawtypes))
+                ValueError("Bad drawing type '%s': must be one of %s" % (
+                    drawtype, self.drawtypes))
         self.t_drawtype = drawtype
         self.t_drawparams = drawparams.copy()
 
@@ -299,9 +294,12 @@ class DrawingMixin(object):
 
     def get_draw_class(self, drawtype):
         drawtype = drawtype.lower()
-        klass = self.draw_dict[drawtype]
+        klass = self.dc[drawtype]
         return klass
 
+    def get_draw_classes(self):
+        return self.dc
+
     def get_drawparams(self):
         return self.t_drawparams.copy()
 
@@ -312,12 +310,11 @@ class DrawingMixin(object):
 
     def register_canvas_type(self, name, klass):
         drawtype = name.lower()
-        self.draw_dict[drawtype] = klass
-        if not drawtype in self.drawtypes:
+        self.dc[drawtype] = klass
+        if drawtype not in self.drawtypes:
             self.drawtypes.append(drawtype)
             self.drawtypes.sort()
 
-
     ##### EDITING LOGIC #####
 
     def get_edit_object(self):
@@ -362,7 +359,7 @@ class DrawingMixin(object):
             self.process_drawing()
         return True
 
-    def _is_editable(self, obj, x, y, is_inside):
+    def _is_editable(self, obj, pt, is_inside):
         return is_inside and obj.editable
 
     def _prepare_to_move(self, obj, data_x, data_y):
@@ -391,7 +388,7 @@ class DrawingMixin(object):
 
             # check for objects at this location
             #print("getting items")
-            objs = canvas.select_items_at(viewer, data_x, data_y,
+            objs = canvas.select_items_at(viewer, (data_x, data_y),
                                           test=self._is_editable)
             #print("items: %s" % (str(objs)))
 
@@ -413,10 +410,10 @@ class DrawingMixin(object):
                 #print("editing: checking for cp")
                 edit_pts = obj.get_edit_points(viewer)
                 #print((self._edit_obj, edit_pts))
-                i = obj.get_pt(viewer, edit_pts, data_x, data_y,
-                               obj.cap_radius)
-                #print(('got point', i))
-                if i is not None:
+                idx = obj.get_pt(viewer, edit_pts, (data_x, data_y),
+                                 obj.cap_radius)
+                if len(idx) > 0:
+                    i = idx[0]
                     #print("editing cp #%d" % (i))
                     # editing a control point from an existing object
                     self._edit_obj = obj
@@ -430,7 +427,8 @@ class DrawingMixin(object):
                     self._edit_update(data_x, data_y, viewer)
                     return True
 
-                ## if obj.contains(data_x, data_y):
+                i = None
+                ## if obj.contains_pt((data_x, data_y)):
                 ##     contains.append(obj)
                 # update: check if objects bbox contains this point
                 x1, y1, x2, y2 = obj.get_llur()
@@ -459,7 +457,7 @@ class DrawingMixin(object):
                     self.clear_selected()
 
                 # see now if there is an unselected item at this location
-                objs = canvas.select_items_at(viewer, data_x, data_y,
+                objs = canvas.select_items_at(viewer, (data_x, data_y),
                                               test=self._is_editable)
                 #print("new items: %s" % (str(objs)))
                 if len(objs) > 0:
@@ -489,8 +487,8 @@ class DrawingMixin(object):
             return False
 
         if (self._edit_tmp != self._edit_obj) or (
-            (self._edit_obj is not None) and
-            (self._edit_status != self.is_selected(self._edit_obj))):
+                (self._edit_obj is not None) and
+                (self._edit_status != self.is_selected(self._edit_obj))):
             # <-- editing status has changed
             #print("making edit-select callback")
             self.make_callback('edit-select', self._edit_obj)
@@ -518,8 +516,8 @@ class DrawingMixin(object):
         if not self.canedit:
             return False
         obj = self._edit_obj
-        if (obj is not None) and self.is_selected(obj) and \
-               (obj.kind in ('polygon', 'path')):
+        if ((obj is not None) and self.is_selected(obj) and
+                (obj.kind in ('polygon', 'path'))):
             self.logger.debug("checking points")
             # determine which line we are adding a point to
             points = list(obj.get_data_points())
@@ -527,18 +525,18 @@ class DrawingMixin(object):
                 points = points + [points[0]]
             x0, y0 = points[0]
             insert = None
-            for i in range(1, len(points[1:])+1):
+            for i in range(1, len(points[1:]) + 1):
                 x1, y1 = points[i]
                 self.logger.debug("checking line %d" % (i))
-                if obj.within_line(viewer, data_x, data_y, x0, y0, x1, y1,
-                                   8):
+                if obj.within_line(viewer, (data_x, data_y),
+                                   (x0, y0), (x1, y1), 8):
                     insert = i
                     break
                 x0, y0 = x1, y1
             if insert is not None:
                 self.logger.debug("inserting point")
                 # Point near a line
-                pt = obj.crdmap.data_to(data_x, data_y)
+                pt = obj.crdmap.data_to((data_x, data_y))
                 obj.insert_pt(insert, pt)
                 self.process_drawing()
             else:
@@ -550,8 +548,8 @@ class DrawingMixin(object):
         if not self.canedit:
             return False
         obj = self._edit_obj
-        if (obj is not None) and self.is_selected(obj) and \
-               (obj.kind in ('polygon', 'path')):
+        if ((obj is not None) and self.is_selected(obj) and
+                (obj.kind in ('polygon', 'path'))):
             self.logger.debug("checking points")
             # determine which point we are deleting
             points = list(obj.get_data_points())
@@ -559,7 +557,7 @@ class DrawingMixin(object):
             for i in range(len(points)):
                 x1, y1 = points[i]
                 self.logger.debug("checking vertex %d" % (i))
-                if obj.within_radius(viewer, data_x, data_y, x1, y1,
+                if obj.within_radius(viewer, (data_x, data_y), (x1, y1),
                                      8):
                     delete = i
                     break
@@ -575,7 +573,7 @@ class DrawingMixin(object):
     def edit_rotate(self, delta_deg, viewer):
         if self._edit_obj is None:
             return False
-        self._edit_obj.rotate_by(delta_deg)
+        self._edit_obj.rotate_by_deg([delta_deg])
         self.process_drawing()
         self.make_callback('edit-event', self._edit_obj)
         return True
@@ -653,64 +651,70 @@ class DrawingMixin(object):
     def select_remove(self, obj):
         try:
             self._selected.remove(obj)
-        except:
+        except Exception:
             pass
 
     def select_add(self, obj):
         if obj not in self._selected:
             self._selected.append(obj)
 
-
     ##### PICK LOGIC #####
 
-    def _do_pick(self, canvas, event, data_x, data_y, cb_name, viewer):
+    def _do_pick(self, canvas, event, data_x, data_y, ptype, viewer):
         # check for objects at this location
-        objs = canvas.select_items_at(viewer, data_x, data_y)
+        objs = canvas.select_items_at(viewer, (data_x, data_y))
 
-        if len(objs) == 0:
-            # <-- no objects under cursor
+        picked = set(filter(lambda obj: obj.pickable, objs))
 
-            if self._pick_cur_obj is not None:
-                # leaving an object that we were in--make pick-leave cb
-                obj, self._pick_cur_obj = self._pick_cur_obj, None
-                pt = obj.crdmap.data_to(data_x, data_y)
-                obj.make_callback('pick-leave', canvas, event, pt)
+        newly_out = self._pick_cur_objs - picked
+        newly_in = picked - self._pick_cur_objs
+        self._pick_cur_objs = picked
 
-            return False
+        if ptype not in ('move', 'up'):
+            self._pick_sel_objs = picked
 
-        # pick top object
-        obj = objs[-1]
-        self.logger.debug("%s event in %s obj at x, y = %d, %d" % (
-            cb_name, obj.kind, data_x, data_y))
+        # leaving an object
+        for obj in newly_out:
+            pt = obj.crdmap.data_to((data_x, data_y))
+            obj.make_callback('pick-leave', canvas, event, pt)
 
-        # get coordinates in native form for this object
-        pt = obj.crdmap.data_to(data_x, data_y)
-
-        if self._pick_cur_obj is None:
-            # entering a new object--make pick-enter cb
-            self._pick_cur_obj = obj
+        # entering an object
+        for obj in newly_in:
+            pt = obj.crdmap.data_to((data_x, data_y))
             obj.make_callback('pick-enter', canvas, event, pt)
 
-        # make pick callback
-        obj.make_callback(cb_name, canvas, event, pt)
-        return True
+        # pick down/up
+        res = False
+        for obj in self._pick_sel_objs:
+            cb_name = 'pick-%s' % (ptype)
+            self.logger.debug("%s event in %s obj at x, y = %d, %d" % (
+                cb_name, obj.kind, data_x, data_y))
+
+            pt = obj.crdmap.data_to((data_x, data_y))
+            if obj.make_callback(cb_name, canvas, event, pt):
+                res = True
+
+        return res
 
     def pick_start(self, canvas, event, data_x, data_y, viewer):
         return self._do_pick(canvas, event, data_x, data_y,
-                             'pick-down', viewer)
+                             'down', viewer)
 
     def pick_motion(self, canvas, event, data_x, data_y, viewer):
         return self._do_pick(canvas, event, data_x, data_y,
-                             'pick-move', viewer)
+                             'move', viewer)
 
     def pick_hover(self, canvas, event, data_x, data_y, viewer):
         return self._do_pick(canvas, event, data_x, data_y,
-                             'pick-hover', viewer)
+                             'hover', viewer)
+
+    def pick_key(self, canvas, event, data_x, data_y, viewer):
+        return self._do_pick(canvas, event, data_x, data_y,
+                             'key', viewer)
 
     def pick_stop(self, canvas, event, data_x, data_y, viewer):
         return self._do_pick(canvas, event, data_x, data_y,
-                             'pick-up', viewer)
-
+                             'up', viewer)
 
     # The canvas drawing
 
@@ -726,20 +730,13 @@ class DrawingMixin(object):
         selected = list(self.get_selected())
         if len(selected) > 0:
             for obj in selected:
-                ## if not self.has_object(obj):
-                ##     # <-- the object has been removed from the canvas
-                ##     # but not removed from the selection
-                ##     self.select_remove(obj)
-                ##     continue
-
                 cr = viewer.renderer.setup_cr(obj)
                 obj.draw_edit(cr, viewer)
 
-
     ### NON-PEP8 EQUIVALENTS -- TO BE DEPRECATED ###
 
     setSurface = set_surface
     getDrawClass = get_draw_class
 
 
-#END
+# END
