@@ -5,24 +5,30 @@
 # This is open-source software licensed under a BSD license.
 # Please see the file LICENSE.txt for details.
 #
-import math
-import numpy
+from __future__ import absolute_import, division, print_function
 
+import math
+import numpy as np
+
+from ginga.AstroImage import AstroImage
 from ginga.canvas.CanvasObject import (CanvasObjectBase, _bool, _color,
                                        Point, MovePoint, ScalePoint,
-                                       RotatePoint,
                                        register_canvas_types, get_canvas_type,
-                                       colors_plus_none)
+                                       colors_plus_none, coord_names)
 from ginga.misc.ParamSet import Param
-from ginga.misc.Bunch import Bunch
 from ginga.util import wcs
+from ginga.util.wcs import raDegToString, decDegToString
 
 from .mixins import OnePointMixin, TwoPointMixin, OnePointOneRadiusMixin
 from .layer import CompoundObject
 
+__all__ = ['Ruler', 'Compass', 'Crosshair', 'AnnulusMixin', 'Annulus',
+           'WCSAxes']
+
 
 class Ruler(TwoPointMixin, CanvasObjectBase):
-    """Draws a WCS ruler (like a right triangle) on a DrawingCanvas.
+    """
+    Draws a WCS ruler (like a right triangle) on a DrawingCanvas.
     Parameters are:
     x1, y1: 0-based coordinates of one end of the diagonal in the data space
     x2, y2: 0-based coordinates of the opposite end of the diagonal
@@ -32,9 +38,9 @@ class Ruler(TwoPointMixin, CanvasObjectBase):
     @classmethod
     def get_params_metadata(cls):
         return [
-            ## Param(name='coord', type=str, default='data',
-            ##       valid=['data', 'wcs'],
-            ##       description="Set type of coordinates"),
+            Param(name='coord', type=str, default='data',
+                  valid=coord_names,
+                  description="Set type of coordinates"),
             Param(name='x1', type=float, default=0.0, argpos=0,
                   description="First X coordinate of object"),
             Param(name='y1', type=float, default=0.0, argpos=1,
@@ -72,7 +78,7 @@ class Ruler(TwoPointMixin, CanvasObjectBase):
             Param(name='showcap', type=_bool,
                   default=False, valid=[False, True],
                   description="Show caps for this object"),
-            ]
+        ]
 
     @classmethod
     def idraw(cls, canvas, cxt):
@@ -83,12 +89,12 @@ class Ruler(TwoPointMixin, CanvasObjectBase):
                  showcap=True, showplumb=True, units='arcmin',
                  font='Sans Serif', fontsize=None, **kwdargs):
         self.kind = 'ruler'
+        points = np.asarray([(x1, y1), (x2, y2)], dtype=np.float)
         CanvasObjectBase.__init__(self, color=color, color2=color2,
                                   alpha=alpha, units=units,
                                   showplumb=showplumb,
                                   linewidth=linewidth, showcap=showcap,
-                                  linestyle=linestyle,
-                                  x1=x1, y1=y1, x2=x2, y2=y2,
+                                  linestyle=linestyle, points=points,
                                   font=font, fontsize=fontsize,
                                   **kwdargs)
         TwoPointMixin.__init__(self)
@@ -98,11 +104,9 @@ class Ruler(TwoPointMixin, CanvasObjectBase):
         points = self.get_data_points(points=points)
         return points
 
-    def select_contains(self, viewer, data_x, data_y):
+    def select_contains_pt(self, viewer, pt):
         points = self.get_points()
-        x1, y1 = points[0]
-        x2, y2 = points[1]
-        return self.within_line(viewer, data_x, data_y, x1, y1, x2, y2,
+        return self.within_line(viewer, pt, points[0], points[1],
                                 self.cap_radius)
 
     def get_ruler_distances(self, viewer):
@@ -134,19 +138,19 @@ class Ruler(TwoPointMixin, CanvasObjectBase):
                 else:
                     sep_h = wcs.deltaStarsRaDecDeg(ra_org, dec_org,
                                                    ra_dst, dec_dst)
-                    text_h = str(sep_h)
+                    text_h = ("%.8f" % sep_h)
                     sep_x = wcs.deltaStarsRaDecDeg(ra_org, dec_org,
                                                    ra_heel, dec_heel)
-                    text_x = str(sep_x)
+                    text_x = ("%.8f" % sep_x)
                     sep_y = wcs.deltaStarsRaDecDeg(ra_heel, dec_heel,
-                                                    ra_dst, dec_dst)
-                    text_y = str(sep_y)
+                                                   ra_dst, dec_dst)
+                    text_y = ("%.8f" % sep_y)
             else:
                 dx = abs(x2 - x1)
                 dy = abs(y2 - y1)
-                dh = math.sqrt(dx**2 + dy**2)
-                text_x = str(dx)
-                text_y = str(dy)
+                dh = np.sqrt(dx**2 + dy**2)
+                text_x = ("%.3f" % dx)
+                text_y = ("%.3f" % dy)
                 text_h = ("%.3f" % dh)
 
         except Exception as e:
@@ -157,11 +161,15 @@ class Ruler(TwoPointMixin, CanvasObjectBase):
         return (text_x, text_y, text_h)
 
     def draw(self, viewer):
+        image = viewer.get_image()
+        if image is None:
+            return
+
         points = self.get_points()
         x1, y1 = points[0]
         x2, y2 = points[1]
-        cx1, cy1 = self.canvascoords(viewer, x1, y1)
-        cx2, cy2 = self.canvascoords(viewer, x2, y2)
+        cx1, cy1 = viewer.get_canvas_xy(x1, y1)
+        cx2, cy2 = viewer.get_canvas_xy(x2, y2)
 
         text_x, text_y, text_h = self.get_ruler_distances(viewer)
 
@@ -185,7 +193,7 @@ class Ruler(TwoPointMixin, CanvasObjectBase):
 
         diag_yoffset = 14
         if abs(cy1 - cy2) < 5:
-            show_angle = 0
+            show_angle = 0  # noqa
         elif cy1 < cy2:
             xplumb_yoffset = -4
         else:
@@ -194,7 +202,7 @@ class Ruler(TwoPointMixin, CanvasObjectBase):
 
         if abs(cx1 - cx2) < 5:
             diag_xoffset = -(4 + htwd)
-            show_angle = 0
+            show_angle = 0  # noqa
         elif (cx1 < cx2):
             diag_xoffset = -(4 + htwd)
             yplumb_xoffset = 4
@@ -202,9 +210,11 @@ class Ruler(TwoPointMixin, CanvasObjectBase):
             diag_xoffset = 4
             yplumb_xoffset = -(4 + ytwd)
 
-        xh = min(cx1, cx2); y = cy1 + xplumb_yoffset
+        xh = min(cx1, cx2)
+        y = cy1 + xplumb_yoffset
         xh += (max(cx1, cx2) - xh) // 2
-        yh = min(cy1, cy2); x = cx2 + yplumb_xoffset
+        yh = min(cy1, cy2)
+        x = cx2 + yplumb_xoffset
         yh += (max(cy1, cy2) - yh) // 2
 
         xd = xh + diag_xoffset
@@ -234,7 +244,8 @@ class Ruler(TwoPointMixin, CanvasObjectBase):
 
 
 class Compass(OnePointOneRadiusMixin, CanvasObjectBase):
-    """Draws a WCS compass on a DrawingCanvas.
+    """
+    Draws a WCS compass on a DrawingCanvas.
     Parameters are:
     x, y: 0-based coordinates of the center in the data space
     radius: radius of the compass arms, in data units
@@ -244,14 +255,14 @@ class Compass(OnePointOneRadiusMixin, CanvasObjectBase):
     @classmethod
     def get_params_metadata(cls):
         return [
-            ## Param(name='coord', type=str, default='data',
-            ##       valid=['data'],
-            ##       description="Set type of coordinates"),
+            Param(name='coord', type=str, default='data',
+                  valid=coord_names,
+                  description="Set type of coordinates"),
             Param(name='x', type=float, default=0.0, argpos=0,
                   description="X coordinate of center of object"),
             Param(name='y', type=float, default=0.0, argpos=1,
                   description="Y coordinate of center of object"),
-            Param(name='radius', type=float, default=1.0,  argpos=2,
+            Param(name='radius', type=float, default=1.0, argpos=2,
                   min=0.0,
                   description="Radius of object"),
             Param(name='linewidth', type=int, default=1,
@@ -274,7 +285,7 @@ class Compass(OnePointOneRadiusMixin, CanvasObjectBase):
             Param(name='showcap', type=_bool,
                   default=False, valid=[False, True],
                   description="Show caps for this object"),
-            ]
+        ]
 
     @classmethod
     def idraw(cls, canvas, cxt):
@@ -285,10 +296,11 @@ class Compass(OnePointOneRadiusMixin, CanvasObjectBase):
                  linewidth=1, fontsize=None, font='Sans Serif',
                  alpha=1.0, linestyle='solid', showcap=True, **kwdargs):
         self.kind = 'compass'
+        points = np.asarray([(x, y)], dtype=np.float)
         CanvasObjectBase.__init__(self, color=color, alpha=alpha,
                                   linewidth=linewidth, showcap=showcap,
                                   linestyle=linestyle,
-                                  x=x, y=y, radius=radius,
+                                  points=points, radius=radius,
                                   font=font, fontsize=fontsize,
                                   **kwdargs)
         OnePointOneRadiusMixin.__init__(self)
@@ -305,24 +317,26 @@ class Compass(OnePointOneRadiusMixin, CanvasObjectBase):
 
     def get_edit_points(self, viewer):
         c_pt, n_pt, e_pt = self.get_points()
-        return [ MovePoint(*c_pt), ScalePoint(*n_pt), ScalePoint(*e_pt) ]
+        return [MovePoint(*c_pt), ScalePoint(*n_pt), ScalePoint(*e_pt)]
 
     def set_edit_point(self, i, pt, detail):
         if i == 0:
-            x, y = pt
-            self.move_to(x, y)
+            self.move_to_pt(pt)
         elif i in (1, 2):
             x, y = pt
             self.radius = max(abs(x - self.x), abs(y - self.y))
         else:
             raise ValueError("No point corresponding to index %d" % (i))
 
-    def select_contains(self, viewer, data_x, data_y):
-        xd, yd = self.crdmap.to_data(self.x, self.y)
-        return self.within_radius(viewer, data_x, data_y, xd, yd,
-                                  self.cap_radius)
+    def select_contains_pt(self, viewer, pt):
+        p0 = self.crdmap.to_data((self.x, self.y))
+        return self.within_radius(viewer, pt, p0, self.cap_radius)
 
     def draw(self, viewer):
+        image = viewer.get_image()
+        if image is None:
+            return
+
         cr = viewer.renderer.setup_cr(self)
         cr.set_font_from_shape(self)
 
@@ -374,9 +388,11 @@ class Compass(OnePointOneRadiusMixin, CanvasObjectBase):
             diag_xoffset = 4
             yplumb_xoffset = -(4 + 0)
 
-        xh = min(cx1, cx2); y = cy1 + xplumb_yoffset
+        xh = min(cx1, cx2)
+        y = cy1 + xplumb_yoffset  # noqa
         xh += (max(cx1, cx2) - xh) // 2
-        yh = min(cy1, cy2); x = cx2 + yplumb_xoffset
+        yh = min(cy1, cy2)
+        x = cx2 + yplumb_xoffset  # noqa
         yh += (max(cy1, cy2) - yh) // 2
 
         xd = xh + diag_xoffset
@@ -385,7 +401,8 @@ class Compass(OnePointOneRadiusMixin, CanvasObjectBase):
 
 
 class Crosshair(OnePointMixin, CanvasObjectBase):
-    """Draws a crosshair on a DrawingCanvas.
+    """
+    Draws a crosshair on a DrawingCanvas.
     Parameters are:
     x, y: 0-based coordinates of the center in the data space
     Optional parameters for linesize, color, etc.
@@ -394,9 +411,9 @@ class Crosshair(OnePointMixin, CanvasObjectBase):
     @classmethod
     def get_params_metadata(cls):
         return [
-            ## Param(name='coord', type=str, default='data',
-            ##       valid=['data'],
-            ##       description="Set type of coordinates"),
+            Param(name='coord', type=str, default='data',
+                  valid=coord_names,
+                  description="Set type of coordinates"),
             Param(name='x', type=float, default=0.0, argpos=0,
                   description="X coordinate of center of object"),
             Param(name='y', type=float, default=0.0, argpos=1,
@@ -426,7 +443,7 @@ class Crosshair(OnePointMixin, CanvasObjectBase):
             Param(name='format', type=str, default='xy',
                   valid=['xy', 'value', 'coords'],
                   description="Format for text annotation (default: xy)"),
-            ]
+        ]
 
     @classmethod
     def idraw(cls, canvas, cxt):
@@ -438,17 +455,17 @@ class Crosshair(OnePointMixin, CanvasObjectBase):
                  fontsize=None, font='Sans Serif', format='xy',
                  **kwdargs):
         self.kind = 'crosshair'
+        points = np.asarray([(x, y)], dtype=np.float)
         CanvasObjectBase.__init__(self, color=color, alpha=alpha,
                                   linewidth=linewidth, linestyle=linestyle,
                                   text=text, textcolor=textcolor,
                                   fontsize=fontsize, font=font,
-                                  x=x, y=y, format=format, **kwdargs)
+                                  points=points, format=format, **kwdargs)
         OnePointMixin.__init__(self)
 
-    def select_contains(self, viewer, data_x, data_y):
-        xd, yd = self.crdmap.to_data(self.x, self.y)
-        return self.within_radius(viewer, data_x, data_y, xd, yd,
-                                  self.cap_radius)
+    def select_contains_pt(self, viewer, pt):
+        p0 = self.crdmap.to_data((self.x, self.y))
+        return self.within_radius(viewer, pt, p0, self.cap_radius)
 
     def draw(self, viewer):
         wd, ht = viewer.get_window_size()
@@ -466,6 +483,8 @@ class Crosshair(OnePointMixin, CanvasObjectBase):
 
             else:
                 image = viewer.get_image()
+                if image is None:
+                    return
                 # NOTE: x, y are assumed to be in data coordinates
                 info = image.info_xy(self.x, self.y, viewer.get_settings())
                 if self.format == 'coords':
@@ -487,35 +506,36 @@ class Crosshair(OnePointMixin, CanvasObjectBase):
 
         txtwd, txtht = cr.text_extents(text)
         cr.set_line(self.textcolor, alpha=self.alpha)
-        cr.draw_text(cx+10, cy+4+txtht, text)
+        cr.draw_text(cx + 10, cy + 4 + txtht, text)
 
 
 class AnnulusMixin(object):
 
-    def contains(self, x, y):
+    def contains_pt(self, pt):
         """Containment test."""
         obj1, obj2 = self.objects
-        return obj2.contains(x, y) and numpy.logical_not(obj1.contains(x, y))
+        return obj2.contains_pt(pt) and np.logical_not(obj1.contains_pt(pt))
 
-    def contains_arr(self, x_arr, y_arr):
+    def contains_pts(self, pts):
         """Containment test on arrays."""
         obj1, obj2 = self.objects
-        arg1 = obj2.contains_arr(x_arr, y_arr)
-        arg2 = numpy.logical_not(obj1.contains_arr(x_arr, y_arr))
-        return numpy.logical_and(arg1, arg2)
+        arg1 = obj2.contains_pts(pts)
+        arg2 = np.logical_not(obj1.contains_pts(pts))
+        return np.logical_and(arg1, arg2)
 
     def get_llur(self):
         """Bounded by outer object."""
         obj2 = self.objects[1]
         return obj2.get_llur()
 
-    def select_contains(self, viewer, data_x, data_y):
+    def select_contains_pt(self, viewer, pt):
         obj2 = self.objects[1]
-        return obj2.select_contains(viewer, data_x, data_y)
+        return obj2.select_contains_pt(viewer, pt)
 
 
 class Annulus(AnnulusMixin, OnePointOneRadiusMixin, CompoundObject):
-    """Special compound object to handle annulus shape that
+    """
+    Special compound object to handle annulus shape that
     consists of two objects with the same centroid.
 
     Examples
@@ -528,14 +548,14 @@ class Annulus(AnnulusMixin, OnePointOneRadiusMixin, CompoundObject):
     @classmethod
     def get_params_metadata(cls):
         return [
-            ## Param(name='coord', type=str, default='data',
-            ##       valid=['data', 'wcs'],
-            ##       description="Set type of coordinates"),
+            Param(name='coord', type=str, default='data',
+                  valid=coord_names,
+                  description="Set type of coordinates"),
             Param(name='x', type=float, default=0.0, argpos=0,
                   description="X coordinate of center of object"),
             Param(name='y', type=float, default=0.0, argpos=1,
                   description="Y coordinate of center of object"),
-            Param(name='radius', type=float, default=1.0,  argpos=2,
+            Param(name='radius', type=float, default=1.0, argpos=2,
                   min=0.0,
                   description="Inner radius of annulus"),
             Param(name='width', type=float, default=None,
@@ -556,12 +576,12 @@ class Annulus(AnnulusMixin, OnePointOneRadiusMixin, CompoundObject):
             Param(name='alpha', type=float, default=1.0,
                   min=0.0, max=1.0, widget='spinfloat', incr=0.05,
                   description="Opacity of outline"),
-            ]
+        ]
 
     @classmethod
     def idraw(cls, canvas, cxt):
-        radius = math.sqrt(abs(cxt.start_x - cxt.x)**2 +
-                            abs(cxt.start_y - cxt.y)**2 )
+        radius = np.sqrt(abs(cxt.start_x - cxt.x)**2 +
+                         abs(cxt.start_y - cxt.y)**2)
         return cls(cxt.start_x, cxt.start_y, radius,
                    **cxt.drawparams)
 
@@ -593,8 +613,10 @@ class Annulus(AnnulusMixin, OnePointOneRadiusMixin, CompoundObject):
                      coord=coord)
         obj2.editable = False
 
+        points = np.asarray([(x, y)], dtype=np.float)
+
         CompoundObject.__init__(self, obj1, obj2,
-                                x=x, y=y, radius=radius,
+                                points=points, radius=radius,
                                 width=width, color=color,
                                 linewidth=linewidth, linestyle=linestyle,
                                 alpha=alpha, **kwdargs)
@@ -607,9 +629,9 @@ class Annulus(AnnulusMixin, OnePointOneRadiusMixin, CompoundObject):
     def get_edit_points(self, viewer):
         points = ((self.x, self.y),
                   self.crdmap.offset_pt((self.x, self.y),
-                                        self.radius, 0),
+                                        (self.radius, 0)),
                   self.crdmap.offset_pt((self.x, self.y),
-                                        self.radius + self.width, 0),
+                                        (self.radius + self.width, 0)),
                   )
         points = self.get_data_points(points=points)
         return [MovePoint(*points[0]),
@@ -624,8 +646,7 @@ class Annulus(AnnulusMixin, OnePointOneRadiusMixin, CompoundObject):
     def set_edit_point(self, i, pt, detail):
         if i == 0:
             # move control point
-            x, y = pt
-            self.move_to(x, y)
+            self.move_to_pt(pt)
         else:
             if i == 1:
                 scalef = self.calc_scale_from_pt(pt, detail)
@@ -649,7 +670,7 @@ class Annulus(AnnulusMixin, OnePointOneRadiusMixin, CompoundObject):
         if oradius < self.radius:
             raise ValueError('Outer boundary < inner boundary')
 
-        d = dict(x=self.x, y=self.y, radius=self.radius, color=self.color,
+        d = dict(points=self.points, radius=self.radius, color=self.color,
                  linewidth=self.linewidth, linestyle=self.linestyle,
                  alpha=self.alpha)
 
@@ -660,13 +681,244 @@ class Annulus(AnnulusMixin, OnePointOneRadiusMixin, CompoundObject):
         d['radius'] = oradius
         self.objects[1].__dict__.update(d)
 
-    def move_to(self, xdst, ydst):
-        super(Annulus, self).move_to(xdst, ydst)
+    def move_to_pt(self, dst_pt):
+        super(Annulus, self).move_to_pt(dst_pt)
 
-        self.set_data_points([(xdst, ydst)])
+        self.set_data_points([dst_pt])
+
+
+class WCSAxes(CompoundObject):
+    """
+    Special compound object to draw WCS axes.
+    """
+    @classmethod
+    def get_params_metadata(cls):
+        return [
+            Param(name='linewidth', type=int, default=1,
+                  min=1, max=20, widget='spinbutton', incr=1,
+                  description="Width of outline"),
+            Param(name='linestyle', type=str, default='dash',
+                  valid=['solid', 'dash'],
+                  description="Style of outline (default dash)"),
+            Param(name='color',
+                  valid=colors_plus_none, type=_color, default='yellow',
+                  description="Color of grid and text"),
+            Param(name='alpha', type=float, default=1.0,
+                  min=0.0, max=1.0, widget='spinfloat', incr=0.05,
+                  description="Opacity of grid and text"),
+            Param(name='font', type=str, default='Sans Serif',
+                  description="Font family for text"),
+            Param(name='fontsize', type=int, default=8,
+                  min=8, max=72,
+                  description="Font size of text (default: 8)"),
+        ]
+
+    def __init__(self, color='cyan',
+                 linewidth=1, linestyle='dash', alpha=1.0,
+                 font='Sans Serif', fontsize=8,
+                 **kwdargs):
+
+        # these could become supplied optional parameters, if desired
+        self.show_label = True
+        self.num_ra = 10
+        self.num_dec = 10
+        self._pix_res = 10
+        self.txt_off = 4
+        self.ra_angle = None
+        self.dec_angle = None
+        # for keeping track of changes to image and orientation
+        self._cur_rot = None
+        self._cur_swap = None
+        self._cur_image = None
+
+        CompoundObject.__init__(self,
+                                color=color, alpha=alpha,
+                                linewidth=linewidth, linestyle=linestyle,
+                                font=font, fontsize=fontsize, **kwdargs)
+
+        self.editable = False
+        self.pickable = False
+        self.opaque = True
+        self.kind = 'wcsaxes'
+
+    def _calc_axes(self, viewer, image, rot_deg, swapxy):
+        self._cur_image = image
+        self._cur_rot = rot_deg
+        self._cur_swap = swapxy
+
+        if not isinstance(image, AstroImage) or not image.has_valid_wcs():
+            return []
+
+        # Approximate bounding box in RA/DEC space
+        xmax = image.width - 1
+        ymax = image.height - 1
+        try:
+            radec = image.wcs.datapt_to_system(
+                [[0, 0], [0, ymax], [xmax, 0], [xmax, ymax]],
+                naxispath=image.naxispath)
+        except Exception:
+            return []
+        ra_min, dec_min = radec.ra.min().deg, radec.dec.min().deg
+        ra_max, dec_max = radec.ra.max().deg, radec.dec.max().deg
+        ra_size = ra_max - ra_min
+        dec_size = dec_max - dec_min
+
+        # Calculate positions of RA/DEC lines
+        d_ra = ra_size / (self.num_ra + 1)
+        d_dec = dec_size / (self.num_dec + 1)
+        ra_arr = np.arange(ra_min + d_ra, ra_max - d_ra * 0.5, d_ra)
+        dec_arr = np.arange(dec_min + d_dec, dec_max - d_ra * 0.5, d_dec)
+
+        # RA/DEC step size for each vector
+        min_imsize = min(image.width, image.height)
+        d_ra_step = ra_size * self._pix_res / min_imsize
+        d_dec_step = dec_size * self._pix_res / min_imsize
+
+        # Create Path objects
+        objs = []
+
+        for cur_ra in ra_arr:
+            crds = [[cur_ra, cur_dec] for cur_dec in
+                    np.arange(dec_min, dec_max + d_dec_step, d_dec_step)]
+            lbl = raDegToString(cur_ra)
+            objs += self._get_path(viewer, image, crds, lbl, 1)
+        for cur_dec in dec_arr:
+            crds = [[cur_ra, cur_dec] for cur_ra in
+                    np.arange(ra_min, ra_max + d_ra_step, d_ra_step)]
+            lbl = decDegToString(cur_dec)
+            objs += self._get_path(viewer, image, crds, lbl, 0)
+
+        return objs
+
+    def _get_path(self, viewer, image, crds, lbl, axis):
+        from ginga.canvas.types.basic import Path, Text
+
+        try:
+            pts = image.wcs.wcspt_to_datapt(crds, naxispath=image.naxispath)
+        except Exception:
+            return []
+
+        # Don't draw outside image area
+        mask = ((pts[:, 0] >= 0) & (pts[:, 0] < image.width) &
+                (pts[:, 1] >= 0) & (pts[:, 1] < image.height))
+        pts = pts[mask]
+
+        path_obj = Path(
+            points=pts, coords='data', linewidth=self.linewidth,
+            linestyle=self.linestyle, color=self.color,
+            alpha=self.alpha)
+        # this is necessary because we are not actually adding to a canvas
+        path_obj.crdmap = viewer.get_coordmap('data')
+
+        if self.show_label:
+            # Calculate label orientation
+            x1, y1 = pts[0]
+            x2, y2 = pts[-1]
+            dx = x2 - x1
+            dy = y2 - y1
+            m = dy / dx
+            c = y1 - m * x1
+
+            if abs(m) < 1:  # x axis varying
+                x = min(x1, x2) + abs(dx) * 0.45
+                y = m * x + c + self.txt_off
+            else:  # y axis varying
+                y = min(y1, y2) + abs(dy) * 0.45
+                if np.isfinite(m):
+                    x = (y - c) / m
+                else:
+                    x = min(x1, x2)
+                x += self.txt_off
+
+            if axis == 0:  # DEC
+                user_angle = self.dec_angle
+                default_rot = 0
+            else:  # RA
+                user_angle = self.ra_angle
+                default_rot = 90
+
+            if user_angle is None:
+                try:
+                    rot = math.atan(m) * 180 / math.pi
+                except ValueError:
+                    rot = default_rot
+                rot = self._cur_rot + rot
+            else:
+                rot = user_angle
+
+            if self._cur_swap:
+                # axes are swapped
+                rot -= 90
+
+            text_obj = Text(x, y, text=lbl, font=self.font,
+                            fontsize=self.fontsize, color=self.color,
+                            alpha=self.alpha, rot_deg=rot, coord='data')
+            text_obj.crdaxis = axis
+            # this is necessary because we are not actually adding to a canvas
+            text_obj.crdmap = viewer.get_coordmap('data')
+
+            return [path_obj, text_obj]
+
+        else:
+            return [path_obj]
+
+    def sync_state(self):
+        for obj in self.objects:
+            if obj.kind == 'text':
+                if self.show_label:
+                    obj.alpha = self.alpha
+                    obj.color = self.color
+                    obj.font = self.font
+                    obj.fontsize = self.fontsize
+                    if obj.crdaxis == 0 and self.dec_angle is not None:
+                        obj.rot_deg = self.dec_angle
+                    elif obj.crdaxis == 1 and self.ra_angle is not None:
+                        obj.rot_deg = self.ra_angle
+                else:
+                    obj.alpha = 0  # hide
+
+            else:  # path
+                obj.alpha = self.alpha
+                obj.color = self.color
+                obj.linewidth = self.linewidth
+                obj.linestyle = self.linestyle
+
+    def draw(self, viewer):
+        # see if we need to recalculate our grid
+        image = viewer.get_image()
+        update = False
+        if self._cur_image != image:
+            # new image loaded
+            update = True
+
+        cur_swap = viewer.get_transforms()[2]
+        if cur_swap != self._cur_swap:
+            # axes have been swapped
+            update = True
+
+        cur_rot = viewer.get_rotation()
+        if cur_rot != self._cur_rot and self.show_label:
+            # rotation has changed
+            # TODO: for a rotation or swap axes change, it would be
+            # sufficient to simply calculate the new rotation angles
+            # and update all the text objects in self.objects
+            update = True
+
+        if len(self.objects) == 0:
+            # initial time
+            update = True
+
+        if update:
+            # only expensive recalculation of grid if needed
+            self.ra_angle = None
+            self.dec_angle = None
+            self.objects = self._calc_axes(viewer, image, cur_rot, cur_swap)
+
+        super(WCSAxes, self).draw(viewer)
 
 
 register_canvas_types(dict(ruler=Ruler, compass=Compass,
-                           crosshair=Crosshair, annulus=Annulus))
+                           crosshair=Crosshair, annulus=Annulus,
+                           wcsaxes=WCSAxes))
 
-#END
+# END
