@@ -7,7 +7,7 @@
 import numpy as np
 
 from ginga.canvas.CanvasObject import (CanvasObjectBase, _bool, _color,
-                                       Point, MovePoint, ScalePoint,
+                                       Point, MovePoint,
                                        register_canvas_types,
                                        colors_plus_none, coord_names)
 from ginga.misc.ParamSet import Param
@@ -16,6 +16,7 @@ from ginga import trcalc
 
 from .mixins import OnePointMixin
 
+
 class Image(OnePointMixin, CanvasObjectBase):
     """Draws an image on a ImageViewCanvas.
     Parameters are:
@@ -56,13 +57,13 @@ class Image(OnePointMixin, CanvasObjectBase):
             Param(name='showcap', type=_bool,
                   default=False, valid=[False, True],
                   description="Show caps for this object"),
-            ## Param(name='flipy', type=_bool,
-            ##       default=True, valid=[False, True],
-            ##       description="Flip image in Y direction"),
+            Param(name='flipy', type=_bool,
+                  default=False, valid=[False, True],
+                  description="Flip image in Y direction"),
             Param(name='optimize', type=_bool,
                   default=True, valid=[False, True],
                   description="Optimize rendering for this object"),
-            ]
+        ]
 
     def __init__(self, x, y, image, alpha=1.0, scale_x=1.0, scale_y=1.0,
                  interpolation='basic',
@@ -135,7 +136,6 @@ class Image(OnePointMixin, CanvasObjectBase):
         if self.showcap:
             self.draw_caps(cr, self.cap, cpoints)
 
-
     def draw_image(self, viewer, dstarr, whence=0.0):
         if self.image is None:
             return
@@ -153,8 +153,7 @@ class Image(OnePointMixin, CanvasObjectBase):
             xmax = int(np.ceil(np.max(pts[0])))
             ymax = int(np.ceil(np.max(pts[1])))
 
-            # destination location in data_coords
-            #dst_x, dst_y = self.x, self.y + ht
+            # get destination location in data_coords
             dst_x, dst_y = self.crdmap.to_data((self.x, self.y))
 
             a1, b1, a2, b2 = 0, 0, self.image.width, self.image.height
@@ -163,9 +162,9 @@ class Image(OnePointMixin, CanvasObjectBase):
             # onto the final image--by only cutting out what is necessary
             # this speeds scaling greatly at zoomed in sizes
             ((dst_x, dst_y), (a1, b1), (a2, b2)) = \
-                 trcalc.calc_image_merge_clip((xmin, ymin), (xmax, ymax),
-                                              (dst_x, dst_y),
-                                              (a1, b1), (a2, b2))
+                trcalc.calc_image_merge_clip((xmin, ymin), (xmax, ymax),
+                                             (dst_x, dst_y),
+                                             (a1, b1), (a2, b2))
 
             # is image completely off the screen?
             if (a2 - a1 <= 0) or (b2 - b1 <= 0):
@@ -179,7 +178,6 @@ class Image(OnePointMixin, CanvasObjectBase):
 
             res = self.image.get_scaled_cutout2((a1, b1), (a2, b2),
                                                 (_scale_x, _scale_y),
-                                                #flipy=self.flipy,
                                                 method=self.interpolation)
 
             # don't ask for an alpha channel from overlaid image if it
@@ -193,7 +191,10 @@ class Image(OnePointMixin, CanvasObjectBase):
             ##                                          image_order)
             ## else:
             ##     cache.cutout = res.data
-            cache.cutout = res.data
+            data = res.data
+            if self.flipy:
+                data = np.flipud(data)
+            cache.cutout = data
 
             # calculate our offset from the pan position
             pan_x, pan_y = viewer.get_pan()
@@ -207,8 +208,8 @@ class Image(OnePointMixin, CanvasObjectBase):
             # dst position in the pre-transformed array should be calculated
             # from the center of the array plus offsets
             ht, wd, dp = dstarr.shape
-            cvs_x = int(np.round(wd / 2.0  + off_x))
-            cvs_y = int(np.round(ht / 2.0  + off_y))
+            cvs_x = int(np.round(wd / 2.0 + off_x))
+            cvs_y = int(np.round(ht / 2.0 + off_y))
             cache.cvs_pos = (cvs_x, cvs_y)
 
         # composite the image into the destination array at the
@@ -355,7 +356,7 @@ class NormImage(Image):
                   default=False, valid=[False, True],
                   description="Show caps for this object"),
             ## Param(name='flipy', type=_bool,
-            ##       default=True, valid=[False, True],
+            ##       default=False, valid=[False, True],
             ##       description="Flip image in Y direction"),
             Param(name='optimize', type=_bool,
                   default=True, valid=[False, True],
@@ -364,11 +365,11 @@ class NormImage(Image):
             ##       description="RGB mapper for the image"),
             ## Param(name='autocuts', type=?,
             ##       description="Cuts manager for the image"),
-            ]
+        ]
 
     def __init__(self, x, y, image, alpha=1.0, scale_x=1.0, scale_y=1.0,
-                 interpolation='basic',
-                 linewidth=0, linestyle='solid', color='lightgreen', showcap=False,
+                 interpolation='basic', linewidth=0, linestyle='solid',
+                 color='lightgreen', showcap=False,
                  optimize=True, rgbmap=None, autocuts=None, **kwdargs):
         self.kind = 'normimage'
         super(NormImage, self).__init__(x, y, image=image, alpha=alpha,
@@ -381,7 +382,6 @@ class NormImage(Image):
         self.rgbmap = rgbmap
         self.autocuts = autocuts
 
-
     def draw_image(self, viewer, dstarr, whence=0.0):
         if self.image is None:
             return
@@ -405,9 +405,9 @@ class NormImage(Image):
             # onto the final image--by only cutting out what is necessary
             # this speeds scaling greatly at zoomed in sizes
             ((dst_x, dst_y), (a1, b1), (a2, b2)) = \
-                 trcalc.calc_image_merge_clip((xmin, ymin), (xmax, ymax),
-                                              (dst_x, dst_y),
-                                              (a1, b1), (a2, b2))
+                trcalc.calc_image_merge_clip((xmin, ymin), (xmax, ymax),
+                                             (dst_x, dst_y),
+                                             (a1, b1), (a2, b2))
 
             # is image completely off the screen?
             if (a2 - a1 <= 0) or (b2 - b1 <= 0):
@@ -436,8 +436,8 @@ class NormImage(Image):
             # dst position in the pre-transformed array should be calculated
             # from the center of the array plus offsets
             ht, wd, dp = dstarr.shape
-            cvs_x = int(np.round(wd / 2.0  + off_x))
-            cvs_y = int(np.round(ht / 2.0  + off_y))
+            cvs_x = int(np.round(wd / 2.0 + off_x))
+            cvs_y = int(np.round(ht / 2.0 + off_y))
             cache.cvs_pos = (cvs_x, cvs_y)
 
         if self.rgbmap is not None:
