@@ -1,17 +1,16 @@
 #
 # CanvasRenderCv.py -- for rendering into a ImageViewCv widget
 #
-# Eric Jeschke (eric@naoj.org)
-#
-# Copyright (c) Eric R. Jeschke.  All rights reserved.
 # This is open-source software licensed under a BSD license.
 # Please see the file LICENSE.txt for details.
 
-import cv2
+import cv2  # noqa
 from . import CvHelp
 
 # force registration of all canvas types
-import ginga.canvas.types.all
+import ginga.canvas.types.all  # noqa
+from ginga import trcalc
+
 
 class RenderContext(object):
 
@@ -80,13 +79,13 @@ class RenderContext(object):
     def text_extents(self, text):
         return self.cr.text_extents(text, self.font)
 
-
     ##### DRAWING OPERATIONS #####
 
     def draw_text(self, cx, cy, text, rot_deg=0.0):
         self.cr.text((cx, cy), text, self.font)
 
     def draw_polygon(self, cpoints):
+        cpoints = trcalc.strip_z(cpoints)
         self.cr.polygon(cpoints, self.pen, self.brush)
 
     def draw_circle(self, cx, cy, cradius):
@@ -100,6 +99,7 @@ class RenderContext(object):
         self.cr.line((cx1, cy1), (cx2, cy2), self.pen)
 
     def draw_path(self, cpoints):
+        cpoints = trcalc.strip_z(cpoints)
         self.cr.path(cpoints, self.pen)
 
 
