@@ -6,8 +6,8 @@
 # Please see the file LICENSE.txt for details.
 #
 from __future__ import print_function
-import sys, os
-import logging, logging.handlers
+
+import sys
 
 from ginga import AstroImage
 from ginga.gtkw import GtkHelp
@@ -20,6 +20,7 @@ import gtk
 
 STD_FORMAT = '%(asctime)s | %(levelname)1.1s | %(filename)s:%(lineno)d (%(funcName)s) | %(message)s'
 
+
 class FitsViewer(object):
 
     def __init__(self, logger):
@@ -44,10 +45,10 @@ class FitsViewer(object):
         fi.set_zoom_algorithm('rate')
         fi.set_zoomrate(1.4)
         fi.show_pan_mark(True)
-        fi.set_callback('drag-drop', self.drop_file)
-        fi.set_callback('none-move', self.motion)
+        fi.set_callback('drag-drop', self.drop_file_cb)
+        fi.set_callback('cursor-changed', self.cursor_cb)
         fi.set_bg(0.2, 0.2, 0.2)
-        fi.ui_setActive(True)
+        fi.ui_set_active(True)
         self.fitsimage = fi
 
         bd = fi.get_bindings()
@@ -63,7 +64,7 @@ class FitsViewer(object):
         private_canvas = fi.get_canvas()
         private_canvas.register_for_cursor_drawing(fi)
         private_canvas.add(canvas)
-        canvas.ui_setActive(True)
+        canvas.ui_set_active(True)
         self.drawtypes = canvas.get_drawtypes()
         self.drawtypes.sort()
 
@@ -143,10 +144,10 @@ class FitsViewer(object):
         fill = self.wfill.get_active()
         alpha = self.walpha.get_value()
 
-        params = { 'color': self.drawcolors[index],
-                   'alpha': alpha,
-                   #'cap': 'ball',
-                   }
+        params = {'color': self.drawcolors[index],
+                  'alpha': alpha,
+                  #'cap': 'ball',
+                  }
         if kind in ('circle', 'rectangle', 'polygon', 'triangle',
                     'righttriangle', 'ellipse', 'square', 'box'):
             params['fill'] = fill
@@ -167,18 +168,20 @@ class FitsViewer(object):
     def open_file(self, w):
         self.select.popup("Open FITS file", self.load_file)
 
-    def drop_file(self, fitsimage, paths):
+    def drop_file_cb(self, fitsimage, paths):
         fileName = paths[0]
         self.load_file(fileName)
 
-    def motion(self, fitsimage, button, data_x, data_y):
-
+    def cursor_cb(self, viewer, button, data_x, data_y):
+        """This gets called when the data position relative to the cursor
+        changes.
+        """
         # Get the value under the data coordinates
         try:
-            #value = fitsimage.get_data(data_x, data_y)
             # We report the value across the pixel, even though the coords
             # change halfway across the pixel
-            value = fitsimage.get_data(int(data_x+0.5), int(data_y+0.5))
+            value = viewer.get_data(int(data_x + viewer.data_off),
+                                    int(data_y + viewer.data_off))
 
         except Exception:
             value = None
@@ -188,7 +191,7 @@ class FitsViewer(object):
         # Calculate WCS RA
         try:
             # NOTE: image function operates on DATA space coords
-            image = fitsimage.get_image()
+            image = viewer.get_image()
             if image is None:
                 # No image loaded
                 return
@@ -197,7 +200,7 @@ class FitsViewer(object):
         except Exception as e:
             self.logger.warning("Bad coordinate conversion: %s" % (
                 str(e)))
-            ra_txt  = 'BAD WCS'
+            ra_txt = 'BAD WCS'
             dec_txt = 'BAD WCS'
 
         text = "RA: %s  DEC: %s  X: %.2f  Y: %.2f  Value: %s" % (
@@ -238,6 +241,7 @@ def main(options, args):
 
     gtk.main()
 
+
 if __name__ == "__main__":
 
     # Parse command line options with nifty optparse module
@@ -246,7 +250,8 @@ if __name__ == "__main__":
     usage = "usage: %prog [options] cmd [args]"
     optprs = OptionParser(usage=usage, version=('%%prog'))
 
-    optprs.add_option("--debug", dest="debug", default=False, action="store_true",
+    optprs.add_option("--debug", dest="debug", default=False,
+                      action="store_true",
                       help="Enter the pdb debugger on main()")
     optprs.add_option("--opencv", dest="opencv", default=False,
                       action="store_true",
@@ -274,7 +279,6 @@ if __name__ == "__main__":
         print(("%s profile:" % sys.argv[0]))
         profile.run('main(options, args)')
 
-
     else:
         main(options, args)
 
