@@ -26,12 +26,15 @@ This shows how you can use the functionality with straight python/matplotlib
 sessions.  Run this by supplying a single FITS file on the command line.
 """
 from __future__ import print_function
-import sys, os
+
+import sys
+import os
 import platform
 # just in case you want to use qt
 os.environ['QT_API'] = 'pyqt'
 
 import matplotlib
+MPL_V1 = matplotlib.__version__.startswith('1')
 options = ['Qt4Agg', 'GTK', 'GTKAgg', 'MacOSX', 'GTKCairo', 'WXAgg',
            'TkAgg', 'QtAgg', 'FltkAgg', 'WX']
 # Force a specific toolkit on mac
@@ -44,12 +47,12 @@ if len(macos_ver) > 0:
 
 import matplotlib.pyplot as plt
 import matplotlib.patches as patches
-import numpy as np
 
 from ginga.mplw.ImageViewCanvasMpl import ImageViewCanvas
 from ginga.misc import log
 from ginga.AstroImage import AstroImage
 from ginga import cmap
+
 # add matplotlib colormaps to ginga's own set
 cmap.add_matplotlib_cmaps()
 
@@ -85,16 +88,17 @@ fi.set_image(image)
 
 # Note adding axis from ginga (mpl backend) object
 ax = fi.add_axes()
-ax.hold(True)
+if MPL_V1:
+    # old matplotlib API
+    ax.hold(True)
 
 wd, ht = image.get_size()
 
 # plot a line
-l = ax.plot((wd*0.33, wd*0.75), (ht*0.5, ht*0.75), 'go-',
-            c="g",
+l = ax.plot((wd * 0.33, wd * 0.75), (ht * 0.5, ht * 0.75), 'go-', c="g",
             label='line1')
 # a rect
-r = patches.Rectangle((wd*0.10, ht*0.10), wd*0.6, ht*0.5, ec='b',
+r = patches.Rectangle((wd * 0.10, ht * 0.10), wd * 0.6, ht * 0.5, ec='b',
                       fill=False)
 ax.add_patch(r)
 
