@@ -21,7 +21,9 @@ While in 'capture mode' you can draw points with the right mouse button.
 Press 'c' to clear the canvas of drawn points.
 """
 from __future__ import print_function
-import sys, os
+
+import sys
+import os
 import platform
 # just in case you want to use qt
 os.environ['QT_API'] = 'pyqt'
@@ -37,18 +39,16 @@ if len(macos_ver) > 0:
     # with ginga
     matplotlib.use('Qt4Agg')
 import matplotlib.pyplot as plt
-import matplotlib.patches as patches
-import numpy as np
 
 from ginga.mplw.ImageViewCanvasMpl import ImageViewCanvas
 from ginga.mplw.ImageViewCanvasTypesMpl import DrawingCanvas
 from ginga.AstroImage import AstroImage
 from ginga.misc import log
-from ginga import cmap
 
 # Set to True to get diagnostic logging output
 use_logger = False
 
+
 class MyGingaFigure(object):
     def __init__(self, logger, fig):
         self.logger = logger
