@@ -4,27 +4,14 @@
 # This is open-source software licensed under a BSD license.
 # Please see the file LICENSE.txt for details.
 
-import sys, os
-
-import numpy
+import os
 
 from ginga.gtk3w import GtkHelp
-from ginga import Mixins, Bindings, colors
+from ginga import Mixins, Bindings
 from ginga.util.paths import icondir
-import ginga.util.six as six
 
-if six.PY2:
-    from ginga.cairow.ImageViewCairo import (ImageViewCairo as ImageView,
-                                             ImageViewCairoError as ImageViewError)
-else:
-    # NOTE [1]: this is a workaround for broken pycairo3--
-    # it lacks the ImageSurface.create_for_data() function present in
-    # pycairo2.  Supposedly this will be added.  Until this is fixed
-    # we use a workaround to draw with PIL, so we use a different base
-    # class and just change the draw handler accordingly
-    #
-    from ginga.pilw.ImageViewPil import (ImageViewPil as ImageView,
-                                         ImageViewPilError as ImageViewError)
+from ginga.cairow.ImageViewCairo import (ImageViewCairo as ImageView,
+                                         ImageViewCairoError as ImageViewError)
 
 from gi.repository import Gtk
 from gi.repository import Gdk
@@ -35,6 +22,7 @@ import cairo
 class ImageViewGtkError(ImageViewError):
     pass
 
+
 class ImageViewGtk(ImageView):
 
     def __init__(self, logger=None, rgbmap=None, settings=None):
@@ -99,7 +87,7 @@ class ImageViewGtk(ImageView):
         daht = self.surface.get_height()
         rgb_buf = bytes(self.surface.get_data())
         pixbuf = GtkHelp.pixbuf_new_from_data(rgb_buf, GdkPixbuf.Colorspace.RGB,
-                                              False, 8, dawd, daht, dawd*3)
+                                              False, 8, dawd, daht, dawd * 3)
 
         return pixbuf
 
@@ -128,19 +116,10 @@ class ImageViewGtk(ImageView):
             # to scrolling
             win.process_updates(True)
 
-
     def draw_event(self, widget, cr):
         self.logger.debug("updating window from surface")
-        if six.PY2:
-            # redraw the screen from backing surface
-            cr.set_source_surface(self.surface, 0, 0)
-        else:
-            # see NOTE [1] above
-            arr8 = self.get_image_as_array()
-            pixbuf = GtkHelp.pixbuf_new_from_array(arr8,
-                                                   GdkPixbuf.Colorspace.RGB,
-                                                   8)
-            Gdk.cairo_set_source_pixbuf(cr, pixbuf, 0, 0)
+        # redraw the screen from backing surface
+        cr.set_source_surface(self.surface, 0, 0)
 
         cr.set_operator(cairo.OPERATOR_SOURCE)
         cr.paint()
@@ -163,10 +142,8 @@ class ImageViewGtk(ImageView):
             if (wwd == width) and (wht == height):
                 return True
 
-        #self.surface = None
         self.logger.debug("allocation is %d,%d %dx%d" % (
             x, y, width, height))
-        #width, height = width*2, height*2
         self.configure_window(width, height)
         return True
 
@@ -237,18 +214,18 @@ class ImageViewEvent(ImageViewGtk):
         imgwin.connect("key_release_event", self.key_release_event)
         imgwin.connect("scroll_event", self.scroll_event)
         mask = imgwin.get_events()
-        imgwin.set_events(mask
-                          | Gdk.EventMask.ENTER_NOTIFY_MASK
-                          | Gdk.EventMask.LEAVE_NOTIFY_MASK
-                          | Gdk.EventMask.FOCUS_CHANGE_MASK
-                          | Gdk.EventMask.STRUCTURE_MASK
-                          | Gdk.EventMask.BUTTON_PRESS_MASK
-                          | Gdk.EventMask.BUTTON_RELEASE_MASK
-                          | Gdk.EventMask.KEY_PRESS_MASK
-                          | Gdk.EventMask.KEY_RELEASE_MASK
-                          | Gdk.EventMask.POINTER_MOTION_MASK
-                          | Gdk.EventMask.POINTER_MOTION_HINT_MASK
-                          | Gdk.EventMask.SCROLL_MASK)
+        imgwin.set_events(mask |
+                          Gdk.EventMask.ENTER_NOTIFY_MASK |
+                          Gdk.EventMask.LEAVE_NOTIFY_MASK |
+                          Gdk.EventMask.FOCUS_CHANGE_MASK |
+                          Gdk.EventMask.STRUCTURE_MASK |
+                          Gdk.EventMask.BUTTON_PRESS_MASK |
+                          Gdk.EventMask.BUTTON_RELEASE_MASK |
+                          Gdk.EventMask.KEY_PRESS_MASK |
+                          Gdk.EventMask.KEY_RELEASE_MASK |
+                          Gdk.EventMask.POINTER_MOTION_MASK |
+                          Gdk.EventMask.POINTER_MOTION_HINT_MASK |
+                          Gdk.EventMask.SCROLL_MASK)
 
         # Set up widget as a drag and drop destination
         imgwin.connect("drag-data-received", self.drop_event_cb)
@@ -331,7 +308,7 @@ class ImageViewEvent(ImageViewGtk):
             'end': 'end',
             'page_up': 'page_up',
             'page_down': 'page_down',
-            }
+        }
 
         # Define cursors
         for curname, filename in (('pan', 'openHandCursor.png'),
@@ -397,7 +374,8 @@ class ImageViewEvent(ImageViewGtk):
 
     def button_press_event(self, widget, event):
         # event.button, event.x, event.y
-        x = event.x; y = event.y
+        x = event.x
+        y = event.y
         self.last_win_x, self.last_win_y = x, y
 
         button = 0
@@ -411,7 +389,8 @@ class ImageViewEvent(ImageViewGtk):
 
     def button_release_event(self, widget, event):
         # event.button, event.x, event.y
-        x = event.x; y = event.y
+        x = event.x
+        y = event.y
         self.last_win_x, self.last_win_y = x, y
 
         button = 0
@@ -446,7 +425,8 @@ class ImageViewEvent(ImageViewGtk):
 
     def scroll_event(self, widget, event):
         # event.button, event.x, event.y
-        x = event.x; y = event.y
+        x = event.x
+        y = event.y
         self.last_win_x, self.last_win_y = x, y
 
         # NOTE: for future use in distinguishing mouse wheel vs.
@@ -465,18 +445,18 @@ class ImageViewEvent(ImageViewGtk):
         data_x, data_y = self.check_cursor_location()
 
         return self.make_ui_callback('scroll', direction, degrees,
-                                  data_x, data_y)
+                                     data_x, data_y)
 
     def drag_drop_cb(self, widget, context, x, y, time):
         self.logger.debug('drag_drop_cb')
         # initiates a drop
-        success = delete = False
+        success = delete = False  # noqa
         targets = context.list_targets()
         for mimetype in targets:
             if str(mimetype) in ("text/thumb", "text/plain", "text/uri-list"):
                 Gdk.drop_reply(context, True, time)
                 success = True
-                return True
+                return success
 
         self.logger.debug("dropped format type did not match known types")
         Gdk.drop_reply(context, False, time)
@@ -676,6 +656,9 @@ class ScrolledView(Gtk.Table):
             bd = self.viewer.get_bindings()
             bd.pan_by_pct(self.viewer, pct_x, pct_y, pad=self.pad)
 
+            # This shouldn't be necessary, but seems to be
+            self.viewer.redraw(whence=0)
+
         finally:
             self._scrolling = False
 
