@@ -6,13 +6,13 @@
 #
 import matplotlib
 matplotlib.use('GTK3Cairo')
-from  matplotlib.backends.backend_gtk3cairo import FigureCanvasGTK3Cairo \
-     as FigureCanvas
+from matplotlib.backends.backend_gtk3cairo import FigureCanvasGTK3Cairo as FigureCanvas  # noqa
 ## matplotlib.use('GTK3Agg')
 ## from  matplotlib.backends.backend_gtk3agg import FigureCanvasGTK3Agg \
 ##      as FigureCanvas
 
-from ginga.gtk3w import Widgets
+from ginga.gtk3w import Widgets  # noqa
+
 
 class PlotWidget(Widgets.WidgetBase):
 
@@ -21,13 +21,19 @@ class PlotWidget(Widgets.WidgetBase):
 
         self.widget = FigureCanvas(plot.get_figure())
         self.plot = plot
+        self.logger = plot.logger
 
         self.widget.set_size_request(width, height)
         self.widget.show_all()
 
+    def set_plot(self, plot):
+        self.plot = plot
+        self.logger = plot.logger
+        self.logger.debug("set_plot called")
+
     def configure_window(self, wd, ht):
         self.logger.debug("canvas resized to %dx%d" % (wd, ht))
         fig = self.plot.get_figure()
         fig.set_size_inches(float(wd) / fig.dpi, float(ht) / fig.dpi)
 
-#END
+# END
