@@ -1,18 +1,15 @@
 #
 # Plot.py -- Plotting function for Gen2 FITS viewer.
 #
-# Eric Jeschke (eric@naoj.org)
-#
-# Copyright (c) Eric R. Jeschke.  All rights reserved.
 # This is open-source software licensed under a BSD license.
 # Please see the file LICENSE.txt for details.
 #
 import matplotlib
 matplotlib.use('GTKCairo')
-from  matplotlib.backends.backend_gtkcairo import FigureCanvasGTKCairo \
-     as FigureCanvas
+from matplotlib.backends.backend_gtkcairo import FigureCanvasGTKCairo as FigureCanvas  # noqa
+
+from ginga.gtkw import Widgets  # noqa
 
-from ginga.gtkw import Widgets
 
 class PlotWidget(Widgets.WidgetBase):
 
@@ -21,13 +18,19 @@ class PlotWidget(Widgets.WidgetBase):
 
         self.widget = FigureCanvas(plot.get_figure())
         self.plot = plot
+        self.logger = plot.logger
 
         self.widget.set_size_request(width, height)
         self.widget.show_all()
 
+    def set_plot(self, plot):
+        self.plot = plot
+        self.logger = plot.logger
+        self.logger.debug("set_plot called")
+
     def configure_window(self, wd, ht):
         self.logger.debug("canvas resized to %dx%d" % (wd, ht))
         fig = self.plot.get_figure()
         fig.set_size_inches(float(wd) / fig.dpi, float(ht) / fig.dpi)
 
-#END
+# END
