@@ -36,6 +36,7 @@ class WidgetError(Exception):
     """For errors thrown in this module."""
     pass
 
+
 # (see TabWidget)
 _widget_move_event = None
 _app = None
@@ -168,7 +169,9 @@ class TextEntry(WidgetBase):
     def set_editable(self, tf):
         self.widget.set_editable(tf)
 
-    def set_font(self, font):
+    def set_font(self, font, size=10):
+        if isinstance(font, six.string_types):
+            font = self.get_font(font, size)
         self.widget.modify_font(font)
 
     def set_length(self, numchars):
@@ -209,7 +212,9 @@ class TextEntrySet(WidgetBase):
     def set_editable(self, tf):
         self.entry.set_editable(tf)
 
-    def set_font(self, font):
+    def set_font(self, font, size=10):
+        if isinstance(font, six.string_types):
+            font = self.get_font(font, size)
         self.widget.modify_font(font)
 
     def set_length(self, numchars):
@@ -292,7 +297,9 @@ class TextArea(WidgetBase):
     def set_editable(self, tf):
         self.tw.set_editable(tf)
 
-    def set_font(self, font):
+    def set_font(self, font, size=10):
+        if isinstance(font, six.string_types):
+            font = self.get_font(font, size)
         self.tw.modify_font(font)
 
     def set_wrap(self, tf):
@@ -361,7 +368,9 @@ class Label(WidgetBase):
     def set_text(self, text):
         self.label.set_text(text)
 
-    def set_font(self, font):
+    def set_font(self, font, size=10):
+        if isinstance(font, six.string_types):
+            font = self.get_font(font, size)
         self.label.modify_font(font)
 
     def set_color(self, fg=None, bg=None):
@@ -416,7 +425,7 @@ class ComboBox(WidgetBase):
             if model[i][0] > text:
                 model.insert(j, tup)
                 return
-        model.insert(j+1, tup)
+        model.insert(j + 1, tup)
 
     def append_text(self, text):
         model = self.widget.get_model()
@@ -490,9 +499,11 @@ class SpinBox(WidgetBase):
 
 
 class Slider(WidgetBase):
-    def __init__(self, orientation='horizontal', track=False):
+    def __init__(self, orientation='horizontal', dtype=int, track=False):
         super(Slider, self).__init__()
 
+        # NOTE: parameter dtype is ignored for now for gtk
+
         if orientation == 'horizontal':
             w = GtkHelp.HScale()
             # TEMP: hack because scales don't seem to expand as expected
@@ -649,6 +660,11 @@ class Image(WidgetBase):
     def _set_image(self, native_image):
         self.image.set_from_pixbuf(native_image.get_pixbuf())
 
+    def load_file(self, img_path, format=None):
+        # format ignored at present
+        pixbuf = GtkHelp.pixbuf_new_from_file(img_path)
+        self.image.set_from_pixbuf(pixbuf)
+
 
 class ProgressBar(WidgetBase):
     def __init__(self):
@@ -683,7 +699,7 @@ class StatusBar(WidgetBase):
     def set_message(self, msg_str, duration=10.0):
         try:
             self.widget.remove_all(self.ctx_id)
-        except:
+        except Exception:
             pass
         self.ctx_id = self.widget.get_context_id('status')
         self.widget.push(self.ctx_id, msg_str)
@@ -854,7 +870,7 @@ class TreeView(WidgetBase):
 
             # recurse for non-leaf interior node
             for key in node:
-                self._add_subtree(level+1, d, model, item, key, node[key])
+                self._add_subtree(level + 1, d, model, item, key, node[key])
 
     def _selection_cb(self, treeview):
         path, column = treeview.get_cursor()
@@ -923,10 +939,13 @@ class TreeView(WidgetBase):
         treeselection = self.tv.get_selection()
         treeselection.unselect_all()
 
-    def select_path(self, path):
+    def select_path(self, path, state=True):
         treeselection = self.tv.get_selection()
         item = self._path_to_item(path)
-        treeselection.select_iter(item)
+        if state:
+            treeselection.select_iter(item)
+        else:
+            treeselection.unselect_iter(item)
 
     def highlight_path(self, path, onoff, font_color='green'):
         item = self._path_to_item(path)  # noqa
@@ -974,16 +993,22 @@ class TreeView(WidgetBase):
             model, iter1, iter2 = args[:3]
             bnch1 = model.get_value(iter1, 0)
             bnch2 = model.get_value(iter2, 0)
-            if isinstance(bnch1, str):
-                if isinstance(bnch2, str):
-                    return bnch1.lower() == bnch2.lower()  # was using cmp
+            if isinstance(bnch1, six.string_types):
+                if isinstance(bnch2, six.string_types):
+                    s1, s2 = bnch1.lower(), bnch2.lower()
+                    if s1 < s2:
+                        return -1
+                    if s1 > s2:
+                        return 1
                 return 0
             val1, val2 = bnch1[idx], bnch2[idx]
-            if isinstance(val1, str):
-                val1 = val1.lower()
-                val2 = val2.lower()
-            res = val1 == val2  # was using nonexistent cmp
-            return res
+            if isinstance(val1, six.string_types):
+                val1, val2 = val1.lower(), val2.lower()
+                if val1 < val2:
+                    return -1
+                if val1 > val2:
+                    return 1
+            return 0
         return fn
 
     def _mkcolfn0(self, idx):
@@ -1056,6 +1081,9 @@ class ContainerBase(WidgetBase):
         super(ContainerBase, self).__init__()
         self.children = []
 
+        for name in ['widget-added', 'widget-removed']:
+            self.enable_callback(name)
+
     def add_ref(self, ref):
         # TODO: should this be a weakref?
         self.children.append(ref)
@@ -1065,16 +1093,17 @@ class ContainerBase(WidgetBase):
         if delete:
             childw.destroy()
 
-    def remove(self, w, delete=False):
-        if w not in self.children:
+    def remove(self, child, delete=False):
+        if child not in self.children:
             raise KeyError("Widget is not a child of this container")
-        self.children.remove(w)
+        self.children.remove(child)
 
-        self._remove(w.get_widget(), delete=delete)
+        self._remove(child.get_widget(), delete=delete)
+        self.make_callback('widget-removed', child)
 
     def remove_all(self, delete=False):
-        for w in list(self.children):
-            self.remove(w, delete=delete)
+        for child in list(self.children):
+            self.remove(child, delete=delete)
 
     def get_children(self):
         return self.children
@@ -1120,6 +1149,7 @@ class Box(ContainerBase):
         expand = (float(stretch) != 0.0)
         self.widget.pack_start(child_w, expand=expand, fill=True)
         self.widget.show_all()
+        self.make_callback('widget-added', child)
 
 
 class VBox(Box):
@@ -1180,8 +1210,10 @@ class TabWidget(ContainerBase):
             nb.connect("create-window", self._tab_detach_cb)
         nb.connect("page-added", self._tab_insert_cb)
         nb.connect("page-removed", self._tab_remove_cb)
-        nb.sconnect("switch-page", self._cb_redirect)
-        # nb.connect("switch-page", self._cb_redirect)
+        # contrary to some other widgets, we want the "tab changed" event
+        # when the index is switched programmatically as well as by user
+        ## nb.sconnect("switch-page", self._cb_redirect)
+        nb.connect("switch-page", self._cb_redirect)
         self.widget = nb
         self.set_tab_position(tabpos)
 
@@ -1225,8 +1257,12 @@ class TabWidget(ContainerBase):
 
     def _tab_remove_cb(self, nbw, nchild_w, page_num):
         global _widget_move_event
-        child = self._native_to_child(nchild_w)
-        _widget_move_event = WidgetMoveEvent(self, child)
+        try:
+            child = self._native_to_child(nchild_w)
+            _widget_move_event = WidgetMoveEvent(self, child)
+        except ValueError:
+            # we were triggered by a removal that is not a move
+            pass
 
     def _cb_redirect(self, nbw, gptr, index):
         child = self.index_to_widget(index)
@@ -1252,6 +1288,7 @@ class TabWidget(ContainerBase):
         self.widget.show_all()
         # attach title to child
         child.extdata.tab_title = title
+        self.make_callback('widget-added', child)
 
     def get_index(self):
         return self.widget.get_current_page()
@@ -1301,10 +1338,14 @@ class MDIWidget(ContainerBase):
         self.widget = sw
         w = GtkHelp.MDIWidget()
         self.mdi_w = w
+        # Monkey patching the internal callbacks so that we can make
+        # the correct callbacks
         w._move_page = w.move_page
         w.move_page = self._window_moved
         w._resize_page = w.resize_page
         w.resize_page = self._window_resized
+        w._set_current_page = w.set_current_page
+        w.set_current_page = self._set_current_page
 
         sw.set_hadjustment(self.mdi_w.get_hadjustment())
         sw.set_vadjustment(self.mdi_w.get_vadjustment())
@@ -1341,16 +1382,13 @@ class MDIWidget(ContainerBase):
             self.mdi_w.move_page(subwin, x, y)
 
         subwin.add_callback('close', self._window_close, child)
+        self.make_callback('widget-added', child)
 
     def _remove(self, childw, delete=False):
         self.mdi_w.remove(childw)
         if delete:
             childw.destroy()
 
-    def _cb_redirect(self, nbw, gptr, index):
-        child = self.index_to_widget(index)
-        self.make_callback('page-switch', child)
-
     def _window_resized(self, subwin, wd, ht):
         self.mdi_w._resize_page(subwin, wd, ht)
 
@@ -1372,6 +1410,13 @@ class MDIWidget(ContainerBase):
     def _window_close(self, subwin, child):
         return self.make_callback('page-close', child)
 
+    def _set_current_page(self, idx):
+        _idx = self.mdi_w.get_current_page()
+        self.mdi_w._set_current_page(idx)
+        if _idx != idx:
+            child = self.index_to_widget(idx)
+            self.make_callback('page-switch', child)
+
     def get_index(self):
         return self.mdi_w.get_current_page()
 
@@ -1465,6 +1510,7 @@ class Splitter(ContainerBase):
             last.pack2(w)
 
         self.widget.show_all()
+        self.make_callback('widget-added', child)
 
     def _get_sizes(self, pane):
         rect = pane.get_allocation()
@@ -1530,15 +1576,16 @@ class GridBox(ContainerBase):
         self.add_ref(child)
         w = child.get_widget()
         if stretch > 0:
-            xoptions = gtk.EXPAND | gtk.FILL
-            yoptions = gtk.EXPAND | gtk.FILL
+            xoptions = gtk.EXPAND | gtk.SHRINK | gtk.FILL
+            yoptions = gtk.EXPAND | gtk.SHRINK | gtk.FILL
         else:
-            xoptions = gtk.FILL
-            yoptions = gtk.FILL
-        self.widget.attach(w, col, col+1, row, row+1,
+            xoptions = gtk.FILL | gtk.SHRINK
+            yoptions = gtk.FILL | gtk.SHRINK
+        self.widget.attach(w, col, col + 1, row, row + 1,
                            xoptions=xoptions, yoptions=yoptions,
                            xpadding=0, ypadding=0)
         self.widget.show_all()
+        self.make_callback('widget-added', child)
 
 
 class Toolbar(ContainerBase):
@@ -1575,6 +1622,8 @@ class Toolbar(ContainerBase):
         self.add_ref(child)
         w = child.get_widget()
         self.widget.append_widget(w, None, None)
+        w.show()
+        self.make_callback('widget-added', child)
 
     def add_menu(self, text, menu=None, mtype='tool'):
         if menu is None:
@@ -1639,6 +1688,7 @@ class Menu(ContainerBase):
         self.widget.append(menuitem_w)
         self.add_ref(child)
         # self.widget.show_all()
+        self.make_callback('widget-added', child)
 
     def add_name(self, name, checkable=False):
         child = MenuAction(text=name, checkable=checkable)
@@ -1681,11 +1731,16 @@ class Menubar(ContainerBase):
         self.widget = gtk.MenuBar()
         self.menus = Bunch.Bunch(caseless=True)
 
-    def add_widget(self, child):
-        menu_w = child.get_widget()
-        self.widget.addMenu(menu_w)
+    def add_widget(self, child, name):
+        if not isinstance(child, Menu):
+            raise ValueError("child widget needs to be a Menu object")
+        item_w = gtk.MenuItem(label=name)
+        item_w.set_submenu(child.get_widget())
         self.add_ref(child)
-        menu_w.show()
+        self.widget.append(item_w)
+        self.menus[name] = child
+        item_w.show()
+        self.make_callback('widget-added', child)
         return child
 
     def add_name(self, name):
@@ -1863,7 +1918,7 @@ class Application(Callback.Callbacks):
             screen = gtk.gdk.screen_get_default()
             self.screen_ht = screen.get_height()
             self.screen_wd = screen.get_width()
-        except:
+        except Exception:
             self.screen_wd = 1600
             self.screen_ht = 1200
         # self.logger.debug("screen dimensions %dx%d" % (
@@ -2111,13 +2166,13 @@ def build_info(captions, orientation='vertical'):
         while col < numcols:
             idx = col * 2
             if idx < len(tup):
-                title, wtype = tup[idx:idx+2]
+                title, wtype = tup[idx:idx + 2]
                 if not title.endswith(':'):
                     name = name_mangle(title)
                 else:
-                    name = name_mangle('lbl_'+title[:-1])
+                    name = name_mangle('lbl_' + title[:-1])
                 w = make_widget(title, wtype)
-                table.attach(w.get_widget(), col, col+1, row, row+1,
+                table.attach(w.get_widget(), col, col + 1, row, row + 1,
                              xoptions=gtk.FILL, yoptions=gtk.FILL,
                              xpadding=1, ypadding=1)
                 wb[name] = w
