@@ -6,11 +6,12 @@
 #
 from __future__ import absolute_import, print_function
 from ..util import six
-from ..util.six.moves import map, zip
+from ..util.six.moves import map
 
 import sys
 import time
-import os
+import threading
+import traceback
 
 if six.PY2:
     import thread
@@ -26,25 +27,26 @@ else:
     _swival = 0.000001
     sys.setswitchinterval(_swival)
 
-import threading
-import traceback
+from . import Callback  # noqa
 
 
 class TaskError(Exception):
     """Exception generated for task errors"""
     pass
 
+
 class TaskTimeout(TaskError):
     """Exception generated when timing out waiting on a task"""
     pass
 
+
 class UserTaskException(Exception):
     pass
 
 
 # ------------ BASIC TASKS ------------
 
-class Task(object):
+class Task(Callback.Callbacks):
     """This class implements a basic Task (command) abstraction.  The
     methods define the interface for starting, cancelling, waiting on a
     task, etc.
@@ -56,7 +58,6 @@ class Task(object):
         initialize() and start() methods.
         """
         self.ev_done = threading.Event()
-        self.callbacks = Queue.Queue()
         self.tag = None
         self.logger = None
         self.threadPool = None
@@ -69,6 +70,7 @@ class Task(object):
 
         super(Task, self).__init__()
 
+        self.enable_callback('resolved')
 
     def initialize(self, taskParent, override=None):
         """This method initializes a task for (re)use.  taskParent is the
@@ -104,7 +106,7 @@ class Task(object):
         if not self.tag:
             try:
                 self.tag = str(taskParent) + '.' + self.tagger.get_tag(self)
-            except:
+            except Exception:
                 # Failed--fall back to internal tagger
                 self.tag = get_tag(taskParent)
 
@@ -130,7 +132,6 @@ class Task(object):
         else:
             raise TaskError("start(): nothing to start for task %s" % self)
 
-
     def init_and_start(self, taskParent, override={}):
         """Convenience method to initialize and start a task.
         """
@@ -139,14 +140,12 @@ class Task(object):
 
         return tag
 
-
     def check_state(self):
         """Abstract method that should check for pause, cancellation, or
         any other sort of preemption event.
         """
         pass
 
-
     def extend_shares(self, varlist):
         shares = set(self.shares)
         for var in varlist:
@@ -155,7 +154,6 @@ class Task(object):
 
         self.shares = shares
 
-
     def stop(self):
         """This method cancels an executing task (if possible).
         Subclass should override this method.
@@ -186,7 +184,7 @@ class Task(object):
         """
         self.ev_done.wait(timeout=timeout)
 
-        if not self.ev_done.isSet():
+        if not self.ev_done.is_set():
             raise TaskTimeout("Task %s timed out." % self)
 
         # --> self.result is set
@@ -201,25 +199,22 @@ class Task(object):
 
         return self.result
 
-
     def step(self):
         """If a task has a way of stepping through an operation.  It can
         implement this method.  Subclass should not call super.step().
         """
-        raise TaskError("Task %s: subclass should override step() method!" % \
+        raise TaskError("Task %s: subclass should override step() method!" %
                         self)
 
-
     def execute(self):
         """This method does the work of a task (if executed by the
         thread pool) and returns when it is finished.  *** Subclass should
         override this method! ***  It should take no arguments, and can
         return anything.
         """
-        raise TaskError("Task %s: subclass should override execute() method!" % \
+        raise TaskError("Task %s: subclass should override execute() method!" %
                         self)
 
-
     def done(self, result, noraise=False):
         """This method is called when a task has finished executing.
         Subclass can override this method if desired, but should call
@@ -228,7 +223,7 @@ class Task(object):
         # [??] Should this be in a critical section?
 
         # Has done() already been called on this task?
-        if self.ev_done.isSet():
+        if self.ev_done.is_set():
             # ??
             if isinstance(self.result, Exception) and (not noraise):
                 raise self.result
@@ -247,7 +242,7 @@ class Task(object):
         self.ev_done.set()
 
         # Perform callbacks for event-style waiters
-        self.do_callbacks()
+        self.make_callback('resolved', self.result)
 
         # If the result is an exception, then our final act is to raise
         # it in the caller, unless the caller explicitly supressed that
@@ -256,35 +251,6 @@ class Task(object):
 
         return result
 
-
-    def register_callback(self, fn, args=None):
-        """This method is called to register a callback function to be
-        called when a task terminates.
-        Subclass should probably not override this method.
-        """
-        if args is None:
-            args = []
-
-        if callable(fn):
-            self.callbacks.put((fn, args))
-        else:
-            raise TaskError("Function argument is not a callable: %s" % \
-                            str(fn))
-
-
-    def do_callbacks(self):
-        """Makes callbacks on all registered functions waiting on this task.
-        """
-
-        while not self.callbacks.empty():
-            (fn, rest) = self.callbacks.get()
-
-            args = [self.result]
-            args.extend(rest)
-
-            fn(*args)
-
-
     def get_tag(self):
         """This is only valid AFTER initialize() has been called on the task.
         """
@@ -302,7 +268,6 @@ class Task(object):
     def getExecutionTime(self):
         return self.totaltime
 
-
     def runTask(self, task, timeout=None):
         """Run a child task to completion.  Returns the result of
         the child task.
@@ -322,7 +287,6 @@ class Task(object):
         # Now we're done
         return res
 
-
     def run(self, task, timeout=None):
         """Run a child task to completion.  Returns the result of
         the child task.  Simply calls runTask().
@@ -341,6 +305,7 @@ class printTask(Task):
     def execute(self):
         print(self.msg)
 
+
 class sleepTask(Task):
     """Simple task that sleeps for delay seconds."""
     def __init__(self, delay):
@@ -350,6 +315,7 @@ class sleepTask(Task):
     def execute(self):
         self.ev_done.wait(timeout=self.delay)
 
+
 class FuncTask(Task):
     """Simple task that calls func and returns func's return value."""
     def __init__(self, func, args, kwdargs, logger=None):
@@ -383,11 +349,11 @@ class FuncTask(Task):
 
         except Exception as e:
             if self.logger:
-                self.logger.error("Task '%s' terminated with exception: %s" % \
+                self.logger.error("Task '%s' terminated with exception: %s" %
                                   (str(self), str(e)))
                 try:
                     (type, value, tb) = sys.exc_info()
-                    self.logger.error("Traceback:\n%s" % \
+                    self.logger.error("Traceback:\n%s" %
                                       "".join(traceback.format_tb(tb)))
 
                     tb = None
@@ -449,13 +415,11 @@ class SequentialTaskset(Task):
 
         self.tasklist = list(taskseq)
 
-
     def initialize(self, taskParent, **kwdargs):
         self.index = 0
 
         super(SequentialTaskset, self).initialize(taskParent, **kwdargs)
 
-
     def step(self):
         """Run the next child task and wait for completion (no timeout)."""
         if self.index >= len(self.tasklist):
@@ -469,20 +433,18 @@ class SequentialTaskset(Task):
 
         return self.runTask(self.task)
 
-
     def execute(self):
         """Run all child tasks, in order, waiting for completion of each.
         Return the result of the final child task's execution.
         """
         while self.index < len(self.tasklist):
             res = self.step()
-            self.logger.debug('SeqSet task %i has completed with result %s' % \
-                             (self.index,res))
+            self.logger.debug('SeqSet task %i has completed with result %s' %
+                              (self.index, res))
 
         # Returns result of last task to quit
         return res
 
-
     def stop(self):
         """Interrupt/cancel execution, but will allow current child task
         to complete."""
@@ -494,7 +456,6 @@ class SequentialTaskset(Task):
         except TaskError as e:
             self.logger.error("Error cancelling child task: %s" % (str(e)))
 
-
     def addTask(self, task):
         """Append a task to the task sequence.  If the SequentialTaskset has
         already completed execution, this will do nothing unless it is
@@ -517,7 +478,6 @@ class oldConcurrentAndTaskset(Task):
         # Used to synchronize compound task termination
         self.regcond = threading.Condition()
 
-
     def execute(self):
         """Run all child tasks concurrently in separate threads.
         Return 0 after all child tasks have completed execution.
@@ -528,16 +488,16 @@ class oldConcurrentAndTaskset(Task):
         self.totaltime = time.time()
 
         # Register termination callbacks for all my child tasks.
-        for task in self.taskseq:
+        for task in list(self.taskseq):
             self.taskset.append(task)
-            task.register_callback(self.child_done, args=[self.count, task])
+            task.add_callback('resolved', self.child_done, self.count)
             self.count += 1
 
         self.numtasks = self.count
 
         # Now start each child task.
         with self.regcond:
-            for task in self.taskset:
+            for task in list(self.taskset):
                 task.initialize(self)
 
                 task.start()
@@ -561,8 +521,7 @@ class oldConcurrentAndTaskset(Task):
 
         return 0
 
-
-    def child_done(self, result, count, task):
+    def child_done(self, task, result, count):
         """Acquire the condition variable for the compound task object.
         Decrement the thread count.  If we are the last thread to
         finish, release compound task thread, which is blocked in execute().
@@ -577,7 +536,6 @@ class oldConcurrentAndTaskset(Task):
             if self.count <= 0:
                 self.regcond.notifyAll()
 
-
     def stop(self):
         """Call stop() on all child tasks, and ignore TaskError exceptions.
         Behavior depends on what the child tasks' stop() method does."""
@@ -590,19 +548,19 @@ class oldConcurrentAndTaskset(Task):
                 # TODO: notify who?
                 pass
 
-
     def addTask(self, task):
         """Add a task to the task set.
         """
         with self.regcond:
             self.taskset.append(task)
-            task.register_callback(self.child_done, args=[self.numtasks, task])
+            task.add_callback('resolved', self.child_done, self.numtasks)
             self.numtasks += 1
             self.count += 1
 
         task.initialize(self)
         task.start()
 
+
 class newConcurrentAndTaskset(Task):
     """Compound task that runs a set of tasks concurrently, and does not
     return until they all terminate.
@@ -619,7 +577,6 @@ class newConcurrentAndTaskset(Task):
         # internal mutex
         self._lock_c = threading.RLock()
 
-
     def execute(self):
         """Run all child tasks concurrently in separate threads.
         Return last result after all child tasks have completed execution.
@@ -684,17 +641,15 @@ class newConcurrentAndTaskset(Task):
         # Return value of last child to complete
         return value
 
-
     def child_done(self, result, task):
         with self._lock_c:
             self.count += 1
             self.logger.debug('Concurrent task %d/%d has completed' % (
-                    self.count, self.numtasks))
+                self.count, self.numtasks))
             self.taskset.remove(task)
             self.totaltime += task.getExecutionTime()
             self.results[(self.count, task)] = result
 
-
     def stop(self):
         """Call stop() on all child tasks, and ignore TaskError exceptions.
         Behavior depends on what the child tasks' stop() method does."""
@@ -711,7 +666,6 @@ class newConcurrentAndTaskset(Task):
         # stop ourself
         #self.ev_intr.set()
 
-
     def addTask(self, task):
         """Add a task to the task set.
         """
@@ -738,6 +692,7 @@ class newConcurrentAndTaskset(Task):
 class ConcurrentAndTaskset(newConcurrentAndTaskset):
     pass
 
+
 class QueueTaskset(Task):
     """Compound task that runs a set of tasks that it reads from a queue
     concurrently.  If _waitflag_ is True, then it will run each task to
@@ -755,11 +710,10 @@ class QueueTaskset(Task):
         self.task = None
         self.ev_cancel = threading.Event()
         self.ev_pause = threading.Event()
-        if ev_quit == None:
+        if ev_quit is None:
             ev_quit = threading.Event()
         self.ev_quit = ev_quit
 
-
     def flush(self):
         # Flush queue of pending tasks
         self.logger.debug("Flushing queue.")
@@ -769,7 +723,6 @@ class QueueTaskset(Task):
             except Queue.Empty:
                 break
 
-
     def stop(self):
         self.flush()
         #self.ev_intr.set()
@@ -785,7 +738,6 @@ class QueueTaskset(Task):
         # put termination sentinel
         self.queue.put(None)
 
-
     def stop_child(self):
         self.flush()
 
@@ -797,23 +749,22 @@ class QueueTaskset(Task):
             #self.logger.error("Error cancelling child task: %s" % (str(e)))
             pass
 
-
     def execute(self):
         self.count = 0
         self.totaltime = 0
         self.logger.debug("Queue Taskset starting")
-        while not self.ev_quit.isSet():
+        while not self.ev_quit.is_set():
             try:
                 self.check_state()
 
                 task = self.queue.get(block=True, timeout=self.timeout)
-                if task == None:
+                if task is None:
                     # termination sentinel
                     break
 
                 self.task = task
 
-                task.register_callback(self.child_done, args=[task])
+                task.add_callback(self.child_done)
 
                 with self.lock:
                     self.count += 1
@@ -831,11 +782,11 @@ class QueueTaskset(Task):
                         self.logger.debug("Task %s terminated with result %s" % (
                                           (str(task), str(res))))
                 except Exception as e:
-                    self.logger.error("Task '%s' terminated with exception: %s" % \
+                    self.logger.error("Task '%s' terminated with exception: %s" %
                                       (str(task), str(e)))
                     try:
                         (type, value, tb) = sys.exc_info()
-                        self.logger.debug("Traceback:\n%s" % \
+                        self.logger.debug("Traceback:\n%s" %
                                           "".join(traceback.format_tb(tb)))
 
                         # NOTE: to avoid creating a cycle that might cause
@@ -853,26 +804,22 @@ class QueueTaskset(Task):
                 # No task available.  Continue trying to get one.
                 continue
 
-
         # TODO: should we wait for self.count > 0?
         self.logger.debug("Queue Taskset terminating")
 
         return self.result
 
-
-    def child_done(self, result, task):
+    def child_done(self, task, result):
         with self.lock:
             self.count -= 1
             self.totaltime += task.getExecutionTime()
             self.result = result
 
-
     def cancel(self):
         self.flush()
 
         super(QueueTaskset, self).cancel()
 
-
     def addTask(self, task):
         self.queue.put(task)
 
@@ -889,6 +836,7 @@ class _WorkerReset(Exception):
     """Local exception used to reset a worker thread."""
     pass
 
+
 class WorkerThread(object):
     """Container for a thread in which to call the execute() method of a task.
     A WorkerThread object waits on the task queue, executes a task when it
@@ -911,7 +859,6 @@ class WorkerThread(object):
         self.status = 'stopped'
         self.time_start = 0.0
 
-
     def setstatus(self, status):
         """Sets our status field so that others can inquire what we are doing.
         Set of status:
@@ -920,14 +867,12 @@ class WorkerThread(object):
         with self.lock:
             self.status = status
 
-
     def getstatus(self):
         """Returns our status--a string describing what we are doing.
         """
         with self.lock:
             return (self.status, self.time_start)
 
-
     def execute(self, task):
         """Execute a task.
         """
@@ -948,12 +893,12 @@ class WorkerThread(object):
                 res = e
 
             except Exception as e:
-                self.logger.error("Task '%s' raised exception: %s" % \
+                self.logger.error("Task '%s' raised exception: %s" %
                                   (str(task), str(e)))
                 res = e
                 try:
                     (type, value, tb) = sys.exc_info()
-                    self.logger.debug("Traceback:\n%s" % \
+                    self.logger.debug("Traceback:\n%s" %
                                       "".join(traceback.format_tb(tb)))
 
                     # NOTE: to avoid creating a cycle that might cause
@@ -974,7 +919,6 @@ class WorkerThread(object):
             self.time_start = 0.0
             self.setstatus('idle')
 
-
     # Basic task execution loop.  Dequeue a task and run it, then look
     # for another one
     def taskloop(self):
@@ -988,14 +932,14 @@ class WorkerThread(object):
 
         try:
             self.setstatus('idle')
-            while not self.ev_quit.isSet():
+            while not self.ev_quit.is_set():
                 try:
 
                     # Wait on our queue for a task; will timeout in
                     # self.timeout secs
                     (priority, task) = self.queue.get(block=True,
                                                       timeout=self.timeout)
-                    if task == None:
+                    if task is None:
                         # termination sentinel
                         self.queue.put((priority, task))
                         break
@@ -1017,16 +961,16 @@ class WorkerThread(object):
 
             self.setstatus('stopped')
 
-
     def start(self):
         self.thread = threading.Thread(target=self.taskloop, args=[])
         self.thread.start()
 
     def stop(self):
         # Put termination sentinal on queue
-        self.queue.put((priority, task))
+        self.queue.put((0, None))
         self.ev_quit.set()
 
+
 # ------------ THREAD POOL ------------
 
 class ThreadPool(object):
@@ -1060,7 +1004,6 @@ class ThreadPool(object):
         self.runningcount = 0
         self.status = 'down'
 
-
     def startall(self, wait=False, **kwdargs):
         """Start all of the threads in the thread pool.  If _wait_ is True
         then don't return until all threads are up and running.  Any extra
@@ -1069,17 +1012,17 @@ class ThreadPool(object):
         self.logger.debug("startall called")
         with self.regcond:
             while self.status != 'down':
-                if self.status in ('start', 'up') or self.ev_quit.isSet():
+                if self.status in ('start', 'up') or self.ev_quit.is_set():
                     # For now, abandon additional request to start
                     self.logger.error("ignoring duplicate request to start thread pool")
                     return
 
-                self.logger.debug("waiting for threads: count=%d" % \
+                self.logger.debug("waiting for threads: count=%d" %
                                   self.runningcount)
                 self.regcond.wait()
 
             #assert(self.status == 'down')
-            if self.ev_quit.isSet():
+            if self.ev_quit.is_set():
                 return
 
             self.runningcount = 0
@@ -1103,8 +1046,8 @@ class ThreadPool(object):
             # themselves and last one up will set status to "up"
             if wait:
                 # Threads are on the way up.  Wait until last one starts.
-                while self.status != 'up' and not self.ev_quit.isSet():
-                    self.logger.debug("waiting for threads: count=%d" % \
+                while self.status != 'up' and not self.ev_quit.is_set():
+                    self.logger.debug("waiting for threads: count=%d" %
                                       self.runningcount)
                     self.regcond.wait()
             else:
@@ -1112,7 +1055,6 @@ class ThreadPool(object):
                 self.status = 'up'
             self.logger.debug("startall done")
 
-
     def addThreads(self, numthreads, **kwdargs):
         with self.regcond:
             # Start all worker threads
@@ -1134,12 +1076,12 @@ class ThreadPool(object):
         self.logger.debug("stopall called")
         with self.regcond:
             while self.status != 'up':
-                if self.status in ('stop', 'down') or self.ev_quit.isSet():
+                if self.status in ('stop', 'down') or self.ev_quit.is_set():
                     # For now, silently abandon additional request to stop
                     self.logger.warning("ignoring duplicate request to stop thread pool.")
                     return
 
-                self.logger.debug("waiting for threads: count=%d" % \
+                self.logger.debug("waiting for threads: count=%d" %
                                   self.runningcount)
                 self.regcond.wait()
 
@@ -1152,17 +1094,15 @@ class ThreadPool(object):
             if wait:
                 # Threads are on the way down.  Wait until last one quits.
                 while self.status != 'down':
-                    self.logger.debug("waiting for threads: count=%d" % \
+                    self.logger.debug("waiting for threads: count=%d" %
                                       self.runningcount)
                     self.regcond.wait()
 
             self.logger.debug("stopall done")
 
-
     def workerStatus(self):
         return list(map(lambda t: t.getstatus(), self.workers))
 
-
     def addTask(self, task, priority=0):
         """Add a task to the queue of tasks.
 
@@ -1171,15 +1111,12 @@ class ThreadPool(object):
         """
         self.queue.put((priority, task))
 
-
     def delTask(self, taskid):
         self.logger.error("delTask not yet implemented")
 
-
     def purgeTasks(self):
         self.logger.error("purgeTasks not yet implemented")
 
-
     def register_up(self):
         """Called by WorkerThread objects to register themselves.
 
@@ -1192,13 +1129,12 @@ class ThreadPool(object):
             self.runningcount += 1
             tid = thread.get_ident()
             self.tids.append(tid)
-            self.logger.debug("register_up: (%d) count is %d" % \
+            self.logger.debug("register_up: (%d) count is %d" %
                               (tid, self.runningcount))
             if self.runningcount == self.numthreads:
                 self.status = 'up'
             self.regcond.notify()
 
-
     def register_dn(self):
         """Called by WorkerThread objects to register themselves.
 
@@ -1222,6 +1158,7 @@ class ThreadPool(object):
 _lock_seqnum = threading.Lock()
 _count_seqnum = 0
 
+
 def get_tag(taskParent):
     global _count_seqnum
     with _lock_seqnum:
@@ -1236,4 +1173,4 @@ def get_tag(taskParent):
     return tag
 
 
-#END
+# END
