@@ -3,28 +3,23 @@
 #
 # This is open-source software licensed under a BSD license.
 # Please see the file LICENSE.txt for details.
+from __future__ import absolute_import
 
-import sys, re
-import numpy
-import threading
-import math
 from io import BytesIO
 
 # Matplotlib imports
 import matplotlib
-from matplotlib.image import FigureImage
-from matplotlib.figure import Figure
-import matplotlib.lines as lines
 #from matplotlib.path import Path
 
 from ginga import ImageView
-from ginga import Mixins, Bindings, colors
+from ginga import Mixins, Bindings
+
 # NOTE: this import is necessary to register the 'ginga' projection
 # used via matplotlib, even though the module is not directly
 # referenced within this code
-from . import transform
-from ginga.mplw.CanvasRenderMpl import CanvasRenderer
-from ginga.mplw.MplHelp import Timer
+from . import transform  # noqa
+from .CanvasRenderMpl import CanvasRenderer
+from .MplHelp import Timer
 
 # Override some matplotlib keyboard UI defaults
 rc = matplotlib.rcParams
@@ -34,7 +29,7 @@ rc.update(matplotlib.rcParamsDefault)
 rc['keymap.fullscreen'] = 'f'    # toggling full screen
 rc['keymap.home'] = 'home'       # home or reset mnemonic
 rc['keymap.back'] = 'left'       # forward / backward keys to enable
-rc['keymap.forward'] = 'right'   #   left handed quick navigation
+rc['keymap.forward'] = 'right'   # left handed quick navigation
 rc['keymap.pan'] = []            # pan mnemonic
 rc['keymap.zoom'] = []           # zoom mnemonic
 rc['keymap.save'] = 'ctrl+s'     # saving current figure
@@ -45,9 +40,13 @@ rc['keymap.xscale'] = []         # toggle scaling of x-axes ('log'/'linear')
 rc['keymap.all_axes'] = []       # enable all axes
 
 
+MPL_V1 = matplotlib.__version__.startswith('1')
+
+
 class ImageViewMplError(ImageView.ImageViewError):
     pass
 
+
 class ImageViewMpl(ImageView.ImageViewBase):
 
     def __init__(self, logger=None, rgbmap=None, settings=None):
@@ -90,7 +89,9 @@ class ImageViewMpl(ImageView.ImageViewBase):
         #ax = fig.add_subplot(111)
         self.ax_img = ax
         # We don't want the axes cleared every time plot() is called
-        ax.hold(False)
+        if MPL_V1:
+            # older versions of matplotlib
+            ax.hold(False)
         # TODO: is this needed, since frame_on == False?
         ax.get_xaxis().set_visible(False)
         ax.get_yaxis().set_visible(False)
@@ -106,7 +107,8 @@ class ImageViewMpl(ImageView.ImageViewBase):
                                      #viewer=self,
                                      #projection='ginga'
                                      )
-        newax.hold(True)
+        if MPL_V1:
+            newax.hold(True)
         newax.autoscale(enable=False)
         newax.get_xaxis().set_visible(False)
         newax.get_yaxis().set_visible(False)
@@ -216,10 +218,10 @@ class ImageViewMpl(ImageView.ImageViewBase):
 
         if self.mpimage is None:
             img = self.ax_img.imshow(arr, interpolation='none',
-                                   origin="upper",
-                                   vmin=0, vmax=255,
-                                   extent=extent,
-                                   aspect=aspect)
+                                     origin="upper",
+                                     vmin=0, vmax=255,
+                                     extent=extent,
+                                     aspect=aspect)
             self.mpimage = img
 
         else:
@@ -282,11 +284,27 @@ class ImageViewMpl(ImageView.ImageViewBase):
         return ax
 
     def get_png_image_as_buffer(self, output=None):
-        ibuf = output
-        if ibuf is None:
-            ibuf = BytesIO()
-        qimg = self.figure.write_to_png(ibuf)
-        return ibuf
+        """DO NOT USE: *** TO BE DEPRECATED ***
+        Use get_rgb_image_as_buffer() instead.
+        """
+        return self.get_rgb_image_as_buffer(output=output, format='png')
+
+    def get_rgb_image_as_buffer(self, output=None, format='png', quality=90):
+        if self.figure is None:
+            raise ImageViewMplError("No matplotlib figure defined")
+
+        obuf = output
+        if obuf is None:
+            obuf = BytesIO()
+
+        self.figure.savefig(obuf, format=format, pad_inches=0)
+        if output is not None:
+            return None
+        return obuf
+
+    def get_rgb_image_as_bytes(self, format='png', quality=90):
+        buf = self.get_rgb_image_as_buffer(format=format, quality=quality)
+        return buf.getvalue()
 
     def update_image(self):
         pass
@@ -360,7 +378,7 @@ class ImageViewEvent(ImageViewMpl):
             'end': 'end',
             'pageup': 'page_up',
             'pagedown': 'page_down',
-            }
+        }
 
         # Define cursors for pick and pan
         #hand = openHandCursor()
@@ -505,7 +523,8 @@ class ImageViewEvent(ImageViewMpl):
         data_x, data_y = self.check_cursor_location()
 
         return self.make_ui_callback('scroll', direction, amount,
-                                  data_x, data_y)
+                                     data_x, data_y)
+
 
 class ImageViewZoom(Mixins.UIMixin, ImageViewEvent):
 
