@@ -4,15 +4,15 @@
 # This is open-source software licensed under a BSD license.
 # Please see the file LICENSE.txt for details.
 #
-import threading
 import time
-from datetime import datetime
 
 from ginga.misc import Bunch, Datasrc, Callback, Future
 
+
 class ChannelError(Exception):
     pass
 
+
 class Channel(Callback.Callbacks):
     """Class to manage a channel.
 
@@ -38,7 +38,6 @@ class Channel(Callback.Callbacks):
         self.fv = fv
         self.settings = settings
         self.logger = fv.logger
-        self.lock = threading.RLock()
 
         # CHANNEL ATTRIBUTES
         self.name = name
@@ -67,7 +66,7 @@ class Channel(Callback.Callbacks):
             'set', self._sort_changed_ext_cb)
 
     def connect_viewer(self, viewer):
-        if not viewer in self.viewers:
+        if viewer not in self.viewers:
             self.viewers.append(viewer)
             self.viewer_dict[viewer.vname] = viewer
 
@@ -100,7 +99,8 @@ class Channel(Callback.Callbacks):
             channel.datasrc[imname] = image
 
             if not silent:
-                self.fv.gui_do(channel._add_image_update, image, info)
+                self.fv.gui_do(channel.add_image_update, image, info,
+                               update_viewer=False)
 
     def remove_image(self, imname):
         info = self.image_index[imname]
@@ -121,7 +121,7 @@ class Channel(Callback.Callbacks):
         return info
 
     def get_image_names(self):
-        return [ info.name for info in self.history ]
+        return [info.name for info in self.history]
 
     def get_loaded_image(self, imname):
         """Get an image from memory.
@@ -148,8 +148,8 @@ class Channel(Callback.Callbacks):
     def add_image(self, image, silent=False, bulk_add=False):
 
         imname = image.get('name', None)
-        assert imname is not None, \
-               ValueError("image has no name")
+        if imname is None:
+            raise ValueError("image has no name")
 
         self.logger.debug("Adding image '%s' in channel %s" % (
             imname, self.name))
@@ -168,21 +168,11 @@ class Channel(Callback.Callbacks):
                                     image_loader=image_loader,
                                     image_future=image_future,
                                     idx=idx)
-
-        # we'll get notified if an image changes and can update
-        # metadata and make a chained callback
-        image.add_callback('modified', self._image_modified_cb)
+            image.set(image_info=info)
 
         if not silent:
-            if not bulk_add:
-                self._add_image_update(image, info)
-                return
-
-            # By using gui_do() here, more images may be bulk added
-            # before the _add_image_update executes--it will then
-            # only update the gui for the latest image, which saves
-            # work
-            self.fv.gui_do(self._add_image_update, image, info)
+            self.add_image_update(image, info,
+                                  update_viewer=not bulk_add)
 
     def add_image_info(self, info):
 
@@ -201,9 +191,33 @@ class Channel(Callback.Callbacks):
     def get_image_info(self, imname):
         return self.image_index[imname]
 
-    def _add_image_update(self, image, info):
+    def update_image_info(self, image, info):
+        imname = image.get('name', None)
+        if (imname is None) or (imname not in self.image_index):
+            return False
+
+        # don't update based on image name alone--actual image must match
+        try:
+            my_img = self.get_loaded_image(imname)
+            if my_img is not image:
+                return False
+
+        except KeyError:
+            return False
+
+        # update the info record
+        iminfo = self.get_image_info(imname)
+        iminfo.update(info)
+
+        self.fv.make_async_gui_callback('add-image-info', self, iminfo)
+        return True
+
+    def add_image_update(self, image, info, update_viewer=False):
         self.fv.make_async_gui_callback('add-image', self.name, image, info)
 
+        if not update_viewer:
+            return
+
         current = self.datasrc.youngest()
         curname = current.get('name')
         self.logger.debug("image=%s youngest=%s" % (image.get('name'), curname))
@@ -220,18 +234,6 @@ class Channel(Callback.Callbacks):
             if channel != self:
                 self.fv.change_channel(self.name)
 
-    def _image_modified_cb(self, image):
-        imname = image.get('name', None)
-        if (imname is None) or (imname not in self.image_index):
-            # not one of ours apparently (maybe used to be, but got removed)
-            return
-
-        info = self.image_index[imname]
-        info.time_modified = datetime.utcnow()
-        self.logger.debug("image modified; making chained callback")
-
-        self.fv.make_async_gui_callback('add-image-info', self, info)
-
     def refresh_cursor_image(self):
         if self.cursor < 0:
             self.viewer.clear()
@@ -239,7 +241,7 @@ class Channel(Callback.Callbacks):
             return
 
         info = self.history[self.cursor]
-        if self.datasrc.has_key(info.name):
+        if info.name in self.datasrc:
             # image still in memory
             image = self.datasrc[info.name]
             self.switch_image(image)
@@ -248,48 +250,45 @@ class Channel(Callback.Callbacks):
             self.switch_name(info.name)
 
     def prev_image(self, loop=True):
-        with self.lock:
-            self.logger.debug("Previous image")
-            if self.cursor <= 0:
-                n = len(self.history) - 1
-                if (not loop) or (n < 0):
-                    self.logger.error("No previous image!")
-                    return True
-                self.cursor = n
-            else:
-                self.cursor -= 1
+        self.logger.debug("Previous image")
+        if self.cursor <= 0:
+            n = len(self.history) - 1
+            if (not loop) or (n < 0):
+                self.logger.error("No previous image!")
+                return True
+            self.cursor = n
+        else:
+            self.cursor -= 1
 
-            self.refresh_cursor_image()
+        self.refresh_cursor_image()
 
         return True
 
     def next_image(self, loop=True):
-        with self.lock:
-            self.logger.debug("Next image")
-            n = len(self.history) - 1
-            if self.cursor >= n:
-                if (not loop) or (n < 0):
-                    self.logger.error("No next image!")
-                    return True
-                self.cursor = 0
-            else:
-                self.cursor += 1
+        self.logger.debug("Next image")
+        n = len(self.history) - 1
+        if self.cursor >= n:
+            if (not loop) or (n < 0):
+                self.logger.error("No next image!")
+                return True
+            self.cursor = 0
+        else:
+            self.cursor += 1
 
-            self.refresh_cursor_image()
+        self.refresh_cursor_image()
 
         return True
 
     def _add_info(self, info):
-        if info in self.image_index:
+        if info.name in self.image_index:
             # image info is already present
             return False
 
-        with self.lock:
-            self.history.append(info)
-            self.image_index[info.name] = info
+        self.history.append(info)
+        self.image_index[info.name] = info
 
-            if self.hist_sort is not None:
-                self.history.sort(key=self.hist_sort)
+        if self.hist_sort is not None:
+            self.history.sort(key=self.hist_sort)
 
         self.fv.make_async_gui_callback('add-image-info', self, info)
 
@@ -354,7 +353,7 @@ class Channel(Callback.Callbacks):
         vname = vnames[0]
 
         # if we don't have this viewer type then install one in the channel
-        if not vname in self.viewer_dict:
+        if vname not in self.viewer_dict:
             self.fv.make_viewer(vname, self)
 
         self.viewer = self.viewer_dict[vname]
@@ -365,47 +364,45 @@ class Channel(Callback.Callbacks):
         # and load the data
         self.viewer.set_image(dataobj)
 
-
     def switch_image(self, image):
 
-        with self.lock:
-            curimage = self.get_current_image()
-            if curimage != image:
-                self.logger.debug("updating viewer...")
-                self.view_object(image)
+        curimage = self.get_current_image()
+        if curimage != image:
+            self.logger.debug("updating viewer...")
+            self.view_object(image)
 
-                # update cursor to match image
-                imname = image.get('name')
-                if imname in self.image_index:
-                    info = self.image_index[imname]
-                    if info in self.history:
-                        self.cursor = self.history.index(info)
+            # update cursor to match image
+            imname = image.get('name')
+            if imname in self.image_index:
+                info = self.image_index[imname]
+                if info in self.history:
+                    self.cursor = self.history.index(info)
 
-                self.fv.channel_image_updated(self, image)
+            self.fv.channel_image_updated(self, image)
 
-                # Check for preloading any images into memory
-                preload = self.settings.get('preload_images', False)
-                if not preload:
-                    return
+            # Check for preloading any images into memory
+            preload = self.settings.get('preload_images', False)
+            if not preload:
+                return
 
-                # queue next and previous files for preloading
-                index = self.cursor
-                if index < len(self.history)-1:
-                    info = self.history[index+1]
-                    if info.path is not None:
-                        self.fv.add_preload(self.name, info)
+            # queue next and previous files for preloading
+            index = self.cursor
+            if index < len(self.history) - 1:
+                info = self.history[index + 1]
+                if info.path is not None:
+                    self.fv.add_preload(self.name, info)
 
-                if index > 0:
-                    info = self.history[index-1]
-                    if info.path is not None:
-                        self.fv.add_preload(self.name, info)
+            if index > 0:
+                info = self.history[index - 1]
+                if info.path is not None:
+                    self.fv.add_preload(self.name, info)
 
-            else:
-                self.logger.debug("Apparently no need to set image.")
+        else:
+            self.logger.debug("Apparently no need to set channel viewer.")
 
     def switch_name(self, imname):
 
-        if self.datasrc.has_key(imname):
+        if imname in self.datasrc:
             # Image is still in the heap
             image = self.datasrc[imname]
             self.switch_image(image)
@@ -422,6 +419,7 @@ class Channel(Callback.Callbacks):
         if info.image_future is not None:
             self.logger.info("Image '%s' is no longer in memory; attempting "
                              "image future" % (imname))
+
             # TODO: recode this--it's a bit messy
             def _switch(image):
                 # this will be executed in the gui thread
