@@ -1,10 +1,54 @@
-#
-# Histogram.py -- Histogram plugin for Ginga reference viewer
-#
 # This is open-source software licensed under a BSD license.
 # Please see the file LICENSE.txt for details.
-#
-import numpy
+"""
+``Histogram`` plots a histogram for a region drawn in the image, or for the
+entire image.
+
+**Plugin Type: Local**
+
+``Histogram`` is a local plugin, which means it is associated with a channel.
+An instance can be opened for each channel.
+
+**Usage**
+
+Click and drag to define a region within the image that will be used to
+calculate the histogram.  To take the histogram of the full image, click
+the button in the UI labeled "Full Image".
+
+.. note:: Depending on the size of the image, calculating the
+          full histogram may take time.
+
+If a new image is selected for the channel, the histogram plot will be
+recalculated based on the current parameters with the new data.
+
+**UI Controls**
+
+Three radio buttons at the bottom of the UI are used to control the
+effects of the click/drag action:
+
+* select "Move" to drag the region to a different location
+* select "Draw" to draw a new region
+* select "Edit" to edit the region
+
+To make a log plot of the histogram, check the "Log Histogram" checkbox.
+To plot by the full range of values in the image instead of by the range
+within the cut values, uncheck the "Plot By Cuts" checkbox.
+
+The "NumBins" parameter determines how many bins are used in calculating
+the histogram.  Type a number in the box and press "Enter" to change the
+default value.
+
+**Cut Levels Convenience Controls**
+
+Because a histogram is useful feedback for setting the cut levels,
+controls are provided in the UI for setting the low and high cut levels
+in the image, as well as for performing an auto cut levels, according to
+the auto cut levels settings in the channel preferences.
+
+**User Configuration**
+
+"""
+import numpy as np
 
 from ginga.gw import Widgets
 from ginga import GingaPlugin
@@ -17,53 +61,11 @@ try:
 except ImportError:
     have_mpl = False
 
+__all__ = ['Histogram']
+
 
 class Histogram(GingaPlugin.LocalPlugin):
-    """
-    Histogram
-    =========
-    Histogram plots a histogram for a region drawn in the image, or for the
-    entire image.
 
-    Plugin Type: Local
-    ------------------
-    Histogram is a local plugin, which means it is associated with a channel.
-    An instance can be opened for each channel.
-
-    If a new image is selected for the channel the histogram plot will be
-    recalculated based on the current parameters with the new data.
-
-    Usage
-    -----
-    Click and drag to define a region within the image that will be used to
-    calculate the histogram.  To take the histogram of the full image, click
-    the button in the UI labeled "Full Image" (NOTE: depending on the size
-    of the image calculating the full histogram can take time).
-
-    UI Controls
-    -----------
-    Three radio buttons at the bottom of the UI are used to control the
-    effects of the click/drag action:
-
-    * select "Move" to drag the region to a different location
-    * select "Draw" to draw a new region
-    * select "Edit" to edit the region
-
-    To make a log plot of the histogram, check the "Log Histogram" checkbox.
-    To plot by the full range of values in the image instead of by the range
-    within the cut values, uncheck the "Plot By Cuts" checkbox.
-
-    The "NumBins" parameter determines how many bins are used in calculating
-    the histogram.  Type a number in the box and press Enter to change the
-    default value.
-
-    Cut Levels Convenience Controls
-    -------------------------------
-    Because a histogram is useful feedback for setting the cut levels,
-    controls are provided in the UI for setting the low and high cut levels
-    in the image, as well as for performing an auto cut levels, according to
-    the auto cut levels settings in the channel preferences.
-    """
     def __init__(self, fv, fitsimage):
         # superclass defines some variables for us, like logger
         super(Histogram, self).__init__(fv, fitsimage)
@@ -234,7 +236,7 @@ class Histogram(GingaPlugin.LocalPlugin):
         # insert canvas, if not already
         p_canvas = self.fitsimage.get_canvas()
         try:
-            obj = p_canvas.get_object_by_tag(self.layertag)
+            p_canvas.get_object_by_tag(self.layertag)
 
         except KeyError:
             # Add ruler layer
@@ -257,7 +259,7 @@ class Histogram(GingaPlugin.LocalPlugin):
         # remove the rect from the canvas
         ## try:
         ##     self.canvas.delete_object_by_tag(self.histtag)
-        ## except:
+        ## except Exception:
         ##     pass
         ##self.histtag = None
 
@@ -265,7 +267,7 @@ class Histogram(GingaPlugin.LocalPlugin):
         p_canvas = self.fitsimage.get_canvas()
         try:
             p_canvas.delete_object_by_tag(self.layertag)
-        except:
+        except Exception:
             pass
         self.gui_up = False
         self.fv.show_status("")
@@ -274,12 +276,12 @@ class Histogram(GingaPlugin.LocalPlugin):
         canvas = self.canvas
         try:
             canvas.delete_object_by_tag(self.histtag)
-        except:
+        except Exception:
             pass
 
         image = self.fitsimage.get_image()
         width, height = image.get_size()
-        x1, y1, x2, y2 = 0, 0, width-1, height-1
+        x1, y1, x2, y2 = 0, 0, width - 1, height - 1
         tag = canvas.add(self.dc.Rectangle(x1, y1, x2, y2,
                                            color='cyan',
                                            linestyle='dash'))
@@ -319,13 +321,13 @@ class Histogram(GingaPlugin.LocalPlugin):
                                  int(bbox.x2), int(bbox.y2),
                                  pct=1.0, numbins=numbins)
             # used with 'steps-post' drawstyle, this x and y assignment
-                # gives correct histogram-steps
+            # gives correct histogram-steps
             x = res.bins
-            y = numpy.append(res.dist, res.dist[-1])
+            y = np.append(res.dist, res.dist[-1])
             ## y, x = y[i:j+1], x[i:j+1]
             ymax = y.max()
             if self.plot.logy:
-                y = numpy.choose(y > 0, (.1, y))
+                y = np.choose(y > 0, (.1, y))
             self.plot.plot(x, y, xtitle="Pixel value", ytitle="Number",
                            title="Pixel Value Distribution",
                            color='blue', alpha=1.0, drawstyle='steps-post')
@@ -339,11 +341,11 @@ class Histogram(GingaPlugin.LocalPlugin):
                 # used with 'steps-post' drawstyle, this x and y assignment
                 # gives correct histogram-steps
                 x = res.bins
-                y = numpy.append(res.dist, res.dist[-1])
+                y = np.append(res.dist, res.dist[-1])
                 ## y, x = y[i:j+1], x[i:j+1]
                 ymax = max(ymax, y.max())
                 if self.plot.logy:
-                    y = numpy.choose(y > 0, (.1, y))
+                    y = np.choose(y > 0, (.1, y))
                 self.plot.plot(x, y, xtitle="Pixel value", ytitle="Number",
                                title="Pixel Value Distribution",
                                color=colors[z], alpha=0.33,
@@ -392,11 +394,11 @@ class Histogram(GingaPlugin.LocalPlugin):
         dy = (data_y - y)
 
         # calculate new coords
-        x1, y1, x2, y2 = bbox.x1+dx, bbox.y1+dy, bbox.x2+dx, bbox.y2+dy
+        x1, y1, x2, y2 = bbox.x1 + dx, bbox.y1 + dy, bbox.x2 + dx, bbox.y2 + dy
 
         try:
             canvas.delete_object_by_tag(self.histtag)
-        except:
+        except Exception:
             pass
 
         tag = canvas.add(self.dc.Rectangle(
@@ -427,12 +429,12 @@ class Histogram(GingaPlugin.LocalPlugin):
         dy = (data_y - y)
 
         # calculate new coords
-        x1, y1, x2, y2 = bbox.x1+dx, bbox.y1+dy, bbox.x2+dx, bbox.y2+dy
+        x1, y1, x2, y2 = bbox.x1 + dx, bbox.y1 + dy, bbox.x2 + dx, bbox.y2 + dy
 
         if obj.kind == 'compound':
             try:
                 canvas.delete_object_by_tag(self.histtag)
-            except:
+            except Exception:
                 pass
 
             self.histtag = canvas.add(self.dc.Rectangle(
@@ -452,7 +454,7 @@ class Histogram(GingaPlugin.LocalPlugin):
         if self.histtag:
             try:
                 canvas.delete_object_by_tag(self.histtag)
-            except:
+            except Exception:
                 pass
 
         x1, y1, x2, y2 = obj.get_llur()
@@ -460,7 +462,7 @@ class Histogram(GingaPlugin.LocalPlugin):
         tag = canvas.add(self.dc.CompoundObject(
             self.dc.Rectangle(x1, y1, x2, y2,
                               color=self.histcolor),
-            self.dc.Text(x1, y2+4, "Histogram",
+            self.dc.Text(x1, y2 + 4, "Histogram",
                          color=self.histcolor)))
         self.histtag = tag
 
@@ -525,7 +527,7 @@ class Histogram(GingaPlugin.LocalPlugin):
         try:
             self.loline.remove()
             self.hiline.remove()
-        except:
+        except Exception:
             pass
         self.loline = self.plot.ax.axvline(loval, 0.0, 0.99,
                                            linestyle='-', color='black')
@@ -581,4 +583,10 @@ class Histogram(GingaPlugin.LocalPlugin):
     def __str__(self):
         return 'histogram'
 
+
+# Append module docstring with config doc for auto insert by Sphinx.
+from ginga.util.toolbox import generate_cfg_example  # noqa
+if __doc__ is not None:
+    __doc__ += generate_cfg_example('plugin_Histogram', package='ginga')
+
 # END
