@@ -1,61 +1,84 @@
-import sys, os
-import logging
+# This is open-source software licensed under a BSD license.
+# Please see the file LICENSE.txt for details.
+"""
+The ``IRAF`` plugin implements a remote control interface for the Ginga FITS
+viewer from an IRAF session.  In particular, it supports the use of the
+IRAF ``display`` and ``imexamine`` commands.
+
+**Plugin Type: Global**
+
+``IRAF`` is a global plugin.  Only one instance can be opened.
+
+**Usage**
+
+The ``IRAF`` plugin allows Ginga to interoperate with IRAF in a manner
+similar to IRAF and DS9.  The following IRAF commands are supported:
+``imexamine``, ``rimcursor``, ``display``, and ``tvmark``.
+
+To use the ``IRAF`` plugin, first make sure the environment variable ``IMTDEV``
+is set appropriately, e.g.::
+
+        $ export IMTDEV=inet:45005
+
+or::
+
+        $ export IMTDEV=unix:/tmp/.imtg45
+
+If the environment variable is not set, Ginga will default to that used
+by IRAF.
+
+Use the "Start IRAF" menu item from the "Plugins" menu.
+The GUI for the ``IRAF`` plugin will appear in the tabs on the right.
+
+It can be more convenient to load images via Ginga than IRAF.  From
+Ginga, you can load images via drag and drop or via the ``FBrowser``
+plugin and then use ``imexamine`` from IRAF to do analysis tasks on
+them.  You can also use the ``display`` command from IRAF to show
+images already loaded in IRAF in Ginga, and then use ``imexamine`` to
+select areas graphically for analysis.
+
+When using ``imexamine`` or ``rimcursor``, the plugin disables
+normal UI processing on the channel image so that keystrokes,
+etc. normally caught by Ginga are passed through to IRAF.  You can
+toggle back and forth between local Ginga control (e.g., keystrokes to
+zoom and pan the image, or apply cut levels, etc.) and IRAF control
+using the radio buttons at the top of the tab.
+
+IRAF deals with images in enumerated "frames", whereas Ginga uses
+named channels.  The bottom of the ``IRAF`` plugin GUI will show the mapping
+from Ginga channels to IRAF frames.
+
+"""
+import os
 import threading
+import time
 import socket
+import array
+
+import numpy
+
 import ginga.util.six as six
+from ginga import GingaPlugin, AstroImage
+from ginga import cmap, imap
+from ginga.gw import Widgets
+from ginga.misc import Bunch
+
+# XImage protocol support
+try:
+    import IIS_DataListener as iis
+except ImportError:  # Need this for Sphinx build
+    pass
+
 if six.PY2:
     import Queue
 else:
     import queue as Queue
-import array
-import numpy
-import time
 
-from ginga import GingaPlugin, AstroImage
-from ginga import cmap, imap
-from ginga.gw import Widgets, Viewers
-from ginga.misc import Bunch
-
-# XImage protocol support
-import IIS_DataListener as iis
+__all__ = ['IRAF']
 
 
 class IRAF(GingaPlugin.GlobalPlugin):
-    """
-    IRAF
-    ====
-    The IRAF plugin implements a remote control interface for the Ginga FITS
-    viewer from an IRAF session.  In particular it supports the use of the
-    IRAF 'display' and 'imexamine' commands.
 
-    Plugin Type: Global
-    -------------------
-    IRAF is a global plugin.  Only one instance can be opened.
-
-    Usage
-    -----
-
-    Set the environment variable IMTDEV appropriately, e.g.::
-
-        $ export IMTDEV=inet:45005         (or)
-        $ export IMTDEV=unix:/tmp/.imtg45
-
-    Ginga will try to use the default value if none is assigned.
-
-    Start IRAF plugin (Plugins->Start IRAF).
-
-    From Ginga you can load images and then use 'imexamine' from IRAF to load
-    them, do photometry, etc.  You can also use the 'display' command from IRAF
-    to show images in Ginga.  The 'IRAF' tab will show the mapping from Ginga
-    channels to IRAF numerical 'frames'.
-
-    When using imexamine, the plugin disables normal UI processing on the
-    channel image so that keystrokes, etc. are passed through to IRAF.  You can
-    toggle back and forth between local Ginga control and IRAF control using
-    the radio buttons at the top of the tab or using the space bar.
-
-    IRAF commands that have been tested: display, imexam, rimcur and tvmark.
-    """
     def __init__(self, fv):
         # superclass defines some variables for us, like logger
         super(IRAF, self).__init__(fv)
@@ -64,7 +87,7 @@ class IRAF(GingaPlugin.GlobalPlugin):
         self.keyevent = threading.Event()
         self.keymap = {
             'comma': ',',
-            }
+        }
         self.ctrldown = False
 
         self.layertag = 'iraf-canvas'
@@ -120,7 +143,7 @@ class IRAF(GingaPlugin.GlobalPlugin):
             ("Set Addr:", 'label', "Set Addr", 'entry'),
             ("Control", 'hbox'),
             ("Channel:", 'label', 'Channel', 'llabel'),
-            ]
+        ]
         w, b = Widgets.build_info(captions)
         self.w.update(b)
 
@@ -178,7 +201,6 @@ class IRAF(GingaPlugin.GlobalPlugin):
         fmap = self.get_channel_frame_mapping()
         self.update_chinfo(fmap)
 
-
     def update_chinfo(self, fmap):
         if not self.gui_up:
             return
@@ -209,7 +231,6 @@ class IRAF(GingaPlugin.GlobalPlugin):
             self.logger.info("setting mode to IRAF")
             self.set_mode('IRAF', chname)
 
-
     def add_channel(self, viewer, chinfo):
         self.logger.debug("channel %s added." % (chinfo.name))
 
@@ -225,7 +246,6 @@ class IRAF(GingaPlugin.GlobalPlugin):
         fmap = self.get_channel_frame_mapping()
         self.fv.gui_do(self.update_chinfo, fmap)
 
-
     def delete_channel(self, viewer, chinfo):
         self.logger.debug("delete channel %s" % (chinfo.name))
 
@@ -250,7 +270,7 @@ class IRAF(GingaPlugin.GlobalPlugin):
         try:
             if self.addr.prot == 'unix':
                 os.remove(self.addr.path)
-        except:
+        except Exception:
             pass
 
         # start the data listener task, if appropriate
@@ -285,7 +305,7 @@ class IRAF(GingaPlugin.GlobalPlugin):
         return None
 
     def get_channel_frame_mapping(self):
-        l = [ (fb.chname, n+1) for n, fb in self.fb.items() ]
+        l = [(fb.chname, n + 1) for n, fb in self.fb.items()]
         return l
 
     def redo(self, channel, image):
@@ -305,9 +325,9 @@ class IRAF(GingaPlugin.GlobalPlugin):
             return
         self.logger.debug("new image, frame is %d" % (n))
         fb = self.get_frame(n)
-        newpath  = image.get('path', 'NO_PATH')
+        newpath = image.get('path', 'NO_PATH')
         host = socket.getfqdn()
-        newhost  = image.get('host', host)
+        newhost = image.get('host', host)
         # protocol has a bizarre 16-char limit on hostname
         newhost = newhost[:16]
 
@@ -367,7 +387,6 @@ class IRAF(GingaPlugin.GlobalPlugin):
         fb.wcs = wcs + mapping
         self.logger.debug("filled wcs info")
 
-
     # ------ BEGIN (methods called by IIS server) ----------------
     def init_frame(self, n):
         """
@@ -419,14 +438,14 @@ class IRAF(GingaPlugin.GlobalPlugin):
         # frames are indexed from 1 in IRAF
         chname = fb.chname
         if chname is None:
-            chname = 'Frame%d' % (frame+1)
+            chname = 'Frame%d' % (frame + 1)
             fb.chname = chname
 
-        self.logger.debug("display to %s" %(chname))
+        self.logger.debug("display to %s" % (chname))
 
         try:
             data = fb.buffer
-            byteswap = False
+            byteswap = False  # noqa
             dims = (fb.height, fb.width)
             dtype = numpy.uint8
             metadata = {}
@@ -515,7 +534,7 @@ class IRAF(GingaPlugin.GlobalPlugin):
         # Find out which frame we are looking at
         #frame = self.current_frame
         frame = self.channel_to_frame(chinfo.name)
-        image = fitsimage.get_image()
+        image = fitsimage.get_image()  # noqa
 
         self.start_imexamine(fitsimage, chinfo.name)
 
@@ -550,7 +569,7 @@ class IRAF(GingaPlugin.GlobalPlugin):
         self.canvas.set_surface(fitsimage)
         # insert layer if it is not already
         try:
-            obj = fitsimage.get_object_by_tag(self.layertag)
+            fitsimage.get_object_by_tag(self.layertag)
 
         except KeyError:
             # Add canvas layer
@@ -594,7 +613,8 @@ class IRAF(GingaPlugin.GlobalPlugin):
                 keyname = chr(4)
 
         # Get cursor position
-        fitsimage = canvas.get_surface()
+        # TODO: not sure that this attribute will be permanent...
+        fitsimage = canvas.viewer
         last_x, last_y = fitsimage.get_last_data_xy()
 
         # Correct for surrounding framebuffer
@@ -655,7 +675,7 @@ class IRAF_AstroImage(AstroImage.AstroImage):
         try:
             # We report the value across the pixel, even though the coords
             # change halfway across the pixel
-            x, y = int(data_x+0.5), int(data_y+0.5)
+            x, y = int(data_x + 0.5), int(data_y + 0.5)
             value = self.get_data_xy(x, y)
 
             # Mapping from bytescaled values back to original values
@@ -669,24 +689,24 @@ class IRAF_AstroImage(AstroImage.AstroImage):
         try:
             # Subtract offsets of data in framebuffer and add offsets of
             # rect beginning in source
-            data_x = data_x - (ct.dx-1) + (ct.sx-1)
-            data_y = data_y - (ct.dy-1) + (ct.sy-1)
+            data_x = data_x - (ct.dx - 1) + (ct.sx - 1)
+            data_y = data_y - (ct.dy - 1) + (ct.sy - 1)
 
             #ra_deg, dec_deg = wcs_coord_transform(ct, data_x, data_y)
 
             #ra_txt, dec_txt = self.wcs.deg2fmt(ra_deg, dec_deg, 'str')
-            ra_txt  = 'BAD WCS'
+            ra_txt = 'BAD WCS'
             dec_txt = 'BAD WCS'
 
         except Exception as e:
             self.logger.warning("Bad coordinate conversion: %s" % (
                 str(e)))
-            ra_txt  = 'BAD WCS'
+            ra_txt = 'BAD WCS'
             dec_txt = 'BAD WCS'
 
         # Note: FITS coordinates are 1-based, whereas numpy FITS arrays
         # are 0-based
-        ra_lbl, dec_lbl = unichr(945), unichr(948)
+        ra_lbl, dec_lbl = six.unichr(945), six.unichr(948)
         fits_x, fits_y = data_x + 1, data_y + 1
 
         info = Bunch.Bunch(itype='astro', data_x=data_x, data_y=data_y,
@@ -702,8 +722,8 @@ class IRAF_AstroImage(AstroImage.AstroImage):
         if ct is not None:
             # Subtract offsets of data in framebuffer and add offsets of
             # rect beginning in source
-            data_x = data_x - (ct.dx-1) + (ct.sx-1)
-            data_y = data_y - (ct.dy-1) + (ct.sy-1)
+            data_x = data_x - (ct.dx - 1) + (ct.sx - 1)
+            data_y = data_y - (ct.dy - 1) + (ct.sy - 1)
         return data_x, data_y
 
 
@@ -966,7 +986,7 @@ cmap_iis_iraf = (
     (1.000000, 0.000000, 0.000000),
     (1.000000, 0.000000, 0.000000),
     (0.000000, 0.000000, 0.000000),   # 255: black
-    )
+)
 
 
-#END
+# END
