@@ -1,37 +1,62 @@
-#
-# MultiDim.py -- Multidimensional plugin for Ginga reference viewer
-#
 # This is open-source software licensed under a BSD license.
 # Please see the file LICENSE.txt for details.
-#
+"""
+A plugin to navigate HDUs in a FITS file or planes in a 3D cube or
+higher dimension dataset.
+
+**Plugin Type: Local**
+
+``MultiDim`` is a local plugin, which means it is associated with a
+channel.  An instance can be opened for each channel.
+
+**Usage**
+
+``MultiDim`` is a plugin designed to handle data cubes and multi-HDU FITS
+files. If you have opened such an image in Ginga, starting this plugin
+will enable you to browse to other slices of the cube or view other
+HDUs.
+
+For a data cube, you can save a slice as an image using the "Save Slice"
+button or create a movie using the "Save Movie" button by entering the
+"Start" and "End" slice indices. This feature requires ``mencoder`` to be
+installed.
+
+For a FITS table, its data are read in using Astropy table.
+Column units are displayed right under the main header ("None" if no unit).
+For masked columns, masked values are replaced with pre-defined fill values.
+
+**Browsing HDUs**
+
+Use the HDU drop down list in the upper part of the UI to browse and
+select an HDU to open in the channel.
+
+**Navigating Cubes**
+
+Use the controls in the lower part of the UI to select the axis and
+to step through the planes in that axis.
+
+**User Configuration**
+
+"""
 import time
 import re
+import os
 from distutils import spawn
 from contextlib import contextmanager
 
 from ginga.gw import Widgets
-from ginga.misc import Future, Bunch
+from ginga.misc import Future
 from ginga import GingaPlugin
 from ginga.util.iohelper import get_hdu_suffix
 from ginga.util.videosink import VideoSink
 
 import numpy as np
-import matplotlib.pyplot as plt
 
 have_mencoder = False
 if spawn.find_executable("mencoder"):
     have_mencoder = True
 
-have_pyfits = False
-try:
-    from astropy.io import fits as pyfits
-    have_pyfits = True
-except ImportError:
-    try:
-        import pyfits
-        have_pyfits = True
-    except ImportError:
-        pass
+__all__ = ['MultiDim']
 
 
 class MultiDim(GingaPlugin.LocalPlugin):
@@ -40,19 +65,19 @@ class MultiDim(GingaPlugin.LocalPlugin):
         # superclass defines some variables for us, like logger
         super(MultiDim, self).__init__(fv, fitsimage)
 
-        self.hdu_info = []
-        self.hdu_db = {}
         self.curhdu = 0
         self.naxispath = []
         self.name_pfx = 'NONAME'
-        self.image = None
+        self.img_path = None
+        self.img_name = None
+        self.file_obj = None
         self.orientation = 'vertical'
 
         # For animation feature
         self.play_image = None
         self.play_axis = 2
-        self.play_idx = 1
-        self.play_max = 1
+        self.play_idx = 0
+        self.play_max = 0
         self.play_int_sec = 0.1
         self.play_min_sec = 1.0 / 30
         self.play_last_time = 0.0
@@ -62,16 +87,13 @@ class MultiDim(GingaPlugin.LocalPlugin):
 
         # Load plugin preferences
         prefs = self.fv.get_preferences()
-        self.settings = prefs.createCategory('plugin_MultiDim')
-        self.settings.setDefaults(auto_start_naxis=False)
+        self.settings = prefs.create_category('plugin_MultiDim')
+        self.settings.set_defaults(auto_start_naxis=False)
         self.settings.load(onError='silent')
 
         self.gui_up = False
 
     def build_gui(self, container):
-        if not have_pyfits:
-            raise Exception("Please install astropy/pyfits to use this plugin")
-
         top = Widgets.VBox()
         top.set_border_width(4)
 
@@ -81,15 +103,6 @@ class MultiDim(GingaPlugin.LocalPlugin):
         vbox.set_border_width(4)
         vbox.set_spacing(2)
 
-        self.msg_font = self.fv.get_font("sansFont", 12)
-        tw = Widgets.TextArea(wrap=True, editable=False)
-        tw.set_font(self.msg_font)
-        self.tw = tw
-
-        fr = Widgets.Expander("Instructions")
-        fr.set_widget(tw)
-        vbox.add_widget(fr, stretch=0)
-
         fr = Widgets.Frame("HDU")
 
         vb1 = Widgets.VBox()
@@ -116,36 +129,38 @@ class MultiDim(GingaPlugin.LocalPlugin):
         self.naxisfr = fr
         vbox.add_widget(fr, stretch=0)
 
-        captions = [("First", 'button', "Prev", 'button', "Stop", 'button'),
-                    ("Last", 'button', "Next", 'button', "Play", 'button'),
-                    ("Interval:", 'label', "Interval", 'spinfloat',
+        tbar = Widgets.Toolbar(orientation='horizontal')
+        for name, actn, cb in (
+                ('first', 'first', lambda w: self.first_slice()),
+                ('last', 'last', lambda w: self.last_slice()),
+                ('reverse', 'prev', lambda w: self.prev_slice()),
+                ('forward', 'next', lambda w: self.next_slice()),
+                ('play', 'play', lambda w: self.play_start()),
+                ('stop', 'stop', lambda w: self.play_stop()), ):
+            iconpath = os.path.join(self.fv.iconpath, "%s_48.png" % name)
+            btn = tbar.add_action(None, iconpath=iconpath)
+            self.w[actn] = btn
+            btn.set_enabled(False)
+            btn.set_tooltip(actn)
+            btn.add_callback('activated', cb)
+        vbox.add_widget(tbar, stretch=0)
+
+        captions = [("Interval:", 'label', "Interval", 'spinfloat',
                      "fps", 'llabel'),
                     ]
         w, b = Widgets.build_info(captions, orientation=orientation)
         self.w.update(b)
-        b.next.add_callback('activated', lambda w: self.next_slice())
-        b.prev.add_callback('activated', lambda w: self.prev_slice())
-        b.first.add_callback('activated', lambda w: self.first_slice())
-        b.last.add_callback('activated', lambda w: self.last_slice())
-        b.play.add_callback('activated', lambda w: self.play_start())
-        b.stop.add_callback('activated', lambda w: self.play_stop())
         lower, upper = self.play_min_sec, 8.0
         b.interval.set_limits(lower, upper, incr_value=0.01)
         b.interval.set_value(self.play_int_sec)
         b.interval.set_decimals(2)
         b.interval.add_callback('value-changed', self.play_int_cb)
 
-        b.next.set_enabled(False)
-        b.prev.set_enabled(False)
-        b.first.set_enabled(False)
-        b.last.set_enabled(False)
-        b.play.set_enabled(False)
-        b.stop.set_enabled(False)
         b.interval.set_enabled(False)
         vbox.add_widget(w, stretch=0)
 
-        captions = [("Slice:", 'label', "Slice", 'llabel',),
-                     #"Value:", 'label', "Value", 'llabel'),
+        captions = [("Slice:", 'label', "Slice", 'llabel'),
+                    # ("Value:", 'label', "Value", 'llabel'),
                     ("Save Slice", 'button'),
                     ]
         w, b = Widgets.build_info(captions, orientation=orientation)
@@ -177,8 +192,8 @@ class MultiDim(GingaPlugin.LocalPlugin):
             fr.set_widget(infolbl)
         vbox.add_widget(fr, stretch=0)
 
-        #spacer = Widgets.Label('')
-        #vbox.add_widget(spacer, stretch=1)
+        # spacer = Widgets.Label('')
+        # vbox.add_widget(spacer, stretch=1)
 
         top.add_widget(sw, stretch=1)
 
@@ -188,6 +203,9 @@ class MultiDim(GingaPlugin.LocalPlugin):
         btn = Widgets.Button("Close")
         btn.add_callback('activated', lambda w: self.close())
         btns.add_widget(btn)
+        btn = Widgets.Button("Help")
+        btn.add_callback('activated', lambda w: self.help())
+        btns.add_widget(btn, stretch=0)
         btns.add_widget(Widgets.Label(''), stretch=1)
 
         top.add_widget(btns, stretch=0)
@@ -197,7 +215,7 @@ class MultiDim(GingaPlugin.LocalPlugin):
         self.gui_up = True
 
     def set_hdu_cb(self, w, val):
-        #idx = int(val)
+        # idx = int(val)
         idx = w.get_index()
         idx = max(0, idx)
         try:
@@ -205,47 +223,56 @@ class MultiDim(GingaPlugin.LocalPlugin):
 
         except Exception as e:
             self.logger.error("Error loading HDU #%d: %s" % (
-                idx+1, str(e)))
+                idx + 1, str(e)))
 
-    def set_naxis_cb(self, w, idx, n):
-        #idx = int(w.get_value()) - 1
-        self.set_naxis(idx, n)
+    def set_naxis_cb(self, widget, idx, n):
+        play_idx = int(idx) - 1
+        self.set_naxis(play_idx, n)
+
+    def build_naxis(self, dims, image):
+        self.naxispath = list(image.naxispath)
 
-    def build_naxis(self, dims):
         # build a vbox of NAXIS controls
         captions = [("NAXIS1:", 'label', 'NAXIS1', 'llabel'),
                     ("NAXIS2:", 'label', 'NAXIS2', 'llabel')]
 
-        self.naxispath = []
         for n in range(2, len(dims)):
-            self.naxispath.append(0)
-            key = 'naxis%d' % (n+1)
+            key = 'naxis%d' % (n + 1)
             title = key.upper()
             maxn = int(dims[n])
-            self.logger.debug("NAXIS%d=%d" % (n+1, maxn))
+            self.logger.debug("NAXIS%d=%d" % (n + 1, maxn))
             if maxn <= 1:
-                captions.append((title+':', 'label', title, 'llabel'))
+                captions.append((title + ':', 'label', title, 'llabel'))
             else:
-                captions.append((title+':', 'label', title, 'llabel',
-                                 #"Choose %s" % (title), 'spinbutton'))
+                captions.append((title + ':', 'label', title, 'llabel',
                                  "Choose %s" % (title), 'hscale'))
 
-        if len(dims) > 3:  # only add radiobuttons if we have more than 3 dim
-            radiobuttons = []
-            for i in range(2, len(dims)):
-                title = 'AXIS%d' % (i+1)
-                radiobuttons.extend((title, 'radiobutton'))
-            captions.append(radiobuttons)
-
         # Remove old naxis widgets
         for key in self.w:
             if key.startswith('choose_'):
                 self.w[key] = None
 
+        hbox = Widgets.HBox()
+
+        if len(dims) > 3:  # only add radiobuttons if we have more than 3 dim
+            group = None
+            for i in range(2, len(dims)):
+                title = 'AXIS%d' % (i + 1)
+                btn = Widgets.RadioButton(title, group=group)
+                if group is None:
+                    group = btn
+                hbox.add_widget(btn)
+                self.w[title.lower()] = btn
+
         w, b = Widgets.build_info(captions, orientation=self.orientation)
         self.w.update(b)
+
+        vbox = Widgets.VBox()
+        vbox.add_widget(w)
+        vbox.add_widget(hbox)
+
         for n in range(0, len(dims)):
-            key = 'naxis%d' % (n+1)
+            key = 'naxis%d' % (n + 1)
             lbl = b[key]
             maxn = int(dims[n])
             lbl.set_text("%d" % maxn)
@@ -255,14 +282,24 @@ class MultiDim(GingaPlugin.LocalPlugin):
                 lower = 1
                 upper = maxn
                 slider.set_limits(lower, upper, incr_value=1)
-                slider.set_value(lower)
+                text = self.naxispath[n - 2] + 1
+                if np.isscalar(text):
+                    slider.set_value(text)
+                else:
+                    slider.set_value(text[n - 2])
+
                 slider.set_tracking(True)
-                #slider.set_digits(0)
-                #slider.set_wrap(True)
+                # slider.set_digits(0)
+                # slider.set_wrap(True)
                 slider.add_callback('value-changed', self.set_naxis_cb, n)
 
+            # Disable playback if there is only 1 slice in the higher dimension
+            if n > 2 and dims[n] == 1:
+                radiobutton = self.w['axis%d' % (n + 1)]
+                radiobutton.set_enabled(False)
+
         # Add vbox of naxis controls to gui
-        self.naxisfr.set_widget(w)
+        self.naxisfr.set_widget(vbox)
 
         # for storing play_idx for each dim of image. used for going back to
         # the idx where you left off.
@@ -276,16 +313,19 @@ class MultiDim(GingaPlugin.LocalPlugin):
                 # widget callable needs (widget, value) args
                 def play_axis_change():
 
-                    self.play_indices[self.play_axis - 2] = (self.play_idx - 1) % dims[self.play_axis]  # noqa
+                    self.play_indices[self.play_axis - 2] = self.play_idx % dims[self.play_axis]  # noqa
 
                     self.play_axis = n
                     self.logger.debug("play_axis changed to %d" % n)
 
                     if self.play_axis < len(dims):
-                        self.play_max = dims[self.play_axis]
+                        self.play_max = dims[self.play_axis] - 1
 
                     self.play_idx = self.play_indices[n - 2]
 
+                    self.fv.gui_do(self.set_naxis, self.play_idx,
+                                   self.play_axis)
+
                 def check_if_we_need_change(w, v):
                     if self.play_axis is not n:
                         play_axis_change()
@@ -299,13 +339,21 @@ class MultiDim(GingaPlugin.LocalPlugin):
                 if n == 2:
                     self.w[key].set_state(True)
 
+        is_dc = len(dims) > 2
         self.play_axis = 2
         if self.play_axis < len(dims):
-            self.play_max = dims[self.play_axis]
-        self.play_idx = 1
+            self.play_max = dims[self.play_axis] - 1
+        if is_dc:
+            self.play_idx = self.naxispath[self.play_axis - 2]
+        else:
+            self.play_idx = 0
+        if self.play_indices:
+            text = [i + 1 for i in self.naxispath]
+        else:
+            text = self.play_idx + 1
+        self.w.slice.set_text(str(text))
 
         # Enable or disable NAXIS animation controls
-        is_dc = len(dims) > 2
         self.w.next.set_enabled(is_dc)
         self.w.prev.set_enabled(is_dc)
         self.w.first.set_enabled(is_dc)
@@ -322,11 +370,7 @@ class MultiDim(GingaPlugin.LocalPlugin):
         self.fv.stop_local_plugin(self.chname, str(self))
         return True
 
-    def instructions(self):
-        self.tw.set_text("""Use mouse wheel to choose HDU or axis of data cube (NAXIS controls).""")  # noqa
-
     def start(self):
-        self.instructions()
         self.resume()
 
     def pause(self):
@@ -340,19 +384,21 @@ class MultiDim(GingaPlugin.LocalPlugin):
         self.gui_up = False
         self.play_stop()
         try:
-            self.fits_f.close()
-        except:
+            self.file_obj.close()
+        except Exception:
             pass
-        self.image = None
+        self.file_obj = None
+        self.img_path = None
+        self.img_name = None
         self.fv.show_status("")
 
     def set_hdu(self, idx):
         self.logger.debug("Loading fits hdu #%d" % (idx))
 
         # determine canonical index of this HDU
-        info = self.hdu_info[idx]
+        info = self.file_obj.hdu_info[idx]
         aidx = (info.name, info.extver)
-        if aidx not in self.fits_f:
+        if aidx not in self.file_obj.hdu_db:
             aidx = idx
         sfx = get_hdu_suffix(aidx)
 
@@ -362,67 +408,44 @@ class MultiDim(GingaPlugin.LocalPlugin):
         chinfo = self.channel
         if imname in chinfo.datasrc:
             self.curhdu = idx
-            self.image = chinfo.datasrc[imname]
+            image = chinfo.datasrc[imname]
             self.fv.switch_name(chname, imname)
 
-            # Still need to build datacube profile
-            hdu = self.fits_f[idx]
-            if hdu.data is not None:
-                dims = list(hdu.data.shape)
-                dims.reverse()
-                self.build_naxis(dims)
-                return
+            return
 
         # Nope, we'll have to load it
         self.logger.debug("HDU %d not in memory; refreshing from file" % (idx))
         try:
             self.curhdu = idx
-            dims = [0, 0]
-            hdu = self.fits_f[idx]
+            info = self.file_obj.hdu_info[idx]
 
-            image = self.fv.fits_opener.load_hdu(hdu, fobj=self.fits_f)
-            self.image = image
-
-            if hdu.data is None:
-                # <- empty data part to this HDU
-                self.logger.warning("Empty data part in HDU #%d" % (idx))
-
-            elif info['htype'].lower() in ('bintablehdu', 'tablehdu',):
-                dims = [0, 0]
-
-            elif info['htype'].lower() not in ('imagehdu', 'primaryhdu'):
-                self.logger.warning("HDU #%d is not an image" % (idx))
-
-            else:
-                dims = list(hdu.data.shape)
-                dims.reverse()
-
-            image.load_hdu(hdu, fobj=self.fits_f)
+            image = self.file_obj.get_hdu(idx)
 
             # create a future for reconstituting this HDU
             future = Future.Future()
-            future.freeze(self.fv.load_image, self.path, idx=aidx)
-            image.set(path=self.path, idx=aidx, name=imname,
+            future.freeze(self.fv.load_image, self.img_path, idx=aidx)
+            image.set(path=self.img_path, idx=aidx, name=imname,
                       image_future=future)
 
             self.fv.add_image(imname, image, chname=chname)
 
-            self.build_naxis(dims)
             self.logger.debug("HDU #%d loaded." % (idx))
 
         except Exception as e:
             errmsg = "Error loading FITS HDU #%d: %s" % (
                 idx, str(e))
             self.logger.error(errmsg)
-            self.fv.show_error(errmsg, raisetab=False)
+            self.fv.show_error(errmsg, raisetab=True)
 
     def set_naxis(self, idx, n):
+        """Change the slice shown in the channel viewer.
+        `idx` is the slice index (0-based); `n` is the axis (0-based)
+        """
         self.play_idx = idx
-        self.w['choose_naxis%d' % (n+1)].set_value(idx)
-        idx = idx - 1
-        self.logger.debug("naxis %d index is %d" % (n+1, idx+1))
+        self.logger.debug("naxis %d index is %d" % (n + 1, idx + 1))
 
         image = self.fitsimage.get_image()
+        slidername = 'choose_naxis%d' % (n + 1)
         try:
             if image is None:
                 raise ValueError("Please load an image cube")
@@ -433,18 +456,26 @@ class MultiDim(GingaPlugin.LocalPlugin):
             self.logger.debug("m=%d naxispath=%s" % (m, str(self.naxispath)))
 
             image.set_naxispath(self.naxispath)
-            self.logger.debug("NAXIS%d slice %d loaded." % (n+1, idx+1))
+            self.logger.debug("NAXIS%d slice %d loaded." % (n + 1, idx + 1))
 
             if self.play_indices:
-                text = self.play_indices
-                text[m] = idx
+                # save play index
+                self.play_indices[m] = idx
+                text = [i + 1 for i in self.naxispath]
+                if slidername in self.w:
+                    self.w[slidername].set_value(text[m])
             else:
-                text = idx
+                text = idx + 1
+                if slidername in self.w:
+                    self.w[slidername].set_value(text)
             self.w.slice.set_text(str(text))
 
+            # schedule a redraw
+            self.fitsimage.redraw(whence=0)
+
         except Exception as e:
             errmsg = "Error loading NAXIS%d slice %d: %s" % (
-                n+1, idx+1, str(e))
+                n + 1, idx + 1, str(e))
             self.logger.error(errmsg)
             self.fv.error(errmsg)
 
@@ -485,9 +516,6 @@ class MultiDim(GingaPlugin.LocalPlugin):
 
         self.next_slice()
 
-        # schedule a redraw
-        self.fitsimage.redraw(whence=0)
-
         # set timer for next turnaround
         delta = deadline - time.time()
         timer.set(max(delta, 0.001))
@@ -499,70 +527,57 @@ class MultiDim(GingaPlugin.LocalPlugin):
             self.play_image = None
 
     def first_slice(self):
-        play_idx = 1
-        self.fv.gui_do(self.set_naxis_cb, None, play_idx, self.play_axis)
+        play_idx = 0
+        self.fv.gui_do(self.set_naxis, play_idx, self.play_axis)
 
     def last_slice(self):
         play_idx = self.play_max
-        self.fv.gui_do(self.set_naxis_cb, None, play_idx, self.play_axis)
+        self.fv.gui_do(self.set_naxis, play_idx, self.play_axis)
 
     def prev_slice(self):
-        play_idx = self.play_idx - 1
-        if play_idx < 1:
+        if np.isscalar(self.play_idx):
+            play_idx = self.play_idx - 1
+        else:
+            m = self.play_axis - 2
+            play_idx = self.play_idx[m] - 1
+        if play_idx < 0:
             play_idx = self.play_max
-        self.fv.gui_do(self.set_naxis_cb, None, play_idx, self.play_axis)
+        self.fv.gui_do(self.set_naxis, play_idx, self.play_axis)
 
     def next_slice(self):
-        play_idx = self.play_idx + 1
+        if np.isscalar(self.play_idx):
+            play_idx = self.play_idx + 1
+        else:
+            m = self.play_axis - 2
+            play_idx = self.play_idx[m] + 1
         if play_idx > self.play_max:
-            play_idx = 1
-        self.fv.gui_do(self.set_naxis_cb, None, play_idx, self.play_axis)
+            play_idx = 0
+        self.fv.gui_do(self.set_naxis, play_idx, self.play_axis)
 
     def play_int_cb(self, w, val):
         # force at least play_min_sec, otherwise playback is untenable
         self.play_int_sec = max(self.play_min_sec, val)
 
-    def prep_hdu_menu(self, w, info):
+    def prep_hdu_menu(self, w, hdu_info):
         # clear old TOC
         w.clear()
-        self.hdu_info = []
-        self.hdu_db = {}
 
-        idx = 0
-        extver_db = {}
-        for tup in info:
-            name = tup[1]
-            # figure out the EXTVER for this HDU
-            extver = extver_db.setdefault(name, 0)
-            extver += 1
-            extver_db[name] = extver
-
-            # prepare a record of pertinent info about the HDU for
-            # lookups by numerical index or (NAME, EXTVER)
-            d = Bunch.Bunch(index=idx, name=name, extver=extver,
-                            htype=tup[2], dtype=tup[5])
-            self.hdu_info.append(d)
-            # different ways of accessing this HDU:
-            # by numerical index
-            self.hdu_db[idx] = d
-            # by (hduname, extver)
-            self.hdu_db[(name, extver)] = d
+        for idx, d in enumerate(hdu_info):
 
             toc_ent = "%(index)4d %(name)-12.12s (%(extver)3d) %(htype)-12.12s %(dtype)-8.8s" % d  # noqa
             w.append_text(toc_ent)
-            idx += 1
 
         idx = w.get_index()
         if idx < 0:
             idx = 0
-        if idx >= len(self.hdu_info):
-            idx = len(self.hdu_info) - 1
-        #w.set_index(idx)
+        if idx >= len(hdu_info):
+            idx = len(hdu_info) - 1
+        # w.set_index(idx)
 
     def redo(self):
         """Called when an image is set in the channel."""
         image = self.channel.get_current_image()
-        if (image is None) or (image == self.image):
+        if image is None:
             return True
 
         path = image.get('path', None)
@@ -571,7 +586,21 @@ class MultiDim(GingaPlugin.LocalPlugin):
                 "Cannot open image: no value for metadata key 'path'")
             return
 
-        self.path = path
+        if path != self.img_path:
+            # <-- New file is being looked at
+            self.img_path = path
+
+            self.file_obj = self.fv.fits_opener.get_factory()
+            # TODO: specify 'readonly' somehow?
+            self.file_obj.open_file(path)
+
+            upper = len(self.file_obj) - 1
+            self.prep_hdu_menu(self.w.hdu, self.file_obj.hdu_info)
+            self.num_hdu = upper
+            self.logger.debug("there are %d hdus" % (upper + 1))
+            self.w.numhdu.set_text("%d" % (upper + 1))
+
+            self.w.hdu.set_enabled(len(self.file_obj) > 0)
 
         name = image.get('name', self.fv.name_image_from_path(path))
         idx = image.get('idx', None)
@@ -581,30 +610,49 @@ class MultiDim(GingaPlugin.LocalPlugin):
             name = match.group(1)
         self.name_pfx = name
 
-        self.fits_f = pyfits.open(path, 'readonly')
-
-        # lower = 0
-        upper = len(self.fits_f) - 1
-        info = self.fits_f.info(output=False)
-        self.prep_hdu_menu(self.w.hdu, info)
-        self.num_hdu = upper
-        self.logger.debug("there are %d hdus" % (upper+1))
-        self.w.numhdu.set_text("%d" % (upper+1))
-
         if idx is not None:
             # set the HDU in the drop down if known
-            info = self.hdu_db.get(idx, None)
+            info = self.file_obj.hdu_db.get(idx, None)
             if info is not None:
                 index = info.index
                 self.w.hdu.set_index(index)
-                self.set_hdu(index)
 
-        self.w.hdu.set_enabled(len(self.fits_f) > 0)
+        # rebuild the NAXIS controls, if necessary
+        # No two images in the same channel can have the same name.
+        # Here we keep track of the name to decide if we need to rebuild
+        if self.img_name != name:
+            self.img_name = name
+            dims = [0, 0]
+            data = image.get_data()
+            if data is None:
+                # <- empty data part to this HDU
+                self.logger.warning("Empty data part in HDU #%d" % (idx))
+
+            elif info.htype.lower() in ('bintablehdu', 'tablehdu',):
+                pass
+
+            elif info.htype.lower() not in ('imagehdu', 'primaryhdu'):
+                self.logger.warning("HDU #%d is not an image" % (idx))
+
+            else:
+                mddata = image.get_mddata()
+                if mddata is not None:
+                    dims = list(mddata.shape)
+                    dims.reverse()
+
+            self.build_naxis(dims, image)
 
     def save_slice_cb(self):
-        target = Widgets.SaveDialog(title='Save slice',
-                                    selectedfilter='*.png').get_path()
-        with open(target, 'w') as target_file:
+        import matplotlib.pyplot as plt
+
+        w = Widgets.SaveDialog(title='Save slice',
+                               selectedfilter='*.png')
+        target = w.get_path()
+        if target is None:
+            # save canceled
+            return
+
+        with open(target, 'wb') as target_file:
             hival = self.fitsimage.get_cut_levels()[1]
             image = self.fitsimage.get_image()
             curr_slice_data = image.get_data()
@@ -618,19 +666,19 @@ class MultiDim(GingaPlugin.LocalPlugin):
         end = int(self.w.end_slice.get_text())
         if not start or not end:
             return
-        elif start < 0 or end > self.play_max:
+        elif start < 1 or end > (self.play_max + 1):
             self.fv.show_status("Wrong slice index")
             return
         elif start > end:
             self.fv.show_status("Wrong slice order")
             return
 
-        if start == 1:
-            start = 0
+        start, end = start - 1, end - 1
 
-        target = Widgets.SaveDialog(title='Save Movie',
-                                    selectedfilter='*.avi').get_path()
-        if target:
+        w = Widgets.SaveDialog(title='Save Movie',
+                               selectedfilter='*.avi')
+        target = w.get_path()
+        if target is not None:
             self.save_movie(start, end, target)
 
     def save_movie(self, start, end, target_file):
@@ -662,8 +710,9 @@ class MultiDim(GingaPlugin.LocalPlugin):
         return 'multidim'
 
 
-# Replace module docstring with config doc for auto insert by Sphinx.
-# In the future, if we need the real docstring, we can append instead of
-# overwrite.
+# Append module docstring with config doc for auto insert by Sphinx.
 from ginga.util.toolbox import generate_cfg_example  # noqa
-__doc__ = generate_cfg_example('plugin_MultiDim', package='ginga')
+if __doc__ is not None:
+    __doc__ += generate_cfg_example('plugin_MultiDim', package='ginga')
+
+# END
