@@ -1,20 +1,30 @@
-#
-# Operations.py -- Operations management plugin for Ginga viewer
-#
 # This is open-source software licensed under a BSD license.
 # Please see the file LICENSE.txt for details.
-#
+"""
+This plugin defines the GUI for managing local plugins, a.k.a., "operations".
+
+**Plugin Type: Global**
+
+``Operations`` is a global plugin.  Only one instance can be opened.
+
+**Usage**
+
+The ``Operations`` plugin acts as a visual interface to the reference viewer
+plugin manager.  With this plugin, you can change the active channel,
+start, stop, or unfocus a local plugin on a channel, and see which local
+plugins are running.
+
+.. note:: By replacing or subclassing this plugin, you can customize the way
+          the reference viewer starts and manages operations.
+
+"""
 from ginga import GingaPlugin
-from ginga.misc import Bunch
 from ginga.gw import Widgets
 
+__all__ = ['Operations']
+
 
 class Operations(GingaPlugin.GlobalPlugin):
-    """
-    This plugin defines the GUI for managing local plugins, AKA "operations".
-    By replacing or subclassing this plugin you can customize the way
-    the reference viewer starts and manages operations.
-    """
 
     def __init__(self, fv):
         # superclass defines some variables for us, like logger
@@ -30,7 +40,6 @@ class Operations(GingaPlugin.GlobalPlugin):
         fv.add_callback('add-channel', self.add_channel_cb)
         fv.add_callback('delete-channel', self.delete_channel_cb)
         fv.add_callback('channel-change', self.change_channel_cb)
-        fv.add_callback('add-operation', self.add_operation_cb)
 
         # Add in global plugin manager
         pl_mgr = self.fv.gpmon
@@ -41,6 +50,7 @@ class Operations(GingaPlugin.GlobalPlugin):
 
         self.focuscolor = self.settings.get('focuscolor', "lightgreen")
         self.use_popup = True
+        self._start_op_args = ()
         self.spacer = None
         self.gui_up = False
 
@@ -58,18 +68,18 @@ class Operations(GingaPlugin.GlobalPlugin):
             hbox.add_widget(cbox1, stretch=0)
 
         self.use_popup = self.settings.get('use_popup_menu', True)
-        if self.use_popup:
-            opmenu = Widgets.Menu()
-            btn = Widgets.Button("Operation")
-        else:
-            opmenu = Widgets.ComboBox()
-            opmenu.set_tooltip("Select an operation")
-            hbox.add_widget(opmenu, stretch=0)
+        opmenu = Widgets.Menu()
+        btn = Widgets.Button("Operation")
+        btn.set_tooltip("Invoke operation")
+        btn.add_callback('activated', self.invoke_popup_cb)
+        if not self.use_popup:
+            hbox.add_widget(btn, stretch=0)
+            self.w.opname = Widgets.Label('')
+            hbox.add_widget(self.w.opname, stretch=0)
             btn = Widgets.Button("Go")
+            btn.add_callback('activated', self.invoke_op_cb)
 
         self.w.operation = opmenu
-        btn.add_callback('activated', self.invoke_op_cb)
-        btn.set_tooltip("Invoke operation")
         self.w.opbtn = btn
         hbox.add_widget(btn, stretch=0)
 
@@ -85,7 +95,6 @@ class Operations(GingaPlugin.GlobalPlugin):
         container.add_widget(hbox, stretch=0)
         self.gui_up = True
 
-
     def add_channel_cb(self, viewer, channel):
         chname = channel.name
         self.w.channel.insert_alpha(chname)
@@ -110,41 +119,48 @@ class Operations(GingaPlugin.GlobalPlugin):
             channel = self.fv.get_channel(name)
             self.add_channel_cb(self.fv, channel)
 
-        # TODO: get the list of plugins and populate our operation control
-        ## operations = self.fv.get_operations()
-        ## for opname in operations:
-        ##     self.add_operation_cb(self.fv, opname, optype, spec)
+        plugins = self.fv.get_plugins()
+        for spec in plugins:
+            if spec.get('hidden', False):
+                continue
+            self.fv.error_wrap(self.add_operation, self.fv, spec)
 
-    def add_operation_cb(self, viewer, opname, optype, spec):
+    def add_operation(self, viewer, spec):
         if not self.gui_up:
             return
-
+        opname = spec.get('name', spec.get('klass', spec.get('module')))
+        ptype = spec.get('ptype', 'local')
         category = spec.get('category', None)
-        menuname = spec.get('menu', opname)
+        categories = None
+        if category is not None:
+            categories = category.split('.')
+        menuname = spec.get('menu', spec.get('tab', opname))
 
         opmenu = self.w.operation
-        if self.use_popup:
-            menu = opmenu
-            if category is not None:
-                categories = category.split('.')
-                for catname in categories:
-                    try:
-                        menu = menu.get_menu(catname)
-                    except KeyError:
-                        menu = menu.add_menu(catname)
+        menu = opmenu
+        if categories is not None:
+            for catname in categories:
+                try:
+                    menu = menu.get_menu(catname)
+                except KeyError:
+                    menu = menu.add_menu(catname)
 
-            item = menu.add_name(menuname)
+        item = menu.add_name(menuname)
+        if self.use_popup:
             item.add_callback('activated',
                               lambda *args: self.start_operation_cb(opname,
-                                                                    optype,
+                                                                    ptype,
                                                                     spec))
         else:
-            # TODO: use menuname
-            opmenu.insert_alpha(opname)
+            item.add_callback('activated',
+                              lambda *args: self.set_operation_cb(menuname,
+                                                                  opname,
+                                                                  ptype,
+                                                                  spec))
 
-    def start_operation_cb(self, name, optype, spec):
+    def start_operation_cb(self, name, ptype, spec):
         self.logger.debug("invoking operation menu")
-        if optype == 'global':
+        if ptype == 'global':
             # global plugin
             self.fv.error_wrap(self.fv.start_global_plugin, name,
                                raise_tab=True)
@@ -154,6 +170,10 @@ class Operations(GingaPlugin.GlobalPlugin):
         chname = str(self.w.channel.get_alpha(idx))
         self.fv.error_wrap(self.fv.start_local_plugin, chname, name, None)
 
+    def set_operation_cb(self, menuname, name, ptype, spec):
+        self._start_op_args = (name, ptype, spec)
+        self.w.opname.set_text(menuname)
+
     def channel_select_cb(self, widget, index):
         if index >= 0:
             chname = self.fv.get_channel_names()[index]
@@ -165,18 +185,23 @@ class Operations(GingaPlugin.GlobalPlugin):
         # Update the channel control
         self.w.channel.show_text(channel.name)
 
-    def invoke_op_cb(self, btn_w):
+    def invoke_popup_cb(self, btn_w):
         self.logger.debug("invoking operation menu")
         menu = self.w.operation
-        if self.use_popup:
-            menu.popup(btn_w)
-        else:
-            idx = menu.get_index()
-            opname = str(menu.get_alpha(idx))
-            self.start_operation_cb(opname)
+        menu.popup(btn_w)
+
+    def invoke_op_cb(self, btn_w):
+        args = self._start_op_args
+        if len(args) == 0:
+            return
+        self.start_operation_cb(*args)
 
     def activate_plugin_cb(self, pl_mgr, bnch):
-        hidden = bnch.pInfo.spec.get('hidden', False)
+        spec = bnch.pInfo.spec
+        optray = spec.get('optray', True)
+        if not optray:
+            return
+        hidden = spec.get('hidden', False)
         if hidden:
             return
 
@@ -235,4 +260,10 @@ class Operations(GingaPlugin.GlobalPlugin):
     def __str__(self):
         return 'operations'
 
-#END
+
+# Append module docstring with config doc for auto insert by Sphinx.
+from ginga.util.toolbox import generate_cfg_example  # noqa
+if __doc__ is not None:
+    __doc__ += generate_cfg_example('plugin_Operations', package='ginga')
+
+# END
