@@ -1,48 +1,48 @@
-#
-# Pan.py -- Pan plugin for fits viewer
-#
 # This is open-source software licensed under a BSD license.
 # Please see the file LICENSE.txt for details.
-#
+"""
+The ``Pan`` plugin provides a small panning image that gives an overall
+"birds-eye" view of the channel image that last had the focus.  If the
+channel image is zoomed in 2X or greater, then the pan region is shown
+graphically in the ``Pan`` image by a rectangle.
+
+**Plugin Type: Global**
+
+``Pan`` is a global plugin.  Only one instance can be opened.
+
+**Usage**
+
+The channel image can be panned by clicking and/or dragging to place
+the rectangle.  Using the right mouse button to drag a rectangle will
+force the channel image viewer to try to match the region (taking into
+account the differences in the aspect ratio between the drawn rectangle
+and the window dimensions).  Scrolling in the ``Pan`` image will zoom the
+channel image.
+
+The color/intensity map and cut levels of the ``Pan`` image are updated
+when they are changed in the corresponding channel image.
+The ``Pan`` image also displays the World Coordinate System (WCS) compass, if
+valid WCS metadata is present in the FITS HDU being viewed in the
+channel.
+
+The ``Pan`` plugin usually appears as a sub-pane under the "Info" tab, next
+to the ``Info`` plugin.
+
+"""
 import sys
 import traceback
 import math
+
 from ginga.BaseImage import BaseImage
 from ginga.gw import Widgets, Viewers
 from ginga.misc import Bunch
 from ginga import GingaPlugin
 
+__all__ = ['Pan']
+
+
 class Pan(GingaPlugin.GlobalPlugin):
-    """
-    Pan
-    ===
-    The Pan plugin provides a small panning image that gives an overall
-    "birds-eye" view of the channel image that last had the focus.  If the
-    channel image is zoomed in 2X or greater then the pan region is shown
-    graphically in the Pan image by a rectangle.
 
-    Plugin Type: Global
-    -------------------
-    Pan is a global plugin.  Only one instance can be opened.
-
-    Usage
-    -----
-    The channel image can be panned by clicking and/or dragging to place
-    the rectangle.  Using the right mouse button to drag a rectangle will
-    force the channel image viewer to try to match the region (taking into
-    account the differences in the aspect ratio between the drawn rectangle
-    and the window dimensions).  Scrolling in the Pan image will zoom the
-    channel image.
-
-    The color/intensity map and cut levels of the Pan image are updated
-    when they are changed in the corresponding channel image.
-    The Pan image also displays the World Coordinate System compass, if
-    valid WCS metadata is present in the FITS HDU being viewed in the
-    channel.
-
-    The Pan plugin usually appears as a sub-pane under the "Info" tab, next
-    to the Info plugin.
-    """
     def __init__(self, fv):
         # superclass defines some variables for us, like logger
         super(Pan, self).__init__(fv)
@@ -75,7 +75,7 @@ class Pan(GingaPlugin.GlobalPlugin):
         self.nb = nb
         container.add_widget(self.nb, stretch=1)
 
-    def _create_pan_image(self, fitsimage):
+    def _create_pan_viewer(self, fitsimage):
         pi = Viewers.CanvasView(logger=self.logger)
         pi.enable_autozoom('on')
         pi.enable_autocuts('off')
@@ -88,10 +88,10 @@ class Pan(GingaPlugin.GlobalPlugin):
         pi.set_callback('none-move', self.motion_cb)
         pi.set_callback('zoom-scroll', self.zoom_cb)
         pi.set_callback('zoom-pinch', self.zoom_pinch_cb)
+        pi.set_callback('pan-pan', self.pan_pan_cb)
         pi.set_callback('configure', self.reconfigure)
         # for debugging
         pi.set_name('panimage')
-        #pi.ui_set_active(True)
 
         my_canvas = pi.get_canvas()
         my_canvas.enable_draw(True)
@@ -111,13 +111,12 @@ class Pan(GingaPlugin.GlobalPlugin):
 
     def add_channel(self, viewer, channel):
         fitsimage = channel.fitsimage
-        panimage = self._create_pan_image(fitsimage)
-        chname = channel.name
+        panimage = self._create_pan_viewer(fitsimage)
 
         iw = Viewers.GingaViewerWidget(panimage)
         iw.resize(self._wd, self._ht)
         self.nb.add_widget(iw)
-        index = self.nb.index_of(iw)
+        #index = self.nb.index_of(iw)
         paninfo = Bunch.Bunch(panimage=panimage, widget=iw,
                               pancompass=None, panrect=None)
         channel.extdata._pan_info = paninfo
@@ -136,22 +135,21 @@ class Pan(GingaPlugin.GlobalPlugin):
         xfrmsettings = ['flip_x', 'flip_y', 'swap_xy']
         if self.settings.get('rotate_pan_image', False):
             xfrmsettings.append('rot_deg')
-        fitssettings.shareSettings(pansettings, xfrmsettings)
+        fitssettings.share_settings(pansettings, xfrmsettings)
         for key in xfrmsettings:
-            pansettings.get_setting(key).add_callback('set', self.settings_cb,
-                                                     fitsimage, channel, paninfo, 0)
+            pansettings.get_setting(key).add_callback(
+                'set', self.settings_cb, fitsimage, channel, paninfo, 0)
 
-
-        fitssettings.shareSettings(pansettings, ['cuts'])
-        pansettings.get_setting('cuts').add_callback('set', self.settings_cb,
-                                                    fitsimage, channel, paninfo, 1)
+        fitssettings.share_settings(pansettings, ['cuts'])
+        pansettings.get_setting('cuts').add_callback(
+            'set', self.settings_cb, fitsimage, channel, paninfo, 1)
 
         zoomsettings = ['zoom_algorithm', 'zoom_rate',
                         'scale_x_base', 'scale_y_base']
-        fitssettings.shareSettings(pansettings, zoomsettings)
+        fitssettings.share_settings(pansettings, zoomsettings)
         for key in zoomsettings:
-            pansettings.get_setting(key).add_callback('set', self.zoom_ext_cb,
-                                                     fitsimage, channel, paninfo)
+            pansettings.get_setting(key).add_callback(
+                'set', self.zoom_ext_cb, fitsimage, channel, paninfo)
 
         fitsimage.add_callback('redraw', self.redraw_cb, channel, paninfo)
 
@@ -178,15 +176,14 @@ class Pan(GingaPlugin.GlobalPlugin):
         panimage.redraw(whence=1)
 
     def redo(self, channel, image):
+        self.logger.debug("redo")
         paninfo = channel.extdata._pan_info
 
         if (image is None) or not isinstance(image, BaseImage):
+            self.logger.debug("no main image--clearing Pan viewer")
             paninfo.panimage.clear()
             return
 
-        loval, hival = channel.fitsimage.get_cut_levels()
-        paninfo.panimage.cut_levels(loval, hival)
-
         self.set_image(channel, paninfo, image)
 
     def blank(self, channel):
@@ -199,7 +196,7 @@ class Pan(GingaPlugin.GlobalPlugin):
         # If the active widget has changed, then raise our Info widget
         # that corresponds to it
         if self.active != chname:
-            if not channel.extdata.has_key('_pan_info'):
+            if '_pan_info' not in channel.extdata:
                 self.add_channel(viewer, channel)
             paninfo = channel.extdata._pan_info
             iw = paninfo.widget
@@ -208,6 +205,13 @@ class Pan(GingaPlugin.GlobalPlugin):
             self.active = chname
             self.info = paninfo
 
+        # TODO: this check should not be necessary.  But under some
+        # circumstances it seems to be needed.
+        image = channel.fitsimage.get_image()
+        p_image = self.info.panimage.get_image()
+        if image != p_image:
+            self.logger.debug("pan viewer seems to be missing image--calling redo()")
+            self.redo(channel, image)
 
     def reconfigure(self, panimage, width, height):
         self.logger.debug("new pan image dimensions are %dx%d" % (
@@ -217,12 +221,10 @@ class Pan(GingaPlugin.GlobalPlugin):
         return True
 
     def redraw_cb(self, fitsimage, channel, paninfo):
-        #paninfo.panimage.redraw(whence=whence)
         self.panset(channel.fitsimage, channel, paninfo)
         return True
 
     def settings_cb(self, setting, value, fitsimage, channel, paninfo, whence):
-        #paninfo.panimage.redraw(whence=whence)
         self.panset(channel.fitsimage, channel, paninfo)
         return True
 
@@ -240,10 +242,12 @@ class Pan(GingaPlugin.GlobalPlugin):
 
     def set_image(self, channel, paninfo, image):
         if (image is None) or not isinstance(image, BaseImage):
+            self.logger.debug("no main image--clearing Pan viewer")
             paninfo.panimage.clear()
             return
 
         if not self.use_shared_canvas:
+            self.logger.debug("setting Pan viewer image")
             paninfo.panimage.set_image(image)
         else:
             paninfo.panimage.zoom_fit()
@@ -267,7 +271,8 @@ class Pan(GingaPlugin.GlobalPlugin):
                 image.add_offset_xy(x, y, 1.0, 1.0)
 
                 paninfo.pancompass = p_canvas.add(self.dc.Compass(
-                    x, y, radius, color=self.settings.get('compass_color', 'skyblue'),
+                    x, y, radius,
+                    color=self.settings.get('compass_color', 'skyblue'),
                     fontsize=14))
 
             except Exception as e:
@@ -294,19 +299,17 @@ class Pan(GingaPlugin.GlobalPlugin):
         points = fitsimage.get_pan_rect()
 
         # calculate pan position point radius
-        p_image = paninfo.panimage.get_image()
+        p_image = paninfo.panimage.get_image()  # noqa
         try:
             obj = paninfo.panimage.canvas.get_object_by_tag('__image')
         except KeyError:
             obj = None
-        #print(('panset', image, p_image, obj, obj.image, paninfo.panimage._imgobj))
 
         width, height = image.get_size()
         edgew = math.sqrt(width**2 + height**2)
         radius = int(0.015 * edgew)
 
         # Mark pan rectangle and pan position
-        #p_canvas = paninfo.panimage.get_canvas()
         p_canvas = paninfo.panimage.get_private_canvas()
         try:
             obj = p_canvas.get_object_by_tag(paninfo.panrect)
@@ -321,12 +324,13 @@ class Pan(GingaPlugin.GlobalPlugin):
 
         except KeyError:
             paninfo.panrect = p_canvas.add(self.dc.CompoundObject(
-                self.dc.Point(x, y, radius=radius, style='plus',
-                              color=self.settings.get('pan_position_color', 'yellow')),
-                self.dc.Polygon(points,
-                                color=self.settings.get('pan_rectangle_color', 'red'))))
+                self.dc.Point(
+                    x, y, radius=radius, style='plus',
+                    color=self.settings.get('pan_position_color', 'yellow')),
+                self.dc.Polygon(
+                    points,
+                    color=self.settings.get('pan_rectangle_color', 'red'))))
 
-        #p_canvas.update_canvas(whence=0)
         paninfo.panimage.zoom_fit()
         return True
 
@@ -369,6 +373,17 @@ class Pan(GingaPlugin.GlobalPlugin):
 
         return False
 
+    def pan_pan_cb(self, fitsimage, event):
+        """Pan event in the pan window.  Just pan the channel viewer.
+        """
+        chviewer = self.fv.getfocus_viewer()
+        bd = chviewer.get_bindings()
+
+        if hasattr(bd, 'pa_pan'):
+            return bd.pa_pan(chviewer, event)
+
+        return False
+
     def draw_cb(self, canvas, tag):
         # Get and delete the drawn object
         obj = canvas.get_object_by_tag(tag)
@@ -394,7 +409,7 @@ class Pan(GingaPlugin.GlobalPlugin):
             self.logger.debug("wd_scale=%f ht_scale=%f scale=%f" % (
                 wd_scale, ht_scale, scale))
             if scale < 1.0:
-                zoomlevel = - max(2, int(math.ceil(1.0/scale)))
+                zoomlevel = - max(2, int(math.ceil(1.0 / scale)))
             else:
                 zoomlevel = max(1, int(math.floor(scale)))
             self.logger.debug("zoomlevel=%d" % (zoomlevel))
@@ -403,8 +418,13 @@ class Pan(GingaPlugin.GlobalPlugin):
 
         return True
 
-
     def __str__(self):
         return 'pan'
 
-#END
+
+# Append module docstring with config doc for auto insert by Sphinx.
+from ginga.util.toolbox import generate_cfg_example  # noqa
+if __doc__ is not None:
+    __doc__ += generate_cfg_example('plugin_Pan', package='ginga')
+
+# END
