@@ -42,6 +42,7 @@ class ModeIndicator(object):
         self.xpad = 8
         self.ypad = 4
         self.offset = 10
+        self.modetbl = dict(locked='L', softlock='S', held='H', oneshot='O')
 
         # for displaying modal keyboard state
         self.mode_obj = None
@@ -49,7 +50,6 @@ class ModeIndicator(object):
         bm.add_callback('mode-set', self.mode_change_cb)
         viewer.add_callback('configure', self._configure_cb)
 
-
     def mode_change_cb(self, bindmap, mode, modetype):
         # delete the old indicator
         obj = self.mode_obj
@@ -58,27 +58,26 @@ class ModeIndicator(object):
         if obj:
             try:
                 canvas.delete_object(obj)
-            except:
+            except Exception:
                 pass
 
         if not self.visible:
             return True
 
         # if not one of the standard modifiers, display the new one
-        if not mode in (None, 'ctrl', 'shift'):
+        if mode not in (None, 'ctrl', 'shift'):
             Text = canvas.get_draw_class('text')
             Polygon = canvas.get_draw_class('polygon')
             Compound = canvas.get_draw_class('compoundobject')
 
-            if modetype == 'locked':
-                text = '%s [L]' % (mode)
-            elif modetype == 'softlock':
-                text = '%s [SL]' % (mode)
-            else:
-                text = mode
+            text = mode
+            if modetype in self.modetbl:
+                text += ' [%s]' % self.modetbl[modetype]
+
+            color = 'cyan' if mode == 'meta' else 'yellow'
 
             o1 = Text(0, 0, text,
-                      fontsize=self.fontsize, color='yellow', coord='window')
+                      fontsize=self.fontsize, color=color, coord='window')
 
             txt_wd, txt_ht = self.viewer.renderer.get_dimensions(o1)
 
@@ -92,9 +91,9 @@ class ModeIndicator(object):
             cx1, cy1, cx2, cy2 = x_base, y_base, x_base + box_wd, y_base + box_ht
             poly_pts = ((cx1, cy1), (cx2, cy1), (cx2, cy2), (cx1, cy2))
             o2 = Compound(Polygon(poly_pts,
-                               color='black', coord='window',
-                               fill=True, fillcolor='black'),
-                               o1)
+                                  color='black', coord='window',
+                                  fill=True, fillcolor='black'),
+                          o1)
             self.mode_obj = o2
             canvas.add(o2)
 
