@@ -6,37 +6,33 @@
 # Please see the file LICENSE.txt for details.
 #
 from __future__ import print_function
-import threading
-import time
 
 from ginga import Mixins, Bindings
-from ginga.misc import log, Bunch
 from ginga.canvas.mixins import DrawingMixin, CanvasMixin, CompoundMixin
 from ginga.util.toolbox import ModeIndicator
-from ginga.web.pgw import PgHelp
 
 
 try:
     # See if we have aggdraw module--best choice
     from ginga.aggw.ImageViewAgg import ImageViewAgg as ImageView, \
-         ImageViewAggError as ImageViewError
+        ImageViewAggError as ImageViewError
 
 except ImportError:
     try:
         # No, hmm..ok, see if we have PIL module...
         from ginga.pilw.ImageViewPil import ImageViewPil as ImageView, \
-             ImageViewPilError as ImageViewError
+            ImageViewPilError as ImageViewError
 
     except ImportError:
         try:
             # No dice. How about the OpenCv module?
             from ginga.cvw.ImageViewCv import ImageViewCv as ImageView, \
-                 ImageViewCvError as ImageViewError
+                ImageViewCvError as ImageViewError
 
         except ImportError:
             # Fall back to mock--there will be no graphic overlays
             from ginga.mockw.ImageViewMock import ImageViewMock as ImageView, \
-                 ImageViewMockError as ImageViewError
+                ImageViewMockError as ImageViewError
 
 
 default_html_fmt = 'jpeg'
@@ -45,6 +41,7 @@ default_html_fmt = 'jpeg'
 class ImageViewPgError(ImageViewError):
     pass
 
+
 class ImageViewPg(ImageView):
 
     def __init__(self, logger=None, rgbmap=None, settings=None):
@@ -57,17 +54,22 @@ class ImageViewPg(ImageView):
         # format for rendering image on HTML5 canvas
         # NOTE: 'jpeg' has much better performance than 'png', but can show
         # some artifacts, especially noticeable with small text
-        self.t_.setDefaults(html5_canvas_format=default_html_fmt)
+        self.t_.set_defaults(html5_canvas_format=default_html_fmt)
 
-        #self.defer_redraw = False
+        # Format 'png' is ok with 'RGBA', but 'jpeg' only works with 'RGB'
+        self.rgb_order = 'RGB'
+        # this should already be so, but just in case...
+        self.defer_redraw = True
 
-
-    def set_widget(self, canvas):
+    def set_widget(self, canvas_w):
         """Call this method with the widget that will be used
         for the display.
         """
-        self.logger.debug("set widget canvas=%s" % canvas)
-        self.pgcanvas = canvas
+        self.logger.debug("set widget canvas_w=%s" % canvas_w)
+        self.pgcanvas = canvas_w
+
+        ## wd, ht = canvas_w.get_size()
+        ## self.configure_window(wd, ht)
 
     def get_widget(self):
         return self.pgcanvas
@@ -264,7 +266,7 @@ class ImageViewEvent(ImageViewPg):
             220: 'backslash',
             221: ']',
             222: 'singlequote',
-            }
+        }
 
         # this is an auxilliary table used to map shifted keys to names
         # see key_down_event() and key_up_event()
@@ -316,7 +318,7 @@ class ImageViewEvent(ImageViewPg):
             ('shift_l', ','): '<',
             ('shift_l', '.'): '>',
             ('shift_l', '/'): '?',
-            }
+        }
 
         # this table is used to map special characters to character names
         # see key_press_event()
@@ -326,7 +328,7 @@ class ImageViewEvent(ImageViewPg):
             "'": 'singlequote',
             "`": 'backquote',
             " ": 'space',
-            }
+        }
 
         # list of keys for which javascript will give us a keydown event,
         # but not a keypress event.  We use this list to synthesize one.
@@ -435,7 +437,8 @@ class ImageViewEvent(ImageViewPg):
         return self.make_ui_callback('key-release', keyname)
 
     def button_press_event(self, event):
-        x = event.x; y = event.y
+        x = event.x
+        y = event.y
         self.last_win_x, self.last_win_y = x, y
         button = 0
         button |= 0x1 << event.button
@@ -447,7 +450,8 @@ class ImageViewEvent(ImageViewPg):
 
     def button_release_event(self, event):
         # event.button, event.x, event.y
-        x = event.x; y = event.y
+        x = event.x
+        y = event.y
         self.last_win_x, self.last_win_y = x, y
         button = 0
         button |= 0x1 << event.button
@@ -472,11 +476,19 @@ class ImageViewEvent(ImageViewPg):
     def scroll_event(self, event):
         x, y = event.x, event.y
         delta = event.delta
+        dx, dy = event.dx, event.dy
         self.last_win_x, self.last_win_y = x, y
 
+        if (dx != 0 or dy != 0):
+            # <= This browser gives us deltas for x and y
+            # Synthesize this as a pan gesture event
+            self.make_ui_callback('pan', 'start', 0, 0)
+            self.make_ui_callback('pan', 'move', dx, dy)
+            return self.make_ui_callback('pan', 'stop', 0, 0)
+
         # 15 deg is standard 1-click turn for a wheel mouse
         # delta usually returns +/- 1.0
-        numDegrees = abs(delta) * 15.0
+        num_degrees = abs(delta) * 15.0
 
         direction = 0.0
         if delta > 0:
@@ -484,12 +496,12 @@ class ImageViewEvent(ImageViewPg):
         elif delta < 0:
             direction = 180.0
         self.logger.debug("scroll deg=%f direction=%f" % (
-            numDegrees, direction))
+            num_degrees, direction))
 
         data_x, data_y = self.check_cursor_location()
 
-        return self.make_ui_callback('scroll', direction, numDegrees,
-                                  data_x, data_y)
+        return self.make_ui_callback('scroll', direction, num_degrees,
+                                     data_x, data_y)
 
     def drop_event(self, event):
         data = event.delta
@@ -498,7 +510,6 @@ class ImageViewEvent(ImageViewPg):
         self.logger.debug("dropped text(s): %s" % (str(paths)))
         return self.make_ui_callback('drag-drop', paths)
 
-
     def pinch_event(self, event):
         self.logger.debug("pinch: event=%s" % (str(event)))
         state = 'move'
@@ -540,7 +551,7 @@ class ImageViewEvent(ImageViewPg):
 
     def swipe_event(self, event):
         if event.isfinal:
-            state = 'end'
+            state = 'end'  # noqa
             self.logger.debug("swipe gesture event=%s" % (str(event)))
             ## self.logger.debug("swipe gesture hdir=%s vdir=%s" % (
             ##     hdir, vdir))
@@ -548,9 +559,10 @@ class ImageViewEvent(ImageViewPg):
 
     def tap_event(self, event):
         if event.isfinal:
-            state = 'end'
+            state = 'end'  # noqa
             self.logger.debug("tap gesture event=%s" % (str(event)))
 
+
 class ImageViewZoom(Mixins.UIMixin, ImageViewEvent):
 
     # class variables for binding map and bindings can be set
@@ -592,6 +604,14 @@ class ImageViewZoom(Mixins.UIMixin, ImageViewEvent):
         self.bindings = bindings
         bindings.set_bindings(self)
 
+    def center_cursor(self):
+        # NOP
+        pass
+
+    def position_cursor(self, data_x, data_y):
+        # NOP
+        pass
+
 
 class CanvasView(ImageViewZoom):
 
